/*-----------------------------------------------------------------------------------------------*\
 *
 *  $id$
 *
 *  $Log: bluetooth_hci.c $
 *  Revision 1.4  2004/03/24 09:18:40Z  ddorau
 *  Revision 1.3  2004/03/05 17:10:22Z  mbahr
 *  Testmodes
 *  Revision 1.2  2004/03/01 13:29:15Z  ddorau
 *  Typo, loop mode
 *  Revision 1.1  2004/02/24 15:52:35Z  mpommerenke
 *  Initial revision
 *
\*-----------------------------------------------------------------------------------------------*/
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/proc_fs.h>
#include <linux/sched.h>
#include <asm/uaccess.h>
#include <linux/config.h>
#include <linux/slab.h>
#include <asm/fcntl.h>
#include <asm/ioctl.h>
#include <linux/devfs_fs_kernel.h>
#include <linux/fs.h>
#include <asm/semaphore.h>
#include <asm/errno.h>
#include <linux/wait.h>
#include <linux/poll.h>
#include <asm/smplock.h>
/*--- #include <linux/bluetooth_hci.h> ---*/
#include "bluetooth_hci.h"
#include "hcitransfer.h"

/*-----------------------------------------------------------------------------------------------*\
\*-----------------------------------------------------------------------------------------------*/
MODULE_AUTHOR("Martin Pommerenke (AVM Berlin)  m.pommerenke@avm.de");
MODULE_DESCRIPTION("Bluetooth hci driver version 0.1");
#define MODULE_VERSION "0.1"

static unsigned int bluetooth_hci_write_window_size = 1;
static unsigned int bluetooth_hci_message_size = 512;
static int loop = 0;
static unsigned char* loop_buffer = NULL;
static int debug = 0;

MODULE_PARM(bluetooth_hci_message_size, "i");
MODULE_PARM(loop, "i");
MODULE_PARM(debug, "i");
MODULE_PARM_DESC(bluetooth_hci_message_size, "Bluetooth HCI message size");
MODULE_PARM_DESC(loop, "If set, all HCI data is looped back to hardware");
MODULE_PARM_DESC(debug, "enable debug output");

struct _bluetooth_hci {
    unsigned int major;
    unsigned int minor;
	devfs_handle_t devfs_handle;
};

struct _bluetooth_hci_open_data {
    unsigned int minor;
    unsigned int flags;
    unsigned char *rx_buffer;
    unsigned int rx_buffer_size;
    unsigned int read_length;
    unsigned char *tx_buffer;
    unsigned int tx_buffer_size;
    struct fown_struct *pf_owner;
    struct fasync_struct *fasync;
	wait_queue_head_t *wait_ind;
};

/*-----------------------------------------------------------------------------------------------*\
\*-----------------------------------------------------------------------------------------------*/
static int bluetooth_hci_open(struct inode *, struct file *);
static int bluetooth_hci_close(struct inode *, struct file *);
/*--- static int bluetooth_hci_ioctl(struct inode *, struct file *, unsigned int, unsigned long); ---*/
static int bluetooth_hci_fasync(int, struct file *, int);
static ssize_t bluetooth_hci_write(struct file *, const char *, size_t , loff_t *);
static ssize_t bluetooth_hci_read(struct file *, char *, size_t , loff_t *);
static void bluetooth_hci_cleanup(void);
static unsigned int bluetooth_hci_poll(struct file *filp, poll_table *wait);

static DECLARE_WAIT_QUEUE_HEAD(bluetooth_hci_default_wait_queue); /*--- wird durch kopieren zum init der filp->wait_ind verwendet ---*/

struct file_operations bluetooth_hci_fops = {
    owner:   THIS_MODULE,
    open:    bluetooth_hci_open,
    release: bluetooth_hci_close,
    read:    bluetooth_hci_read,
    write:   bluetooth_hci_write,
    /*--- ioctl:   bluetooth_hci_ioctl, ---*/
    fasync:  bluetooth_hci_fasync,
    poll:    bluetooth_hci_poll,
};


static struct _bluetooth_hci bluetooth_hci;
static struct semaphore bluetooth_hci_sema;
static struct semaphore bluetooth_hci_read_sema;
static unsigned int bluetooth_hci_read_count;
static struct semaphore bluetooth_hci_write_sema;
static unsigned int bluetooth_hci_write_count;
static wait_queue_head_t *bluetooth_hci_write_wait;
static wait_queue_head_t *bluetooth_hci_read_wait;

/*-------------------------------------------------------------------------------------*\
\*-------------------------------------------------------------------------------------*/
void HCI_ATM_DSP_Status(void) {
    union _hcimcbsp_status_flag DSP_Status;
    static union _hcimcbsp_status_flag DSP_OldStatus;

    HCI_ATM_GetDSPStatus((unsigned int *)&DSP_Status);

    if(DSP_OldStatus.Register >> 16  !=  DSP_Status.Register >> 16) { 
        DEB_ERR("hci: HCI_DSP-Status: RxCnt(%u) TxCnt(%u) McBSPFull(%u) McBSPRxSnycErr(%u) McBSPTxSnycErr(%u) RxDMAErr(%u) BT_CTS(%u) DSP_CTS(%u) HCIErr(%u)\n",
                    DSP_Status.Bits.DMA_RxIrqCounter,
                    DSP_Status.Bits.DMA_TxIrqCounter,
                    DSP_Status.Bits.MCBSP_RxFull,
                    DSP_Status.Bits.MCBSP_RxSyncError,
                    DSP_Status.Bits.MCBSP_TxSyncError,
                    DSP_Status.Bits.DMA_RxError,
                    DSP_Status.Bits.SIW_CTS,
                    DSP_Status.Bits.DSP_CTS,
                    DSP_Status.Bits.HCI_ParseError);
        DSP_OldStatus = DSP_Status;
    }
}

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static void bluetooth_hci_rx_trigger(void) {
    bluetooth_hci_read_count++;
    HCI_ATM_DSP_Status();
    if (loop) {
        unsigned len = HCI_ATM_RxData(loop_buffer);
        DEB_TRC("hci: HCI_ATM_RxData: Received:%u %c %c ... %c\n", len, loop_buffer[0], loop_buffer[1], len ? loop_buffer[len-1] : ' ');
        if (len && loop == 1) {
            unsigned result;
            result = HCI_ATM_TxData(loop_buffer, len);
            DEB_TRC("hci: HCI_ATM_TxData: Sended:%u\n", result);
        }
    } else {
        DEB_TRC("hci: bluetooth_hci_rx_trigger called\n");
        up(&bluetooth_hci_read_sema);
        if(bluetooth_hci_write_wait) {
            DEB_TRC("wake up ");
            wake_up_interruptible(bluetooth_hci_write_wait);
            DEB_TRC("done\n");
        }
    }
}

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static void bluetooth_hci_tx_trigger(void) {
    HCI_ATM_DSP_Status();
    if(loop == 2) {
        char pHCIBuf[] = {0x1, 0x1a, 0xc, 0x1, 0x3 }; /*--- Write-Scan-Enable ---*/
        unsigned result = HCI_ATM_TxData(pHCIBuf, sizeof(pHCIBuf));
        DEB_TRC("hci-tx: HCI_ATM_TxData: Sended:%u\n", result);
        loop = 3;
        return;
    }
    bluetooth_hci_write_count++;
    /* write aufwecken, wenn blockiert */
    if (bluetooth_hci_write_count == 1)
        up(&bluetooth_hci_write_sema);
    if(bluetooth_hci_read_wait)
        wake_up_interruptible(bluetooth_hci_read_wait);
}

/*-----------------------------------------------------------------------------------------------*\
\*-----------------------------------------------------------------------------------------------*/
int __init bluetooth_hci_init(void) {

    DEB_INFO("%s: register_chrdev()\n", MODULE_NAME);
    bluetooth_hci.major = register_chrdev(0 /*--- dynamic major ---*/, MODULE_NAME, &bluetooth_hci_fops);
    if(bluetooth_hci.major < 1) {
        DEB_ERR("%s: register_chrdrv failed: reason %d\n", MODULE_NAME, bluetooth_hci.major);
        return -ERESTARTSYS;
    }

    if (loop) DEB_INFO("%s: using loop mode\n", MODULE_NAME);

    bluetooth_hci.minor   = 0;
	bluetooth_hci.devfs_handle = devfs_register(NULL, MODULE_NAME, DEVFS_FL_DEFAULT, bluetooth_hci.major, bluetooth_hci.minor,
                                	   S_IFCHR | S_IRUGO | S_IWUSR, &bluetooth_hci_fops, NULL);

    sema_init(&bluetooth_hci_sema, 1);
    sema_init(&bluetooth_hci_write_sema, 0);
    sema_init(&bluetooth_hci_read_sema, 0);
    
    bluetooth_hci_write_count = 0;
    bluetooth_hci_read_count = 0;

    if (loop)
    {
        loop_buffer = kmalloc(bluetooth_hci_message_size, GFP_KERNEL);
        if (loop_buffer == NULL) return -ENOMEM;
    }
    else loop_buffer = NULL;

    HCI_ATM_Init(bluetooth_hci_rx_trigger, bluetooth_hci_tx_trigger, bluetooth_hci_message_size);

    DEB_INFO("%s: major %d (success)\n", MODULE_NAME, bluetooth_hci.major);

    return 0;
}
module_init(bluetooth_hci_init);

/*-----------------------------------------------------------------------------------------------*\
\*-----------------------------------------------------------------------------------------------*/
static void __exit bluetooth_hci_cleanup(void) {
    
    DEB_INFO("%s: unregister_chrdev(%u)\n", MODULE_NAME, bluetooth_hci.major);
    HCI_ATM_DSP_Status();
    if(down_interruptible(&bluetooth_hci_sema)) {
        DEB_ERR("%s down_interruptible() failed\n", MODULE_NAME);
        return;
    }

    if(bluetooth_hci.devfs_handle)
	    devfs_unregister(bluetooth_hci.devfs_handle);
	devfs_unregister_chrdev(bluetooth_hci.major, MODULE_NAME);
    HCI_ATM_Exit();
    up(&bluetooth_hci_sema);
    if (loop)
    {
        if (loop_buffer) kfree(loop_buffer);
    }
    return;
}
module_exit(bluetooth_hci_cleanup);

/*-----------------------------------------------------------------------------------------------*\
\*-----------------------------------------------------------------------------------------------*/
static int bluetooth_hci_fasync(int fd, struct file *filp, int mode) {
    struct _bluetooth_hci_open_data *open_data = (struct _bluetooth_hci_open_data *)filp->private_data;
    /*--- DEB_INFO("%s bluetooth_hci_fasync:\n", MODULE_NAME); ---*/
    return fasync_helper(fd, filp, mode, &(open_data->fasync));
}

/*-----------------------------------------------------------------------------------------------*\
\*-----------------------------------------------------------------------------------------------*/
static unsigned int bluetooth_hci_poll(struct file *filp, poll_table *wait) {
    struct _bluetooth_hci_open_data *open_data = (struct _bluetooth_hci_open_data *)filp->private_data;

    /*--- DEB_INFO("%s bluetooth_hci_poll:\n", MODULE_NAME); ---*/
	poll_wait (filp, open_data->wait_ind, wait);

    if((open_data->flags & O_WRONLY) && (bluetooth_hci_write_count > 0))
	    return POLLIN | POLLRDNORM;
    if((open_data->flags & O_RDONLY) && (bluetooth_hci_read_count > 0))
	    return POLLIN | POLLRDNORM;

    return 0;
}

/*-----------------------------------------------------------------------------------------------*\
\*-----------------------------------------------------------------------------------------------*/
static int bluetooth_hci_open(struct inode *inode, struct file *filp) {
    int minor = inode->i_rdev & 0xFF;

#if defined(MANU_DEBUG)
    int major = inode->i_rdev >> 8;
    DEB_INFO("%s: bluetooth_hci_open: MAJOR %u MINOR %u\n", MODULE_NAME, major, minor);
#endif /*--- #if defined(MANU_DEBUG) ---*/

    if(down_interruptible(&bluetooth_hci_sema)) {
        DEB_ERR("%s down_interruptible() failed\n", MODULE_NAME);
        return -ERESTARTSYS;
    }

    /*-------------------------------------------------------------------------------------------*\
    \*-------------------------------------------------------------------------------------------*/
    if(filp->f_flags & O_APPEND) {
        DEB_ERR("%s: bluetooth_hci_open: open O_APPEND not supported\n", MODULE_NAME);
        up(&bluetooth_hci_sema);
        return -EFAULT;
    }
    if((filp->f_flags & O_RDONLY) && (filp->f_flags & O_WRONLY)) {
        DEB_ERR("%s: bluetooth_hci_open: open O_RDONLY and O_WRONLY simultary not supported\n", MODULE_NAME);
        up(&bluetooth_hci_sema);
        return -EFAULT;
    }

    /*-------------------------------------------------------------------------------------------*\
    \*-------------------------------------------------------------------------------------------*/
    if(!filp->private_data) {
        struct _bluetooth_hci_open_data *open_data = (struct _bluetooth_hci_open_data *)kmalloc(sizeof(struct _bluetooth_hci_open_data), GFP_KERNEL);
        if(!open_data) {
            DEB_ERR("%s: bluetooth_hci_open: open malloc failed\n", MODULE_NAME);
            up(&bluetooth_hci_sema);
            return -EFAULT;
        }
        memset(open_data, 0, sizeof(*open_data));

        /*---------------------------------------------------------------------------------------*\
        \*---------------------------------------------------------------------------------------*/
        open_data->wait_ind = &bluetooth_hci_default_wait_queue; /*--- init der wait queue ---*/
        if(filp->f_flags & O_RDONLY) {
            bluetooth_hci_read_wait = open_data->wait_ind;
        } else if(filp->f_flags & O_WRONLY) {
            bluetooth_hci_write_wait = open_data->wait_ind;
        }
        open_data->flags = filp->f_flags;

        /*---------------------------------------------------------------------------------------*\
        \*---------------------------------------------------------------------------------------*/
        open_data->rx_buffer = kmalloc(bluetooth_hci_message_size, GFP_KERNEL);
        if(open_data->rx_buffer == NULL) {
            kfree(open_data);
            DEB_ERR("%s: bluetooth_hci_open: kmalloc(%u) failed, rx_buffer\n", MODULE_NAME, bluetooth_hci_message_size);
            up(&bluetooth_hci_sema);
            return -EFAULT;
        }
        open_data->rx_buffer_size = bluetooth_hci_message_size;

        open_data->tx_buffer = kmalloc(bluetooth_hci_message_size , GFP_KERNEL);
        if(open_data->tx_buffer == NULL) {
            kfree(open_data->rx_buffer);
            kfree(open_data);
            DEB_ERR("%s: bluetooth_hci_open: kmalloc(%u) failed, tx_buffer\n", MODULE_NAME, bluetooth_hci_message_size);
            up(&bluetooth_hci_sema);
            return -EFAULT;
        }
        open_data->tx_buffer_size = bluetooth_hci_message_size;

        /*---------------------------------------------------------------------------------------*\
        \*---------------------------------------------------------------------------------------*/
        open_data->minor = minor;
        open_data->pf_owner = &(filp->f_owner);
        filp->private_data = (void *)open_data;
    }

    MOD_INC_USE_COUNT;

    DEB_INFO("%s: bluetooth_hci_open: open success flags=0x%x\n", MODULE_NAME, filp->f_flags);

    up(&bluetooth_hci_sema);
    return 0;
}

/*-----------------------------------------------------------------------------------------------*\
\*-----------------------------------------------------------------------------------------------*/
static int bluetooth_hci_close(struct inode *inode, struct file *filp) {

    if(down_interruptible(&bluetooth_hci_sema)) {
        DEB_ERR("%s down_interruptible() failed\n", MODULE_NAME);
        return -ERESTARTSYS;
    }
    /*--- achtung auf ind wartende "gefreien" und warten bis alle fertig ---*/

    if(filp->private_data) {
        struct _bluetooth_hci_open_data *open_data = (struct _bluetooth_hci_open_data *)filp->private_data;

        if(bluetooth_hci_write_wait == open_data->wait_ind)
            bluetooth_hci_write_wait = NULL;
        if(bluetooth_hci_read_wait == open_data->wait_ind)
            bluetooth_hci_read_wait = NULL;
        bluetooth_hci_fasync(-1, filp, 0);  /*--- remove this file from asynchonously notified filp ---*/
        kfree(open_data->rx_buffer);
        kfree(open_data->tx_buffer);
        kfree(open_data);
    }

    MOD_DEC_USE_COUNT;
    up(&bluetooth_hci_sema);
    return 0;
}

/*-----------------------------------------------------------------------------------------------*\
\*-----------------------------------------------------------------------------------------------*/
static ssize_t bluetooth_hci_read(struct file *filp, char *read_buffer, size_t max_read_length, loff_t *read_pos) {
    unsigned int status;
    unsigned int copy_length = 0;
    struct _bluetooth_hci_open_data *open_data = (struct _bluetooth_hci_open_data *)filp->private_data;

    /*------------------------------------------------------------------------------------------*\
     *  ist alles gelesen, dann neuen Block anfordern
    \*------------------------------------------------------------------------------------------*/
    if(*read_pos >= open_data->read_length) {
        /* FIXME: wozu ist bluetooth_hci_read_count??? */
        bluetooth_hci_read_count--;
        open_data->read_length = HCI_ATM_RxData(open_data->rx_buffer);
        if (open_data->read_length == 0) {
            /*--------------------------------------------------------------------------------------*\
             * hci block count semaphore decrementieren
            \*--------------------------------------------------------------------------------------*/
            if(down_interruptible(&bluetooth_hci_read_sema)) {
                DEB_ERR("%s down_interruptible() failed\n", MODULE_NAME);
                return -ERESTARTSYS;
            }
            open_data->read_length = HCI_ATM_RxData(open_data->rx_buffer);
        }
        DEB_TRC("%s: bluetooth_hci_read: HCI_ATM_RxData Received=%u\n", MODULE_NAME, open_data->read_length);
        *read_pos = 0;
    }
    copy_length = min((unsigned int)(open_data->read_length - *read_pos), (unsigned int)max_read_length);

    if(copy_to_user(read_buffer, open_data->rx_buffer + *read_pos, copy_length)) {
        DEB_ERR("%s: tffs_read: copy_to_user failed\n", MODULE_NAME);
        status = (unsigned int)-EFAULT;
    } else {
        *read_pos += copy_length;
        status     = copy_length;
    }

    return status;
}

/*-----------------------------------------------------------------------------------------------*\
\*-----------------------------------------------------------------------------------------------*/
static ssize_t bluetooth_hci_write(struct file *filp, const char *write_buffer, size_t write_length, loff_t *write_pos) {
    unsigned int copy_length = 0;
    unsigned int status;
    struct _bluetooth_hci_open_data *open_data = (struct _bluetooth_hci_open_data *)filp->private_data;

    copy_length = min(open_data->tx_buffer_size, write_length);
    if(copy_from_user(open_data->tx_buffer, write_buffer, copy_length)) {
        DEB_ERR("%s: tffs_read: copy_to_user failed\n", MODULE_NAME);
        status = (unsigned int)-EFAULT;
    } else {
        status        = copy_length;
    }

    status = HCI_ATM_TxData(open_data->tx_buffer, copy_length);
    DEB_TRC("%s: bluetooth_hci_write: HCI_ATM_TxData Sended=%u\n", MODULE_NAME, status);
    
    if (status == 0)
    {
        bluetooth_hci_write_count = 0;
        /*------------------------------------------------------------------------------------------*\
         * schlafen legen bis Tx Trigger kommt
        \*------------------------------------------------------------------------------------------*/
        if(down_interruptible(&bluetooth_hci_write_sema)) {
            DEB_ERR("%s down_interruptible() failed\n", MODULE_NAME);
            return -ERESTARTSYS;
        }
        status = HCI_ATM_TxData(open_data->tx_buffer, copy_length);
        DEB_TRC("%s: bluetooth_hci_write: HCI_ATM_TxData Sended=%u\n", MODULE_NAME, status);
    }
    return status;
}


EXPORT_NO_SYMBOLS;

/*-----------------------------------------------------------------------------------------------*\
\*-----------------------------------------------------------------------------------------------*/
