/*-------------------------------------------------------------------------------------*\
 *     $Header: //avm-fs01/Entwicklung/Basis/Generierung/AR7/kernel/kernel/linux-2.4.17_mvl21/drivers/bluetooth_hci/rcs/hcitransfer.h 1.2 2004/03/05 17:10:22Z mbahr Exp $   
 *     $Id: hcitransfer.h 1.2 2004/03/05 17:10:22Z mbahr Exp $
 *     $Log: hcitransfer.h $
 *     Revision 1.2  2004/03/05 17:10:22Z  mbahr
 *     Testmodes
 *     Revision 1.1  2004/02/24 15:52:36Z  mpommerenke
 *     Initial revision
\*-------------------------------------------------------------------------------------*/

#ifndef __hcitransfer_h__
#define __hcitransfer_h__
/*-------------------------------------------------------------------------------------*\
 * Der Transfer der HCI-Daten zwischen DSP<->MIPS erfolgt ber shared Memory
 * Falls Daten vom SIW anstehen, so wird ber HCIMcBSP_TriggerBT() ein Kommando 
 * (ber TI-HAL) an den MIPS gesendet und dort die Funktion 
 * HCILogicalDevice_DataTriggerFromDSP() aufgerufen
 * HCI-Daten vom MIPS werden mittels HCIMcBSP_Poll() getriggert
\*-------------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------------*\
 * Part 1   DSP-Schnittstelle 
 * - Funktionen die von HCIMcBSP-Einheit bereitgestellt werden
\*-------------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------------*\
 * Initialisiert den McBSP-Einheit
 * SharedMemory: Buffer zum Austausch der HCI-daten ber Rx und Tx
\*-------------------------------------------------------------------------------------*/
void HCIMcBSP_Init(unsigned char *SharedMemory, unsigned int SharedMemoryLength);

/*-------------------------------------------------------------------------------------*\
* DSP signalisiert, das Daten zum SIW anstehen
\*-------------------------------------------------------------------------------------*/
void HCIMcBSP_TriggerBT(void);

/*-------------------------------------------------------------------------------------*\
* Poll-Funktion, die ca. alle msec aufgerufen werden soll (fr Tx-Daten senden)
\*-------------------------------------------------------------------------------------*/
void HCIMcBSP_Poll(void);

/*-------------------------------------------------------------------------------------*\
 * Part 2   MIPS-Schnittstelle
 * Funktionen die von der HCI-ATM-Einheit bereitgestellt werden
\*-------------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------------*\
 * Shared-Buffer-Verwaltung 
 * AdressOffset Len    Name             Kommentar 
 * 0x000        4     Rx_Readindex       Emfangspipe (von DSP) - Readindex (geschrieben von HCI-ATM)
 * 0x004        4     Rx_Writeindex      Emfangspipe (von DSP) - Writeindex (gelesen von HCI-ATM)
 * 0x008       2036   Rx_RingBuffer      wird von DSP gefllt         
 * 0x7FC        4     Dummy
 * 0x800        4     Tx-ReadIndex       SendePipe (zur DSP) - ReadIndex (gelesen von HCI-ATM)
 * 0x804        4     Tx-WriteIndex      SendePipe (zur DSP) - WriteIndex (geschrieben von HCI-ATM)
 * 0x808       2036   Tx-Ringbuffer      wird von HCI-ATM gefllt
 * 0xFFC        4     McBSP-Status       fr Debugzwecke
\*-------------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------------*\
 *  McBSP-Status
\*-------------------------------------------------------------------------------------*/
union _hcimcbsp_status_flag {
    struct {
        unsigned int DMA_RxIrqCounter:8;
        unsigned int DMA_TxIrqCounter:8;
        unsigned int HCI_ParseError:8;
        unsigned int SIW_CTS:1;
        unsigned int DSP_CTS:1;
        unsigned int DMA_RxError:1;
        unsigned int MCBSP_RxFull:1;
        unsigned int MCBSP_RxSyncError:1;
        unsigned int MCBSP_TxSyncError:1;
        unsigned int Reserved:2;
    }Bits;
    unsigned int Register;
};

/*-------------------------------------------------------------------------------------*\
 *    Diese wird vom HCI-Character-Device aufgerufen um HCI zu initialisieren
 *    ATM_HCI_RxTrigger: Callback wird von HCI-ATM aufgerufen, wenn Daten in der Rx-Bytepipe
 *                       vorhanden
 *    ATM_HCI_TxTrigger: Callback wird von HCI-ATM aufgerufen, wenn Platz fr Daten in der 
 *                       Tx-Bytepipe (Flllevel unter 2/3)
\*-------------------------------------------------------------------------------------*/
void HCI_ATM_Init(void (*ATM_HCI_RxTrigger)(void), void (*ATM_HCI_TxTrigger)(void), unsigned int MaxLen);

/*-------------------------------------------------------------------------------------*\
 * HCI-ATM deregistrieren
\*-------------------------------------------------------------------------------------*/
void HCI_ATM_Exit(void);

/*-------------------------------------------------------------------------------------*\
 * Wird vom HCI-Character-Device aufgerufen um Daten abzuholen
 * RxData Buffer zum fllen (maximal:MaxLen)
 * Returnwert: in den RxData-Buffer kopierte Bytes (kann auch 0 sein!)
\*-------------------------------------------------------------------------------------*/
unsigned int HCI_ATM_RxData(unsigned char *RxData);

/*-------------------------------------------------------------------------------------*\
 * Wird vom HCI-Character-Device aufgerufen um Daten zu senden
 * TxData Buffer zum fllen 
 * TxLen(maximal:MaxLen)
 * Returnwert: Anzahl der abgenommenen Bytes (kann auch 0 sein!)
\*-------------------------------------------------------------------------------------*/
unsigned int HCI_ATM_TxData(unsigned char *TxData, unsigned int TxLen);

/*-------------------------------------------------------------------------------------*\
 * liefert Status-Value (Adressoffset 0xFFC  -> 4 Byte des Shared-Buffer)
\*-------------------------------------------------------------------------------------*/
void HCI_ATM_GetDSPStatus(unsigned int *DSP_Status);

#endif/*--- #ifndef __hcitransfer_h__ ---*/
