/*-------------------------------------------------------------------------------------*\

    $Header$

    $Id$

    $Log$

\*-------------------------------------------------------------------------------------*/

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/devfs_fs_kernel.h>
#include <linux/fs.h>
#include <linux/file.h>
#include <asm/uaccess.h>
#include <asm/errno.h>
#include <asm/io.h>
#include <asm/avalanche/avalanche_map.h>
#include <asm/avalanche/generic/avalanche_misc.h>
#include <asm/avalanche/sangam/sangam.h>
#include <asm/avalanche/sangam/hw_boot.h>
#include <asm/avalanche/sangam/hw_emif.h>
#include <asm/avalanche/sangam/hw_gpio.h>

#ifndef TRUE
#define TRUE 1
#endif /*--- #ifndef TRUE ---*/
#ifndef FALSE
#define FALSE 0
#endif /*--- #ifndef FALSE ---*/


/*-------------------------------------------------------------------------------------*\
\*-------------------------------------------------------------------------------------*/
#define MODULE_NAME "avm_piglet"
MODULE_DESCRIPTION("Piglet support by AVM");
MODULE_LICENSE("\n(C) Copyright 2005 by AVM\n");
MODULE_AUTHOR("AVM GmbH");

unsigned int piglet_cs = 0;
MODULE_PARM(piglet_cs, "i");
MODULE_PARM_DESC(piglet_cs, "Piglet chip select");

unsigned char *piglet_bitfile = NULL;
MODULE_PARM(piglet_bitfile, "s");
MODULE_PARM_DESC(piglet_bitfile, "Piglet bitfile name");

int piglet_reset_bit = 2;
MODULE_PARM(piglet_reset_bit, "i");
MODULE_PARM_DESC(piglet_reset_bit, "Piglet reset bit");

int piglet_bitfile_offset = 0x4e;
MODULE_PARM(piglet_bitfile_offset , "i");
MODULE_PARM_DESC(piglet_bitfile_offset, "Piglet bitfile offset");

int piglet_bitfile_write = -1;
MODULE_PARM(piglet_bitfile_write , "i");
MODULE_PARM_DESC(piglet_bitfile_write, "Piglet bitfile size to be written (default = -1 : whole file)");

int piglet_bitfile_revbytes = 0;
MODULE_PARM(piglet_bitfile_revbytes , "i");
MODULE_PARM_DESC(piglet_bitfile_revbytes, "Swap bits in Piglet bitfile bytes: no (=0) or yes (!=0)");

int piglet_usb_power_bit = 15;
MODULE_PARM(piglet_usb_power_bit, "i");
MODULE_PARM_DESC(piglet_usb_power_bit, "Piglet bit for usb power on");

/*-------------------------------------------------------------------------------------*\
\*-------------------------------------------------------------------------------------*/
int piglet_major;

struct file_operations piglet_fops = {
    owner:   THIS_MODULE,
};

struct EMIF_register_memory_map *Piglet_EMIF = (struct EMIF_register_memory_map *) AVALANCHE_EMIF_CONTROL_BASE;
struct _hw_boot *Piglet_BOOT = (struct _hw_boot *)AVALANCHE_DEVICE_CONFIG_LATCH_BASE;

union EMIF_AsyncBankCR EMIF_Configs[2] = {
    {
        {   /*--- program mode ---*/
            0,      /*--- asize (0=8-Bit, 1=16-Bit, 2=32 Bit ---*/
            0x3,    /*--- ta ---*/
            0x7,    /*--- r_hold ---*/
            0x3f,   /*--- r_strobe ---*/
            0xf,    /*--- r_setup ---*/
            0x7,    /*--- w_hold ---*/
            0x3f,   /*--- w_strobe ---*/
            0xf,    /*--- w_setup ---*/
            0x0,    /*--- ew (1=extendend wait mode enabled) ---*/
            0x0     /*--- ss (1=select strobe mode) ---*/
        }
    },
    {
        {  /*--- run mode ---*/
            0,      /*--- asize (0=8-Bit, 1=16-Bit, 2=32 Bit ---*/
            0x3,    /*--- ta ---*/
            0x0,    /*--- r_hold ---*/
            0x06,   /*--- r_strobe ---*/
            0x0,    /*--- r_setup ---*/
            0x0,    /*--- w_hold ---*/
            0x06,   /*--- w_strobe ---*/
            0x0,    /*--- w_setup ---*/
            0x1,    /*--- ew (1=extendend wait mode enabled) ---*/
            0x0     /*--- ss (1=select strobe mode) ---*/
        }
    }
};

unsigned int *piglet_cs_bases[4] = { 
    (unsigned int *) 0xb8000000,
    (unsigned int *) 0xbc000000,
    (unsigned int *) 0xbd000000,
    (unsigned int *) 0xbe000000
};

unsigned char Piglet_RevBytes[256] = {
    0x00, 0x80, 0x40, 0xc0, 0x20, 0xa0, 0x60, 0xe0, 0x10, 0x90, 0x50, 0xd0, 0x30, 0xb0, 0x70, 0xf0,
    0x08, 0x88, 0x48, 0xc8, 0x28, 0xa8, 0x68, 0xe8, 0x18, 0x98, 0x58, 0xd8, 0x38, 0xb8, 0x78, 0xf8, 
    0x04, 0x84, 0x44, 0xc4, 0x24, 0xa4, 0x64, 0xe4, 0x14, 0x94, 0x54, 0xd4, 0x34, 0xb4, 0x74, 0xf4,
    0x0c, 0x8c, 0x4c, 0xcc, 0x2c, 0xac, 0x6c, 0xec, 0x1c, 0x9c, 0x5c, 0xdc, 0x3c, 0xbc, 0x7c, 0xfc,
    0x02, 0x82, 0x42, 0xc2, 0x22, 0xa2, 0x62, 0xe2, 0x12, 0x92, 0x52, 0xd2, 0x32, 0xb2, 0x72, 0xf2,
    0x0a, 0x8a, 0x4a, 0xca, 0x2a, 0xaa, 0x6a, 0xea, 0x1a, 0x9a, 0x5a, 0xda, 0x3a, 0xba, 0x7a, 0xfa,
    0x06, 0x86, 0x46, 0xc6, 0x26, 0xa6, 0x66, 0xe6, 0x16, 0x96, 0x56, 0xd6, 0x36, 0xb6, 0x76, 0xf6,
    0x0e, 0x8e, 0x4e, 0xce, 0x2e, 0xae, 0x6e, 0xee, 0x1e, 0x9e, 0x5e, 0xde, 0x3e, 0xbe, 0x7e, 0xfe,
    0x01, 0x81, 0x41, 0xc1, 0x21, 0xa1, 0x61, 0xe1, 0x11, 0x91, 0x51, 0xd1, 0x31, 0xb1, 0x71, 0xf1,
    0x09, 0x89, 0x49, 0xc9, 0x29, 0xa9, 0x69, 0xe9, 0x19, 0x99, 0x59, 0xd9, 0x39, 0xb9, 0x79, 0xf9,
    0x05, 0x85, 0x45, 0xc5, 0x25, 0xa5, 0x65, 0xe5, 0x15, 0x95, 0x55, 0xd5, 0x35, 0xb5, 0x75, 0xf5,
    0x0d, 0x8d, 0x4d, 0xcd, 0x2d, 0xad, 0x6d, 0xed, 0x1d, 0x9d, 0x5d, 0xdd, 0x3d, 0xbd, 0x7d, 0xfd,
    0x03, 0x83, 0x43, 0xc3, 0x23, 0xa3, 0x63, 0xe3, 0x13, 0x93, 0x53, 0xd3, 0x33, 0xb3, 0x73, 0xf3,
    0x0b, 0x8b, 0x4b, 0xcb, 0x2b, 0xab, 0x6b, 0xeb, 0x1b, 0x9b, 0x5b, 0xdb, 0x3b, 0xbb, 0x7b, 0xfb,
    0x07, 0x87, 0x47, 0xc7, 0x27, 0xa7, 0x67, 0xe7, 0x17, 0x97, 0x57, 0xd7, 0x37, 0xb7, 0x77, 0xf7,
    0x0f, 0x8f, 0x4f, 0xcf, 0x2f, 0xaf, 0x6f, 0xef, 0x1f, 0x9f, 0x5f, 0xdf, 0x3f, 0xbf, 0x7f, 0xff,
};


/*-------------------------------------------------------------------------------------*\
 * Reset des Piglet starten oder beenden                                               *
 *                                                                                     *
 * Argument: Start - Boolescher Wert fuer Reset starten (!=0) oder beenden (=0)        *
\*-------------------------------------------------------------------------------------*/
void piglet_reset(unsigned int Start)
{
    int bit;
    
    if(piglet_reset_bit < 0)
    {
        bit = -piglet_reset_bit;
        Start = !Start;
    }
    else
    {
        bit = piglet_reset_bit;
    }
        
    printk(KERN_ERR "Piglet reset routine: bit = %d, Start = %d\n", bit, Start);
    avalanche_gpio_ctrl(bit, GPIO_PIN, GPIO_OUTPUT_PIN);
    avalanche_gpio_out_bit(bit, Start);
}


/*-------------------------------------------------------------------------------------*\
 * Bitfile einlesen und auswerten                                                      *
 *                                                                                     *
 * Return: 0, bei keinem Fehler; !=0 sonst                                             *
\*-------------------------------------------------------------------------------------*/
int piglet_bitfile_process(void)
{
    unsigned char buffer[4096];
    unsigned int lastdword = 0, bytesRead, totalBytesRead = 0, i, startpos = piglet_bitfile_offset;
    static struct file *fp;
    mm_segment_t  oldfs;  

    printk(KERN_ERR "[Piglet] Opening file '%s'\n", piglet_bitfile);

    fp = filp_open(piglet_bitfile, 00, O_RDONLY);
    if(IS_ERR(fp))
    {
        printk(KERN_ERR "[Piglet] Failed: Could not open bitfile\n");
        return -1;
    }
    /* Lesezugriff auf File(system) erlaubt? */
    if (fp->f_op->read == NULL)
    {
        printk(KERN_ERR "[Piglet] Failed: Could not read bitfile\n");
        return -1;
    }

    /* Von Anfang an lesen */
    fp->f_pos = 0;
    printk(KERN_ERR "[Piglet] Bitfile offset %x/%u\n", startpos, startpos);
    for( ;; )
    {
        oldfs = get_fs();
        set_fs(KERNEL_DS);
        bytesRead = fp->f_op->read(fp, buffer, 4096, &fp->f_pos);
        set_fs(oldfs);
        totalBytesRead += bytesRead;

        printk(KERN_ERR "[Piglet] Read %u bytes from file\n", bytesRead);
        if(bytesRead == 0)
            break;

        /* Daten schreiben */
        for(i = startpos; piglet_bitfile_write && (i < bytesRead); i++, piglet_bitfile_write--)
        {
            if(piglet_bitfile_revbytes)
            {
                lastdword = (lastdword << 4) | Piglet_RevBytes[buffer[i]];
                *((unsigned char *) piglet_cs_bases[piglet_cs]) = Piglet_RevBytes[buffer[i]];
            }
            else
            {
                lastdword = (lastdword << 4) | buffer[i];
                *((unsigned char *) piglet_cs_bases[piglet_cs]) = buffer[i];
            }
        }
        if(!piglet_bitfile_write)
        {
            printk(KERN_ERR "[Piglet] Last written dword: %x\n", lastdword);
        }

        startpos = 0;

        if(bytesRead != 4096)
            break;
    }
    /* Okay, Datei schliessen */
    fput(fp);

    printk(KERN_ERR "[Piglet] Read %u bytes total from file\n", totalBytesRead);
    return 0;
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
unsigned int piglet_registertest(void *Startaddress, unsigned int test_nr) {
    volatile unsigned int *test_ptr;

    switch(test_nr) {
        case 0:
            test_ptr = (volatile unsigned int *)Startaddress;
            printk("piglet_registertest:  read 0x%p = 0x%x\n", 
                    test_ptr, *test_ptr);
            return 0;
        case 1:
            test_ptr = (volatile unsigned int *)Startaddress;
            printk("piglet_registertest:  read 0x%p = *(volatile unsigned char *)0x%x\n", 
                    test_ptr, *((volatile unsigned char *)test_ptr));
            return 0;
        case 2:
            test_ptr = (volatile unsigned int *)Startaddress;
            printk("piglet_registertest:  read 0x%p = *(volatile unsigned short *)0x%x\n", 
                    test_ptr, *((volatile unsigned short *)test_ptr));
            return 0;
        default:
            return 1;
    }

    return 1;
}

/*-------------------------------------------------------------------------------------*\
 * Testen des angegebenen RAM-Bereichs                                                 *
 *                                                                                     *
 * Argumente: Startaddress - Adresse, an der begonnen werden soll                      *
 *            Size         - Groesse des zu testenden Bereichs in unsigned ints        *
 *                                                                                     *
 * Return: 0 bei Erfolg, !=0 sonst                                                     *
\*-------------------------------------------------------------------------------------*/
unsigned int piglet_ramtest(void *Startaddress, unsigned int Size)
{
    unsigned int Testvalue;
    volatile unsigned int *Testptr, *Endaddress;
    
    /* Init Variablen */
    Testptr = (volatile unsigned int *) Startaddress;
    Endaddress = (volatile unsigned int *) ((unsigned int)Startaddress + Size);
    Testvalue = 0;

    /* Testwerte vergleichen */
    while(Testptr < Endaddress)
    {
        *Testptr = Testvalue;
        if(*Testptr != Testvalue)
        {
            printk(KERN_ERR "Piglet failed RAM test at address %p: Got 0x%x, expected 0x%x\n", 
                            Testptr, *Testptr, Testvalue);
            return 1;
        }
        Testptr++;
        Testvalue++;
    }
        
    return 0;
}


/*-------------------------------------------------------------------------------------*\
 * Modul initialisieren                                                                *
\*-------------------------------------------------------------------------------------*/
int __init piglet_init(void)
{
    printk(KERN_INFO "\nPiglet module starting\n");  /* FIXME */

    /* Argumente ueberpruefen */
    if(    (piglet_bitfile == NULL) 
        || (piglet_cs < 2) || (piglet_cs > 5)
        || (piglet_reset_bit > 27) || (piglet_reset_bit < -27))
    {
        printk(KERN_ERR "Wrong arguments! Use piglet_bitfile for the bitfile, piglet_cs as a value (2-5), piglet_reset_bit (-27 - 27)\n");
        return -ERESTARTSYS;
    }
    piglet_cs -= 2;

    /* 48 MHz herausrouten */
    printk(KERN_INFO "\nset GPIO %u to FUNCTIONAL_PIN\n", 6);
    avalanche_gpio_ctrl(6, FUNCTIONAL_PIN, GPIO_OUTPUT_PIN);

    printk(KERN_INFO "\nset hw_boot_test_mux_3 (0x%p)\n", &Piglet_BOOT->hw_boot_test_mux_3);
    Piglet_BOOT->hw_boot_test_mux_3.Bits.pll_pin_out_id  = 2; /*--- usb ---*/
    Piglet_BOOT->hw_boot_test_mux_3.Bits.pll_pin_out_div_enable = 0; /*--- kein divisor ---*/
    Piglet_BOOT->hw_boot_test_mux_3.Bits.pll_pin_output_enable = 1; /*--- enable ---*/
    printk(KERN_INFO "\nhw_boot_test_mux_3=0x%x\n", Piglet_BOOT->hw_boot_test_mux_3.Register);

    /* sleep 100ms */
    printk(KERN_INFO "\nPiglet wait\n");  /* FIXME */
    set_current_state(TASK_INTERRUPTIBLE);
    schedule_timeout(HZ / 10);

    /* Piglet Reset starten */
    printk(KERN_INFO "\nPiglet reset start\n");  /* FIXME */
    piglet_reset(TRUE);

    /* Emif konfigurieren */
    printk(KERN_INFO "\nPiglet EMIF config 1\n");  /* FIXME */
    Piglet_EMIF->AsyncBankCR[piglet_cs].i = EMIF_Configs[0].i;

    /* sleep 100ms */
    printk(KERN_INFO "\nPiglet wait\n");  /* FIXME */
    set_current_state(TASK_INTERRUPTIBLE);
    schedule_timeout(HZ / 10);

    /* Piglet-Reset beenden */
    printk(KERN_INFO "\nPiglet reset stop\n");  /* FIXME */
    piglet_reset(FALSE);

    /* Datei verarbeiten */
    printk(KERN_INFO "\nPiglet process file\n");  /* FIXME */
    if(piglet_bitfile_process())
        goto piglet_reset;

    /* sleep 100ms */
    printk(KERN_INFO "\nPiglet wait (100 ms)\n");  /* FIXME */
    set_current_state(TASK_INTERRUPTIBLE);
    schedule_timeout(HZ / 10);

    /* Emif konfigurieren */
    printk(KERN_INFO "\nPiglet EMIF config 2\n");  /* FIXME */
    Piglet_EMIF->AsyncBankCR[piglet_cs].i = EMIF_Configs[1].i;
    
    printk(KERN_INFO "\nPiglet AsyncWaitCycleCR before change: 0x%x\n", Piglet_EMIF->AsyncWaitCycleCR);  /* FIXME */
    Piglet_EMIF->AsyncWaitCycleCR = 0x80;
    printk(KERN_INFO "\nPiglet AsyncWaitCycleCR after change: 0x%x\n", Piglet_EMIF->AsyncWaitCycleCR);  /* FIXME */

    /* RAM testen */
    printk(KERN_INFO "\nPiglet RAM test\n");  /* FIXME */
    if(piglet_ramtest(piglet_cs_bases[piglet_cs], 0x4000))
        goto piglet_reset;
    /*--------------------------------------------------------------------------------------*\
    \*--------------------------------------------------------------------------------------*/
#if 0
    /* sleep 60s */
    printk(KERN_INFO "\nPiglet wait (60 sec)\n");  /* FIXME */
    set_current_state(TASK_INTERRUPTIBLE);
    schedule_timeout(HZ * 60);

    if(piglet_registertest((unsigned int)piglet_cs_bases[piglet_cs] + 0x4000, 1 /* byte zugriff */))
        goto piglet_reset;
#endif

    /*--------------------------------------------------------------------------------------*\
    \*--------------------------------------------------------------------------------------*/
#if 0
    /* sleep 60s */
    printk(KERN_INFO "\nPiglet wait (60 sec)\n");  /* FIXME */
    set_current_state(TASK_INTERRUPTIBLE);
    schedule_timeout(HZ * 60);

    if(piglet_registertest((unsigned int)piglet_cs_bases[piglet_cs] + 0x4000, 2 /* short zugriff */))
        goto piglet_reset;
#endif


    /*--------------------------------------------------------------------------------------*\
    \*--------------------------------------------------------------------------------------*/
#if 0
    /* sleep 60s */
    printk(KERN_INFO "\nPiglet wait (60 sec)\n");  /* FIXME */
    set_current_state(TASK_INTERRUPTIBLE);
    schedule_timeout(HZ * 60);

    if(piglet_registertest((unsigned int)piglet_cs_bases[piglet_cs] + 0x4000, 0 /* word zugriff */))
        goto piglet_reset;
#endif
    printk(KERN_ERR "Piglet USB power on (gpio %u)\n", piglet_usb_power_bit);
    avalanche_gpio_ctrl(piglet_usb_power_bit, GPIO_PIN, GPIO_OUTPUT_PIN);
    avalanche_gpio_out_bit(piglet_usb_power_bit, 1);

    /* Alles okay, keine Fehler */
    printk(KERN_INFO "\nPiglet module started\n");  /* FIXME */
    return 0;

piglet_reset:
    printk(KERN_ERR "\nPiglet module aborted\n");  /* FIXME */
    return -1;
}
module_init(piglet_init);


/*-------------------------------------------------------------------------------------*\
 * Modul beenden                                                                       *
\*-------------------------------------------------------------------------------------*/
void __exit piglet_cleanup(void)
{
    printk(KERN_INFO "\nPiglet AVM test module end\n"); /* FIXME */
	devfs_unregister_chrdev(piglet_major, MODULE_NAME);
    return;
}
module_exit(piglet_cleanup);

