
#define MAC_IN_VECTOR_TX_INT_OR    (1 << 17)
#define MAC_IN_VECTOR_RX_INT_OR    (1 << 16)

#define scInit(Module)  if (HalDev->State <  enInitialized) return (Module|EC_FUNC_CHSETUP|EC_VAL_INVALID_STATE)
#define scOpen(Module)  if (HalDev->State <  enOpened)      return (Module|EC_FUNC_CHSETUP|EC_VAL_INVALID_STATE)

#include "cpswhaleswitchprivate.h"
typedef void PHY_DEVICE;
#include "cpswhalutlmiimdio.h"

/* #define _CPHAL_HDMA */                                            /*-M001*/

#define PARTIAL 1     /**< Used in @c Close() and @c ChannelTeardown() */
#define FULL 2        /**< Used in @c Close() and @c ChannelTeardown() */

#define MAX_DIR 2
#define TX_CH 0
#define RX_CH 1
#define HAL_ERROR_DEVICE_NOT_FOUND    1
#define HAL_ERROR_FAILED_MALLOC       2
#define HAL_ERROR_OSFUNC_SIZE         3
#define HAL_DEFAULT  0xFFFFFFFF
#define VALID(val)    (val!=HAL_DEFAULT)

int cpswHalEswitchInitModule(HAL_DEVICE **pHalDev, OS_DEVICE *OsDev, HAL_FUNCTIONS **pHalFunc,
                             OS_FUNCTIONS *OsFunc, int OsFuncSize,   int *pHalFuncSize, int Inst)
  {
  HAL_DEVICE *myHalDev;
  HAL_FUNCTIONS *myHalFunc;
  HAL_FUNCTIONS_EXT *myHalFuncExt;
  bit32u psize;
  void *phy0,*phy1;
  volatile bit32u *uip;
  volatile bit8u *ucp;

  if (OsFuncSize < sizeof(OS_FUNCTIONS))
    return(EC_ESWITCH|EC_FUNC_HAL_INIT|EC_VAL_OS_VERSION_NOT_SUPPORTED);

  myHalDev = (HAL_DEVICE *) OsFunc->MallocDev(sizeof(HAL_DEVICE));
  if (!myHalDev)
    return(EC_ESWITCH|EC_FUNC_HAL_INIT|EC_VAL_MALLOC_DEV_FAILED);

  /* clear the HalDev area */
  OsFunc->Memset(myHalDev, 0, sizeof(HAL_DEVICE));

  /* Initialize the size of hal functions */
  *pHalFuncSize = sizeof (HAL_FUNCTIONS);

  myHalFunc = (HAL_FUNCTIONS *) OsFunc->Malloc(sizeof(HAL_FUNCTIONS));
  if (!myHalFunc)
    {
    OsFunc->FreeDev(myHalDev);
    return(EC_ESWITCH|EC_FUNC_HAL_INIT|EC_VAL_MALLOC_FAILED);
    }
  myHalFuncExt = (HAL_FUNCTIONS_EXT *) OsFunc->Malloc(sizeof(HAL_FUNCTIONS_EXT));
  if (!myHalFuncExt)
    {
    OsFunc->FreeDev(myHalDev);
    OsFunc->Free(myHalFunc);
    return(EC_ESWITCH|EC_FUNC_HAL_INIT|EC_VAL_MALLOC_FAILED);
    }

  psize=cpswHalUtlMiiMdioGetPhyDevSize();
  phy0=OsFunc->Malloc(psize);
  if (!phy0)
    {
    OsFunc->FreeDev(myHalDev);
    OsFunc->Free(myHalFunc);
    OsFunc->Free(myHalFuncExt);
    return(EC_ESWITCH|EC_FUNC_HAL_INIT|EC_VAL_MALLOC_FAILED);
    }
  phy1=OsFunc->Malloc(psize);
  if (!phy1)
    {
    OsFunc->FreeDev(myHalDev);
    OsFunc->Free(myHalFunc);
    OsFunc->Free(myHalFuncExt);
    OsFunc->Free(phy0);
    return(EC_ESWITCH|EC_FUNC_HAL_INIT|EC_VAL_MALLOC_FAILED);
    }

  /* clear the function pointers */
  OsFunc->Memset(myHalFunc, 0, sizeof(HAL_FUNCTIONS));
  OsFunc->Memset(myHalFuncExt, 0, sizeof(HAL_FUNCTIONS_EXT));
  OsFunc->Memset(phy0, 0, psize);
  OsFunc->Memset(phy1, 0, psize);

  myHalDev->OsDev   = OsDev;
  myHalDev->Inst    = Inst;
  myHalDev->OsFunc  = OsFunc;
  myHalDev->HalFunc = myHalFunc; /* GSG 11/20 changed name to match cppi */
  myHalDev->PhyState[0]=phy0;
  myHalDev->PhyState[1]=phy1;

  /****************************************************************/
  /*                 POPULATE HALFUNC                             */
  /****************************************************************/
  myHalFunc->ChannelSetup     = cpswHalEswitchChannelSetup;
  myHalFunc->ChannelTeardown  = cpswHalEswitchChannelTeardown; /* GSG 11/20 */
  myHalFunc->Close            = cpswHalEswitchClose; /* GSG 11/20 */
  myHalFunc->Control          = cpswHalEswitchControl; /* GSG 11/22 */
  myHalFunc->Init             = cpswHalEswitchInit;
  myHalFunc->Open             = cpswHalEswitchOpen;
  myHalFunc->PacketProcessEnd = cpswHalEswitchPacketProcessEnd;
  myHalFunc->Probe            = cpswHalEswitchProbe;
  myHalFunc->RxReturn         = cpswHalEswitchRxReturn;
  myHalFunc->Send             = cpswHalEswitchSend;
  myHalFunc->Shutdown         = cpswHalEswitchShutdown;
  myHalFunc->Tick             = cpswHalEswitchTick;
  myHalFunc->HalFuncExt       = myHalFuncExt;

  myHalFuncExt->EswitchDisablePort                = cpswHalEswitchDisablePort;
  myHalFuncExt->EswitchEnablePort                 = cpswHalEswitchEnablePort;
  myHalFuncExt->EswitchEnableMode                 = cpswHalEswitchEnableMode;
  myHalFuncExt->EswitchDisableMode                = cpswHalEswitchDisableMode;
  myHalFuncExt->EswitchCamDump                    = cpswHalEswitchCamDump;
  myHalFuncExt->EswitchCamAddAddress              = cpswHalEswitchCamAddAddress;
  myHalFuncExt->EswitchCamAddVlan                 = cpswHalEswitchCamAddVlan;
  myHalFuncExt->EswitchCamDelAddress              = cpswHalEswitchCamDelAddress;
  myHalFuncExt->EswitchCamDelVlan                 = cpswHalEswitchCamDelVlan;
  myHalFuncExt->EswitchCamFindAddress             = cpswHalEswitchCamFindAddress;
  myHalFuncExt->EswitchCamFindVlan                = cpswHalEswitchCamFindVlan;
  myHalFuncExt->EswitchEnableVIDIngress           = cpswHalEswitchEnableVIDIngress;
  myHalFuncExt->EswitchDisableVIDIngress          = cpswHalEswitchDisableVIDIngress;
  myHalFuncExt->EswitchEnableDropUnTaggedIngress  = cpswHalEswitchEnableDropUnTaggedIngress;
  myHalFuncExt->EswitchDisableDropUnTaggedIngress = cpswHalEswitchDisableDropUnTaggedIngress;
  myHalFuncExt->EswitchSetPriorityMapping         = cpswHalEswitchSetPriorityMapping;
  myHalFuncExt->EswitchSetPortVlanId              = cpswHalEswitchSetPortVlanId;
  myHalFuncExt->EswitchStpPortState               = cpswHalEswitchStpPortState;
  myHalFuncExt->EswitchSetAgeInterval             = cpswHalEswitchSetAgeInterval;

  /* pass the HalDev and HalFunc back to the caller */

  *pHalDev  = myHalDev;
  *pHalFunc = myHalFunc;

  myHalDev->State = enConnected;   /* Initialize the hardware state */
  if (!myHalDev->OsFunc->DeviceFindInfo)
    {
    myHalFunc->Probe            = 0;
    myHalFunc->Init             = 0;

    /* Stub off DeviceFindInfo, DeviceFindParmUint, and DeviceFindParmValue */
    myHalDev->State = enInitialized;   /* Initialize the hardware state */
    }

  uip=(bit32u*)&myHalDev->EswitchBase;
  ucp=(bit8u*)&myHalDev->EswitchBase;
  *uip=0xffffff00;
  if (*ucp) myHalDev->BigEndian=TRUE;
  cpswHalEswitchInitParams(myHalDev);
  return(0);
  }

int cpswHalEswitchOpen(HAL_DEVICE *HalDev)
  {
  char *DummyRam;
  bit32u sts,DescriptorSizePlusBufferSize,RxTxQueueSize,DescriptorBufTotalCount,HostPortBufMaxCount;
  bit32u HostPortBufMinCount,RxDescPerChan,ma0,ma1;
  bit32u RxPort0Desc,RxPort1Desc,RxPort2Desc,RxPort3Desc,BufPtr,DescPtr;
  int port,channel,EswitchPortThresh,EswitchPortTopOff;
  int i;
  int j;
  int rc,Ticks;

   /* Verify proper device state */
  if (HalDev->State != enInitialized)
    return (EC_HDMA|EC_FUNC_OPEN|EC_VAL_INVALID_STATE);

  sts=cpswHalEswitchCheckAllParams(HalDev);
  if (sts) return(sts);

  HalDev->OneSec=HalDev->CpuFrequency;

  HalDev->Cpmac0Base=HalDev->EswitchBase+0x0000;
  HalDev->Cpmac1Base=HalDev->EswitchBase+0x0800;
  HalDev->HdmaBase=HalDev->EswitchBase+0x1000;
  HalDev->MdioBase=HalDev->EswitchBase+0x2000;

  HalDev->dev_base=HalDev->HdmaBase;
  HalDev->State=enInitialized;
  HalDev->PhyStatus=0;
  HalDev->NeedsCount=0;

  for(i=0;i<MAX_CHAN;i++)
    {
    HalDev->ChData[i].TxNumBuffers   = 1;
    HalDev->ChData[i].TxNumQueues    = 1;
    HalDev->TxCtlBlk[i][0].TxServiceMax    = HalDev->TxServiceMax;
    HalDev->TxCtlBlk[i][1].TxServiceMax    = HalDev->TxServiceMax;
    HalDev->ChData[i].RxNumBuffers   = 1;
    HalDev->ChData[i].RxBufferOffset = 0;
    HalDev->ChData[i].RxBufSize      = 1518;
    HalDev->RxCtlBlk[i].RxServiceMax   = HalDev->RxServiceMax;
    }

  HalDev->ChData[1].RxBufSize    = HalDev->RxIPBufSize;
  HalDev->ChData[7].RxBufSize    = HalDev->RxSSBufSize;
  HalDev->ChData[1].RxNumBuffers = HalDev->RxIPNumBuffers;
  HalDev->ChData[7].RxNumBuffers = HalDev->RxSSNumBuffers;
  HalDev->ChData[0].TxNumBuffers = HalDev->TxMaxPending;

  cpswHalUtlUnReset(HalDev->ResetBase,HalDev->EswitchResetBit);
  cpswHalUtlUnReset(HalDev->ResetBase,HalDev->Cpmac0ResetBit);
  cpswHalUtlUnReset(HalDev->ResetBase,HalDev->Cpmac1ResetBit);
  cpswHalUtlUnReset(HalDev->ResetBase,HalDev->MdioResetBit);
  cpswHalUtlUnReset(HalDev->ResetBase,HalDev->Phy0ResetBit);
  cpswHalUtlUnReset(HalDev->ResetBase,HalDev->Phy1ResetBit);
  cpswHalUtlUnReset(HalDev->ResetBase,HalDev->HdmaResetBit);

  PDSP_IMEM_CTL=0;
  if (cpswHalEswitchTestPdspRam(HalDev,"PDSP Packet RAM",PDSP_PCKMEM_BASE,PDSP_PCKMEM_SIZE)) return(PDSP_PCKMEM_FAILURE);
  if (cpswHalEswitchTestPdspRam(HalDev,"PDSP IMEM",PDSP_IMEM_BASE+32,PDSP_IMEM_SIZE-32)) return(PDSP_IMEM_FAILURE);

  /* Start Phy STate Machines                                                */
#ifdef __CPHAL_DEBUG
  HalDev->OsFunc->Printf("Phy Start ...\n");
#endif
  cpswHalUtlMiiMdioInit(HalDev->PhyState[0],HalDev->MdioBase,0,HalDev->MdioPhyMask0,0,HalDev->ResetBase,
                     HalDev->MdioResetBit,HalDev->EswitchFrequency,500000,HalDev->debug&(1<<28),HalDev->OsFunc->Printf);
  cpswHalUtlMiiMdioInit(HalDev->PhyState[1],HalDev->MdioBase,1,HalDev->MdioPhyMask1,0,HalDev->ResetBase,
                     HalDev->MdioResetBit,HalDev->EswitchFrequency,500000,HalDev->debug&(1<<28),HalDev->OsFunc->Printf);

  if (HalDev->EswitchFrequency<50100000)
    {
    HalDev->MdioPhyMode0&=~(NWAY_FD100|NWAY_HD100);
    HalDev->MdioPhyMode1&=~(NWAY_FD100|NWAY_HD100);
    }

  cpswHalUtlMiiMdioSetPhyMode(HalDev->PhyState[0],HalDev->MdioPhyMode0);
  HalDev->CurrentMdioPhyMode0=HalDev->MdioPhyMode0;
  cpswHalUtlMiiMdioSetPhyMode(HalDev->PhyState[1],HalDev->MdioPhyMode1);
  HalDev->CurrentMdioPhyMode1=HalDev->MdioPhyMode1;

  PDSP_SCRATCHPADRAM=(PDSP_PCKMEM_BASE-HalDev->EswitchBase)+PDSP_PCKMEM_SIZE-(6*QUEUE_PTR_SIZE);

  if (cpswHalEswitchPdspLoad(HalDev)) return(PDSP_LOAD_FAILURE);

  if (cpswHalEswitchPdspCmdRamTest(HalDev,0x00010000,PDSP_PCKMEM_SIZE)) return(PDSP_INT_RAMTEST_FAILURE);
  DummyRam=HalDev->OsFunc->MallocDmaXfer(1024,0,0xffffffff);
  if (!DummyRam)
    {
    HalDev->OsFunc->Printf("ERROR: Cannot Malloc DummyRam\n");
    return(MALLOC_FAILURE);
    }
  sts=cpswHalEswitchPdspCmdRamTest(HalDev,VirtToPhys(DummyRam),1024);
  HalDev->OsFunc->FreeDmaXfer(DummyRam);
  if (sts) return(PDSP_EXT_RAMTEST_FAILURE);

  DescriptorSizePlusBufferSize=HalDev->EswitchBufferSize+ESWITCH_DESCRIPTOR_SIZE;
  RxTxQueueSize=QUEUE_PTR_SIZE*ESWITCH_PORTS*ESWITCH_CHANNELS_PER_PORT*2;
  DescriptorBufTotalCount=(PDSP_PCKMEM_SIZE-RxTxQueueSize)/DescriptorSizePlusBufferSize;
  HostPortBufMaxCount=((1536*2)/HalDev->EswitchBufferSize)+1;
  HostPortBufMinCount=(1536/HalDev->EswitchBufferSize)+1;

  if (HalDev->EswitchHostBufCount==0)
    HalDev->EswitchHostBufCount=HostPortBufMaxCount;
   else
    {
    if (HalDev->EswitchHostBufCount>HostPortBufMaxCount) HalDev->EswitchHostBufCount=HostPortBufMaxCount;
    if (HalDev->EswitchHostBufCount<HostPortBufMinCount) HalDev->EswitchHostBufCount=HostPortBufMinCount;
    }

  RxDescPerChan=(DescriptorBufTotalCount-HostPortBufMaxCount)/2;
  EswitchPortThresh=RxDescPerChan/2;
  EswitchPortTopOff=RxDescPerChan/2;

  if (HalDev->debug&(1<<27))
    {
    HalDev->OsFunc->Printf("  HalDev->EswitchBufferSize=%d\n",HalDev->EswitchBufferSize);
    HalDev->OsFunc->Printf("  DescriptorSizePlusBufferSize=%d\n",DescriptorSizePlusBufferSize);
    HalDev->OsFunc->Printf("  RxTxQueueSize=%d\n",RxTxQueueSize);
    HalDev->OsFunc->Printf("  DescriptorBufTotalCount=%d\n",DescriptorBufTotalCount);
    HalDev->OsFunc->Printf("  HostPortBufMaxCount=%d\n",HostPortBufMaxCount);
    HalDev->OsFunc->Printf("  HostPortBufMinCount=%d\n",HostPortBufMinCount);
    HalDev->OsFunc->Printf("  HalDev->EswitchHostBufCount=%d\n",HalDev->EswitchHostBufCount);
    HalDev->OsFunc->Printf("  RxDescPerChan=%d\n",RxDescPerChan);
    }

  cpswHalEswitchPdspCmdSetTenthCount(HalDev,HalDev->EswitchFrequency/ESWITCH_TICS_PER_SEC);/*Set Tics per second*/
  cpswHalEswitchPdspCmdSetAgeInterval(HalDev,10*ESWITCH_TICS_PER_SEC);    /*Age Time in Tics*/
  cpswHalEswitchPdspCmdSetPhyInterval(HalDev,1*ESWITCH_TICS_PER_SEC);     /*Phy Poll time Tics*/
  cpswHalEswitchPdspCmdRestartCam(HalDev);                               /* reinitialize cam to default (vlan unaware)*/
  for(i=0;i<16;i++) HalDev->VlanidValid[i]=FALSE;
  HalDev->VlanidValid[0]=TRUE;
  HalDev->VlanidValue[0]=0;
  cpswHalEswitchPdspCmdSetCamSize(HalDev,HalDev->EswitchDefaultCamSize);                 /* set number of CAM entries available for address learning*/
  cpswHalEswitchPdspCmdSetMode(HalDev,HalDev->CurrentMode=0x00000700);                        /* no garp/stp, allow untag ingress*/
  cpswHalEswitchPdspCmdStpPortMask(HalDev,HalDev->CurrentPortState=0x00000707);                    /* all ports forward and learn*/

  cpswHalEswitchOverFlowQueueInit(HalDev,HalDev->EswitchOverFlowCount,HalDev->EswitchOverFlowSize,EswitchPortThresh,EswitchPortTopOff);

  /*Initialize the Rx Queue Base                                             */

  PDSP_RXQUEUERAM=PDSP_PCKMEM_EOFFSET+PDSP_PCKMEM_SIZE-(QUEUE_PTR_SIZE*ESWITCH_PORTS*ESWITCH_CHANNELS_PER_PORT);

  BufPtr=PDSP_PCKMEM_EOFFSET;
  DescPtr=PDSP_PCKMEM_EOFFSET;

  BufPtr+=((RxDescPerChan*2)+HalDev->EswitchHostBufCount)*ESWITCH_DESCRIPTOR_SIZE;

  RxPort0Desc=DescPtr;
  cpswHalEswitchQueueInit(HalDev,0,0,DescPtr,BufPtr,RxDescPerChan,HalDev->EswitchBufferSize);

  BufPtr+=(HalDev->EswitchBufferSize*RxDescPerChan);
  DescPtr+=(ESWITCH_DESCRIPTOR_SIZE*RxDescPerChan);
  RxPort1Desc=DescPtr;
  cpswHalEswitchQueueInit(HalDev,1,0,DescPtr,BufPtr,RxDescPerChan,HalDev->EswitchBufferSize);

  BufPtr+=(HalDev->EswitchBufferSize*RxDescPerChan);
  DescPtr+=(ESWITCH_DESCRIPTOR_SIZE*RxDescPerChan);
  RxPort2Desc=DescPtr;
  cpswHalEswitchQueueInit(HalDev,2,0,DescPtr,BufPtr,HalDev->EswitchHostBufCount,HalDev->EswitchBufferSize);

  BufPtr+=(HalDev->EswitchBufferSize*HalDev->EswitchHostBufCount);
  DescPtr+=(ESWITCH_DESCRIPTOR_SIZE*HalDev->EswitchHostBufCount);
  RxPort3Desc=DescPtr;

  cpswHalEswitchPdspCmdSetDescPool(HalDev,RxPort0Desc,RxPort1Desc,RxPort2Desc,RxPort3Desc);

  cpswHalEswitchInitCposPool(HalDev,(HalDev->EswitchOverFlowCount+(RxDescPerChan*2)+HalDev->EswitchHostBufCount)*2);

  PDSP_TXQUEUERAM=PDSP_PCKMEM_EOFFSET+PDSP_PCKMEM_SIZE-(QUEUE_PTR_SIZE*ESWITCH_PORTS*ESWITCH_CHANNELS_PER_PORT*2);

  for(port=0;port<3;port++)
    for(channel=0;channel<8;channel++)
      cpswHalEswitchPdspCmdSetTxQueue(HalDev,(channel<<8)|port,0,0,0);

  PDSP_HDMAINTMASK = ( (1<<PDSP_HOST_RXCHAN)|((1<<PDSP_SS_TXCHAN)<<8)|((1<<PDSP_IP_TXCHAN)<<8) );
  cpswHalEswitchPdspCmdSetBufLen(HalDev,HalDev->EswitchBufferSize);

  cpswHalEswitchInitHdmaPort(HalDev);
  cpswHalEswitchPdspCmdInitPort(HalDev,2);
  cpswHalEswitchPdspCmdSetPriorityMapping(HalDev,2, 0, (0<<8) + PDSP_IP_TXCHAN);
  cpswHalEswitchPdspCmdSetPriorityMapping(HalDev,2, 1, (1<<8) + PDSP_IP_TXCHAN);
  cpswHalEswitchPdspCmdSetPriorityMapping(HalDev,2, 2, (2<<8) + PDSP_IP_TXCHAN);
  cpswHalEswitchPdspCmdSetPriorityMapping(HalDev,2, 3, (3<<8) + PDSP_IP_TXCHAN);
  cpswHalEswitchPdspCmdSetPriorityMapping(HalDev,2, 4, (4<<8) + PDSP_IP_TXCHAN);
  cpswHalEswitchPdspCmdSetPriorityMapping(HalDev,2, 5, (5<<8) + PDSP_IP_TXCHAN);
  cpswHalEswitchPdspCmdSetPriorityMapping(HalDev,2, 6, (6<<8) + PDSP_IP_TXCHAN);
  cpswHalEswitchPdspCmdSetPriorityMapping(HalDev,2, 7, (7<<8) + PDSP_IP_TXCHAN);

  ma0 =(((bit32u)HalDev->MacAdr[0])&0x0ff);
  ma0|=(((bit32u)HalDev->MacAdr[1])&0x0ff)<<8;
  ma0|=(((bit32u)HalDev->MacAdr[2])&0x0ff)<<16;
  ma0|=(((bit32u)HalDev->MacAdr[3])&0x0ff)<<24;

  ma1 =(((bit32u)HalDev->MacAdr[4])&0x0ff);
  ma1|=(((bit32u)HalDev->MacAdr[5])&0x0ff)<<8;

  cpswHalEswitchPDSPCommand(HalDev, MIPS_SET_MAC_ADDR, ma0, ma1, 0, 0);
  cpswHalEswitchPdspCmdSetPortVlanId(HalDev,2,0);
  cpswHalEswitchEnablePort(HalDev,2);

  cpswHalEswitchPdspCmdInitPort(HalDev,0);
  cpswHalEswitchPdspCmdSetPriorityMapping(HalDev,0, 0, 0x0002);
  cpswHalEswitchPdspCmdSetPriorityMapping(HalDev,0, 1, 0x0100);
  cpswHalEswitchPdspCmdSetPriorityMapping(HalDev,0, 2, 0x0201);
  cpswHalEswitchPdspCmdSetPriorityMapping(HalDev,0, 3, 0x0303);
  cpswHalEswitchPdspCmdSetPriorityMapping(HalDev,0, 4, 0x0404);
  cpswHalEswitchPdspCmdSetPriorityMapping(HalDev,0, 5, 0x0505);
  cpswHalEswitchPdspCmdSetPriorityMapping(HalDev,0, 6, 0x0606);
  cpswHalEswitchPdspCmdSetPriorityMapping(HalDev,0, 7, 0x0707);
  cpswHalEswitchPdspCmdSetPortVlanId(HalDev,0,0);
  cpswHalEswitchPdspCmdSetDuplex(HalDev,0, 1);
  cpswHalEswitchEnablePort(HalDev,0);

  cpswHalEswitchPdspCmdInitPort(HalDev,1);
  cpswHalEswitchPdspCmdSetPriorityMapping(HalDev,1, 0, 0x0002);
  cpswHalEswitchPdspCmdSetPriorityMapping(HalDev,1, 1, 0x0100);
  cpswHalEswitchPdspCmdSetPriorityMapping(HalDev,1, 2, 0x0201);
  cpswHalEswitchPdspCmdSetPriorityMapping(HalDev,1, 3, 0x0303);
  cpswHalEswitchPdspCmdSetPriorityMapping(HalDev,1, 4, 0x0404);
  cpswHalEswitchPdspCmdSetPriorityMapping(HalDev,1, 5, 0x0505);
  cpswHalEswitchPdspCmdSetPriorityMapping(HalDev,1, 6, 0x0606);
  cpswHalEswitchPdspCmdSetPriorityMapping(HalDev,1, 7, 0x0707);
  cpswHalEswitchPdspCmdSetPortVlanId(HalDev,1,0);
  cpswHalEswitchPdspCmdSetDuplex(HalDev,1, 1);/*!! set duplex based on autonegotiate phys, above!*/
  cpswHalEswitchEnablePort(HalDev,1);

  /* take CPMAC out of reset - GSG 11/20*/

  HalDev->State = enOpened; /* Change device state */

  /* Any pre-open configuration */

  /* For any channels that have been pre-initialized, set them up now */
  /* Note : This loop should not use MAX_CHN, it should only
            loop through Channels Setup, memory should not be reserved
            until Channel is Setup
  */
  for(i=0; i<MAX_CHAN; i++) /* i loops through Channels */
  for(j=0; j<2; j++)        /* j loops through DIRECTION values, 0 and 1  */
    {
     if(HalDev->ChIsSetup[i][j]==TRUE)   /* If the Channel and Direction have been Setup */
       if(HalDev->ChIsOpen[i][j]==FALSE) /* but not opened, then Apply Values now */
         {
          CHANNEL_INFO HalChn;
          HalChn.Channel   = i;
          HalChn.Direction = j;
          rc = cpswHalEswitchChannelConfigApply(HalDev, &HalChn);
          if(rc != EC_NO_ERRORS)
            return(rc);
         }
    }  /* End of looping through Channel/Direction */

  Ticks = HalDev->CpuFrequency / 100;  /*  10 milli-secs in cpu clock cycles */
  HalDev->OsFunc->Control(HalDev->OsDev, hcTick, hcSet, &Ticks); /* GSG 11/22 */

  HalDev->OsFunc->IsrRegister(HalDev->OsDev, cpswHalEswitchIsr, HalDev->EswitchIntBit);
  cpswHalEswitchPacketProcessEnd(HalDev);
  return(EC_NO_ERRORS);
  }



/*--------------------------------------PDSP Memory Map 0x0000               */
/*                                                                           */
/*Rx Desc Port 0                                                             */
/*                                                                           */
/*--------                                                                   */
/*                                                                           */
/*Rx Desc Port 1                                                             */
/*                                                                           */
/*--------                                                                   */
/*                                                                           */
/*Rx Desc Port 2                                                             */
/*                                                                           */
/*--------                                                                   */
/*                                                                           */
/*Bufs Port 0                                                                */
/*                                                                           */
/*--------                                                                   */
/*                                                                           */
/*Bufs Port 1                                                                */
/*                                                                           */
/*--------                                                                   */
/*                                                                           */
/*Bufs Port 2                                                                */
/*                                                                           */
/*--------                                                                   */
/* Scrap                                                                     */
/*------                                                                     */
/*                                                                           */
/*  Tx Queues 3*8                                                            */
/*                                                                           */
/*----------------------||                                                   */
/*                      ||                                                   */
/*  Rx Queues 3*8 queues||                                                   */
/*                      ||                                                   */
/*                      ||-----------------------------------                */
/*                      || ScratchPad Overlays channels 2-7                  */
/*--------------------------------------PDSP Memory Map 0x7ffc               */

/*#define HAL_DEVICE halDevice_s*/

int cpswHalEswitchChannelConfigApply(HAL_DEVICE *HalDev, CHANNEL_INFO *HalChn)
  {
   int Ch        = HalChn->Channel;
   int Direction = HalChn->Direction;

   if (Direction == DIRECTION_TX)
   {
     if (HalDev->ChIsOpen[Ch][Direction] == TRUE)
     {
      return(EC_HDMA|EC_FUNC_CHSETUP|EC_VAL_TX_CH_ALREADY_OPEN);
     }

     /* Initialize Queue Data */
     HalDev->TxCtlBlk[Ch][0].TxActQueueHead   = 0;
     HalDev->TxCtlBlk[Ch][0].TxActQueueCount  = 0;
     HalDev->TxCtlBlk[Ch][0].TxActive         = FALSE;
     HalDev->TxCtlBlk[Ch][1].TxActQueueHead   = 0;
     HalDev->TxCtlBlk[Ch][1].TxActQueueCount  = 0;
     HalDev->TxCtlBlk[Ch][1].TxActive         = FALSE;
     HalDev->TxCtlBlk[Ch][1].LastTcbProcessed = 0;

     /* Initialize buffer memory for the channel */
     cpswHalEswitchInitTcb(HalDev, Ch);

     HDMA_TX_INTMASK_SET(HalDev->dev_base) = (1<<Ch); /* GSG 11/22 */
   }
   else
   {
     if (HalDev->ChIsOpen[Ch][Direction] == TRUE)
     {
      return(EC_HDMA|EC_FUNC_CHSETUP|EC_VAL_RX_CH_ALREADY_OPEN);
     }

     HalDev->InRxInt[Ch]=FALSE;

     /* Initialize Queue Data */
     HalDev->RxCtlBlk[Ch].RxActQueueHead     = 0;
     HalDev->RxCtlBlk[Ch].RxActQueueCount    = 0;
     HalDev->RxCtlBlk[Ch].RxActive           = FALSE;

     /* Initialize buffer memory for the channel */
     cpswHalEswitchInitRcb(HalDev, Ch);

     HDMA_RX_INTMASK_SET(HalDev->dev_base)  = (1<<Ch); /* GSG 11/22 */
   }

   HalDev->ChIsOpen[Ch][Direction] = TRUE; /* channel is open */

   return (EC_NO_ERRORS);
  }

int cpswHalEswitchChannelSetup(HAL_DEVICE *HalDev, CHANNEL_INFO *HalChn, OS_SETUP *OsSetup)
  {
  int Direction;
  int Ch;
  int rc;

  /* Verify proper device state */
  if (HalDev->State < enInitialized)
    return (EC_HDMA|EC_FUNC_CHSETUP|EC_VAL_INVALID_STATE);

  /* We require the channel structure to be passed, even if it only contains
     the channel number */
  if (HalChn == NULL)
    {
     return(EC_HDMA|EC_FUNC_CHSETUP|EC_VAL_NULL_CH_STRUCT);
    }

  Ch        = HalChn->Channel;
  Direction = HalChn->Direction;


  if (!((Ch==0)||(Ch==1)||(Ch==7)))
     return(EC_HDMA|EC_FUNC_CHSETUP|EC_VAL_INVALID_CH);

  /* if channel is already open, this call is invalid */
  if (HalDev->ChIsOpen[Ch][Direction] == TRUE)
    {
     return(EC_HDMA|EC_FUNC_CHSETUP|EC_VAL_CH_ALREADY_OPEN);
    }

#ifdef _CPHAL_HDMA                                                /*MJH+030207*/
     if(Direction == DIRECTION_RX)                                /*MJH+030207*/
       {
       HalDev->ChData[Ch].Receive = HalChn->Receive;              /*MJH+030207*/
       HalDev->RxCtlBlk[Ch].OsSetup = OsSetup;
       }
#endif                                                            /*MJH+030207*/

  /* channel is closed, but might be setup.  If so, reopen the hardware channel. */
  if (HalDev->ChIsSetup[Ch][Direction] == FALSE)
    {
     /* Setup channel configuration */
     HalDev->ChData[Ch].Channel = Ch;

     /* Store OS_SETUP */

     HalDev->ChIsSetup[Ch][Direction] = TRUE;
    }

  rc = EC_NO_ERRORS;

  /* If the hardware has been opened (is out of reset), then configure the channel
     in the hardware. NOTE: cpswHalEswitchChannelConfigApply calls the CPSAR ChannelSetup()! */
  if (HalDev->State == enOpened)
    {
     rc = cpswHalEswitchChannelConfigApply(HalDev, HalChn);
    }

  return (rc);
  }

int cpswHalEswitchIsr(HAL_DEVICE *HalDev, int *MorePackets)
{
  bit32u IntVec;
  int Channel;
  int TxMorePackets=0;
  int RxMorePackets=0;

#ifdef __CPHAL_DEBUG
  if (DBG(0))
    {
    dbgPrintf("\nhalIsr Entry\n"); osfuncSioFlush();/* GSG 11/22 */
    }
#endif

  /* Verify proper device state - important because a call prior to Open would
      result in a lockup */
  if (HalDev->State != enOpened)
     return(EC_HDMA|EC_FUNC_DEVICE_INT|EC_VAL_INVALID_STATE);

  IntVec = HDMA_IN_VECTOR(HalDev->dev_base);

#ifdef __CPHAL_DEBUG
  if (DBG(0))
    {
    dbgPrintf("halIsr: inst %d, IntVec 0x%X\n", HalDev->Inst, IntVec); osfuncSioFlush();/* GSG 11/22 */
    }
#endif

  HalDev->IntVec = IntVec;
  if (IntVec & MAC_IN_VECTOR_TX_INT_OR)
    {
     Channel = ((IntVec>>8) & 0x7);
     cpswHalEswitchTxInt(HalDev, Channel, 0, &TxMorePackets);
    }

  if (IntVec & MAC_IN_VECTOR_RX_INT_OR)
    {
     Channel = (IntVec >> 0) & 0x7;
     cpswHalEswitchRxInt(HalDev, Channel, &RxMorePackets);
    }

  *MorePackets = (TxMorePackets | RxMorePackets);
  return (EC_NO_ERRORS);
}

void cpswHalEswitchDumpPortal(HAL_DEVICE *HalDev,char *str,bit32 adr)
  {
  PDSP_PORTALADR=adr;
  HalDev->OsFunc->Printf("%s - %08x ",str,PDSP_PORTALDAT);
  }

#define CAM_VALID (1<<13)
#define CAM_HIT   (1<<15)
#define CAM_MHIT  (1<<14)

#ifdef _CPSWHAL_DOXYGEN
/**
 * @ingroup ESWITCH_Functions
 * This function dumps the CAM
 *
 * @param   HalDev  CPHAL module instance. (set by cpswHalEswitchInitModule())
 *
 * @return  0 - OK
 */
#endif
int cpswHalEswitchCamDump(HAL_DEVICE *HalDev)
  {
  bit32u i,dlo,dhi,sts;

  HalDev->OsFunc->Printf("\n");
  cpswHalEswitchDumpPortal(HalDev,"MacAdr0 ",0);
  cpswHalEswitchDumpPortal(HalDev,"MacAdr1 ",1);
  HalDev->OsFunc->Printf("\n");

  for(i=0;i<HalDev->EswitchDefaultCamSize;i++)
    if (cpswHalEswitchPdspCmdReadCam(HalDev,i,&dlo,&dhi,&sts)==0)
      {
/*      if (sts&(1<<13))                                                     */
/*      HalDev->OsFunc->Printf("%08x -> %08x %08x\n",sts,dhi,dlo);
      HalDev->OsFunc->Printf(" - %s",(sts&CAM_VALID)?"Valid":"     ");
 */     if (sts&CAM_VALID)
        {
        if (dhi&(1<<31))
          {
          HalDev->OsFunc->Printf(" Vlanid: %03x, Filtid: %01x, Member: %x, UMult: %x, Mult: %x, UTagged: %x\n",
                 ((dlo>>16)&0x0fff),((dhi>>27)&0x0f),
                 ((dhi>>24)&0x07),((dhi>>16)&0x07),((dhi>>8)&0x07),((dhi>>0)&0x07));
          }
         else
          {
          HalDev->OsFunc->Printf(" Address: %02x%02x%02x%02x%02x%02x, Filtid: %01x, Member: %01x\n",
                 ((dlo>>0)&0x0ff),((dlo>>8)&0x0ff),((dlo>>16)&0x0ff),((dlo>>24)&0x0ff),((dhi>>0)&0x0ff),((dhi>>8)&0x0ff),
                 ((dhi>>27)&0x0f),((dhi>>24)&0x07));
          }
        }
    /*   else
        HalDev->OsFunc->Printf("\n");
     */ }
  return(ESWITCH_NOERROR);
  }
#ifdef _CPSWHAL_DOXYGEN
/**
 * @ingroup ESWITCH_Functions
 * This function enables the mode of the eswitch, packets matching the address of the enabled modes are redirected to the SS host port
 *
 * @param   HalDev  CPHAL module instance. (set by cpswHalEswitchInitModule())
 * @param   mode    STP_SUPPORT  - enables STP packet operation<BR>
 *                  GVRP_SUPPORT - enables GVRP packet operation<BR>
 *                  GMRP_SUPPORT - enables GMRP packet operation
 *
 * @return  0 - OK<BR>
 *          ESWITCH_HWTIMEOUT     - The Eswitch PDSP timeout error has occurred 
 */
#endif
int cpswHalEswitchEnableMode(HAL_DEVICE *HalDev,bit32u mode)
  {

  mode&=0xe0000000;
  HalDev->CurrentMode|=mode;
  return(cpswHalEswitchPdspCmdSetMode(HalDev,HalDev->CurrentMode));
  }
#ifdef _CPSWHAL_DOXYGEN
/**
 * @ingroup ESWITCH_Functions
 * This function disables mode of the eswitch, packets matching the address of the enabled modes are redirected to the SS host port
 *
 * @param   HalDev  CPHAL module instance. (set by cpswHalEswitchInitModule())
 * @param   mode    STP_SUPPORT  - disables STP packet operation<BR>
 *                  GVRP_SUPPORT - disables GVRP packet operation<BR>
 *                  GMRP_SUPPORT - disables GMRP packet operation
 *
 * @return  0 - OK<BR>
 *          ESWITCH_HWTIMEOUT     - The Eswitch PDSP timeout error has occurred 
 */
#endif
int cpswHalEswitchDisableMode(HAL_DEVICE *HalDev,bit32u mode)
  {

  mode=~(0xe0000000&mode);
  HalDev->CurrentMode&=mode;
  return(cpswHalEswitchPdspCmdSetMode(HalDev,HalDev->CurrentMode));
  }

#ifdef _CPSWHAL_DOXYGEN
/**
 * @ingroup ESWITCH_Functions
 * This function enables a port VIDIngress check option. If a port's VIDIngres option is enabled, 
 * incomming packets are forwaded only if the port is a member of the VLAN. The VlanId either 
 * comes from the Default VlanId for the port or the incomming packet.
 *
 * @param   HalDev          - CPHAL module instance. (set by cpswHalEswitchInitModule())
 * @param   Port            - The port to enable the VIDIngress for. Valid values are 0, 1, or 2
 *
 * @return  0 - OK<BR>
 *          ESWITCH_INVALIDPORT   -   Invalid Port specified<BR> 
 *          ESWITCH_HWTIMEOUT     - The Eswitch PDSP timeout error has occurred 
 */
#endif
int cpswHalEswitchEnableVIDIngress(HAL_DEVICE *HalDev,bit32u Port)
  {

  if (Port>=3) return(ESWITCH_INVALIDPORT);
  HalDev->CurrentMode|=(1<<(Port));
  return(cpswHalEswitchPdspCmdSetMode(HalDev,HalDev->CurrentMode));
  }

#ifdef _CPSWHAL_DOXYGEN
/**
 * @ingroup ESWITCH_Functions
 * This function diables a port VIDIngress check option. If a ports VIDIngres option is disabled, 
 * incomming tagged packets are always forwaded. The VlanId either 
 * comes from the Default VlanId for the port or the incomming packet.
 *
 * @param   HalDev          - CPHAL module instance. (set by cpswHalEswitchInitModule())
 * @param   Port            - The port to disable the VIDIngress for. Valid values are 0, 1, or 2
 *
 * @return  0 - OK<BR>
 *          ESWITCH_INVALIDPORT   -   Invalid Port specified<BR> 
 *          ESWITCH_HWTIMEOUT     - The Eswitch PDSP timeout error has occurred 
 */
#endif
int cpswHalEswitchDisableVIDIngress(HAL_DEVICE *HalDev,bit32u Port)
  {

  if (Port>=3) return(ESWITCH_INVALIDPORT);
  HalDev->CurrentMode&=~(1<<(Port));
  return(cpswHalEswitchPdspCmdSetMode(HalDev,HalDev->CurrentMode));
  }
#ifdef _CPSWHAL_DOXYGEN
/**
 * @ingroup ESWITCH_Functions
 * This function enables a port DropUnTaggedIngress option. If a ports DropUnTaggedIngress option is enabled, 
 * incomming packets without a VLAN Tag are dropped.
 *
 * @param   HalDev          - CPHAL module instance. (set by cpswHalEswitchInitModule())
 * @param   Port            - The port to enable the DropUnTaggedIngress for. Valid values are 0, 1, or 2
 *
 * @return  0 - OK<BR>
 *          ESWITCH_INVALIDPORT   -   Invalid Port specified<BR> 
 *          ESWITCH_HWTIMEOUT     - The Eswitch PDSP timeout error has occurred 
 */
#endif
int cpswHalEswitchEnableDropUnTaggedIngress(HAL_DEVICE *HalDev,bit32u Port)
  {

  if (Port>=3) return(ESWITCH_INVALIDPORT);
  HalDev->CurrentMode&=~(1<<(Port+8));
  return(cpswHalEswitchPdspCmdSetMode(HalDev,HalDev->CurrentMode));
  }

#ifdef _CPSWHAL_DOXYGEN
/**
 * @ingroup ESWITCH_Functions
 * This function disables a port DropUnTaggedIngress option. If a ports DropUnTaggedIngress option is disabled, 
 * incomming packets without a VLAN Tag are forwarded.
 *
 * @param   HalDev          - CPHAL module instance. (set by cpswHalEswitchInitModule())
 * @param   Port            - The port to disable the DropUnTaggedIngress for. Valid values are 0, 1, or 2
 *
 * @return  0 - OK<BR>
 *          ESWITCH_INVALIDPORT   -   Invalid Port specified<BR> 
 *          ESWITCH_HWTIMEOUT     - The Eswitch PDSP timeout error has occurred 
 */
#endif
int cpswHalEswitchDisableDropUnTaggedIngress(HAL_DEVICE *HalDev,bit32u Port)
  {

  if (Port>=3) return(ESWITCH_INVALIDPORT);
  HalDev->CurrentMode|=(1<<(Port+8));
  return(cpswHalEswitchPdspCmdSetMode(HalDev,HalDev->CurrentMode));
  }

#ifdef _CPSWHAL_DOXYGEN
/**
 * @ingroup ESWITCH_Functions
 * This function adds an Address to a VlanId, If the address does not exist and 
 * there is room in the CAM the information is added. If the Address already exist, 
 * the Member set is updated.
 *
 * @param   HalDev  CPSWHAL module instance. (set by cpswHalEswitchInitModule())
 * @param   addr    6 byte Mac Address to be added
 * @param   VlanId  VLAN ID to add address to
 * @param   Member  Port Mask that addresses packets are forwarded
 *
 * @return  0 - OK<BR>
 *          ESWITCH_INVALIDVLANID - No such VlanId<BR>
 *          ESWITCH_NOSPACE       - No Space in CAM for add<BR>
 *          ESWITCH_HWTIMEOUT     - The Eswitch PDSP timeout error has occurred 
 */
#endif
int cpswHalEswitchCamAddAddress(HAL_DEVICE *HalDev,char *addr,bit32u VlanId,bit32u Member)
  {
  bit32u dhi,dlo,sts,result,i;

  VlanId&=0x0fff;
  Member&=7;

  dlo=(addr[3]<<24)|(addr[2]<<16)|(addr[1]<<8)|(addr[0]<<0);
  dhi=(Member<<24)|(addr[5]<<8)|(addr[4]<<0);
  for(i=0;i<16;i++)
    {
    if ((HalDev->VlanidValid[i])&&(HalDev->VlanidValue[i]==VlanId))
      {
      dhi|=i<<27;
      sts=cpswHalEswitchPdspCmdCamOp(HalDev,1,dlo,dhi,&result);
      if (sts) return(sts);
      if ((result&CAM_VALID)==0) return(ESWITCH_NOSPACE);
      return(ESWITCH_NOERROR);
      }
    }
  return(ESWITCH_INVALIDVLANID);
  }


#ifdef _CPSWHAL_DOXYGEN
/**
 * @ingroup ESWITCH_Functions
 * If the VLAN does not exist, and room remains in the lookup 
 * engine, the VLAN information is added.  If the VLAN already 
 * exists, the specified information is updated in the existing 
 * entry. Only 16 VlanId values can be active at any time.
 *
 * @param   HalDev  -   CPSWHAL module instance. (set by cpswHalEswitchInitModule())
 * @param   VlanId  -   VLAN ID add
 * @param   MemberSet   -   Ports to add to this VLAN.  Each bit represents a 
 *                          one port.  Any value representing a port greater than 2 will be truncated.
 * @param   FwdAllMulticast -   Allow forwarding of all multicast addresses.  
 *                          Each bit represents a one port.  Any value representing a port greater than 2 will be truncated.
 * @param   FwdUnregMulticast   -   Allow forwarding of unregistered multicast addresses.  
 *                          Each bit represents a one port.  Any value representing a port greater than 2 will be truncated.
 * @param   FwdUntagged -   Forward packets untagged.  Each bit represents a one port.  
 *                          Any value representing a port greater than 2 will be truncated.
 *
 * @return  0 - OK<BR>
 *          ESWITCH_NOSPACE      -   All CAM entries are in use<BR>
 *          ESWITCH_HWTIMEOUT    -   The PDSP timeout error has occurred 
 */
#endif
int cpswHalEswitchCamAddVlan(HAL_DEVICE *HalDev,bit32u VlanId,bit32u MemberSet,bit32u FwdAllMulticast,bit32u FwdUnregMulticast,bit32u FwdUntagged)
  {
  bit32u dlo,dhi,i,sts,result;

  VlanId&=0x0fff;
  MemberSet&=0x7;
  FwdUnregMulticast&=0x7;
  FwdAllMulticast&=0x7;
  FwdUntagged&=0x7;
  dlo=(VlanId<<16)|1;
  dhi=(1<<31)|(MemberSet<<24)|(FwdUnregMulticast<<16)|(FwdAllMulticast<<8)|(FwdUntagged<<0);

  for(i=0;i<16;i++)
    {
    if ((HalDev->VlanidValid[i])&&(HalDev->VlanidValue[i]==VlanId))
      {
      HalDev->VlanidInfo[i]=dhi;
      dhi|=i<<27;
      sts=cpswHalEswitchPdspCmdCamOp(HalDev,1,dlo,dhi,&result);
      if (sts) return(ESWITCH_HWTIMEOUT);
      if ((result&CAM_VALID)==0) return(ESWITCH_NOSPACE);
      return(ESWITCH_NOERROR);
      }
    }

  for(i=0;i<16;i++)
    {
    if (!HalDev->VlanidValid[i])
      {
      dhi|=i<<27;
      HalDev->VlanidValue[i]=VlanId;
      sts=cpswHalEswitchPdspCmdCamOp(HalDev,1,dlo,dhi,&result);
      if (sts) return(sts);
      if ((result&CAM_VALID)==0) return(ESWITCH_NOSPACE);
      HalDev->VlanidValid[i]=TRUE;
      HalDev->VlanidInfo[i]=dhi;
      return(ESWITCH_NOERROR);
      }
    }
  return(ESWITCH_NOSPACE);
  }
#ifdef _CPSWHAL_DOXYGEN
/**
 * @ingroup ESWITCH_Functions
 * This function deletes an Address
 *
 * @param   HalDev  CPSWHAL module instance. (set by cpswHalEswitchInitModule())
 * @param   addr    6 byte Mac Address to be deleted from the VLAN
 * @param   VlanId  VLAN ID
 *
 * @return  0 - OK<BR>
 *          ESWITCH_INVALIDVLANID - VlanId not found
 *          ESWITCH_NOTFOUND      - Address not found within Vlan
 *          ESWITCH_HWTIMEOUT     -   The PDSP timeout error has occurred 
 */
#endif
int cpswHalEswitchCamDelAddress(HAL_DEVICE *HalDev,char *addr,bit32u VlanId)
  {
  int i;
  bit32u dhi,dlo,sts,result;

  for(i=0;i<16;i++)
    {
    if ((HalDev->VlanidValid[i])&&(HalDev->VlanidValue[i]==VlanId))
      {
      dlo=(addr[3]<<24)|(addr[2]<<16)|(addr[1]<<8)|(addr[0]<<0);
      dhi=(i<<27)|(addr[5]<<8)|(addr[4]<<0);
      sts=cpswHalEswitchPdspCmdCamOp(HalDev,2,dlo,dhi,&result);
      if (sts) return(sts);
      if ((result&CAM_VALID)==0) return(ESWITCH_NOTFOUND);
      return(ESWITCH_NOERROR);
      }
    }
  return(ESWITCH_INVALIDVLANID);
  }

#ifdef _CPSWHAL_DOXYGEN
/**
 * @ingroup ESWITCH_Functions
 * This function deletes a VLAN from the CAM and all the assotiated addresses
 *
 * @param   HalDev  CPSWHAL module instance. (set by cpswHalEswitchInitModule())
 * @param   VlanId  VLAN ID to be removed
 *
 * @return  0 - OK<BR>
 *          ESWITCH_INVALIDVLANID - No such VlanId found<BR>
 *          ESWITCH_NOTFOUND      - CAM failed to find the requested VlanId<BR>
 *          ESWITCH_HWTIMEOUT     - The PDSP timeout error has occurred 
 */
#endif
int cpswHalEswitchCamDelVlan(HAL_DEVICE *HalDev,bit32u VlanId)
  {
  int i;
  bit32u dhi,dlo,sts,result;

  for(i=0;i<16;i++)
    {
    if ((HalDev->VlanidValid[i])&&(HalDev->VlanidValue[i]==VlanId))
      {
      dlo=(VlanId<<16)|1;
      dhi=(1<<31)|(i<<27);
      sts=cpswHalEswitchPdspCmdCamOp(HalDev,2,dlo,dhi,&result);
      if (sts) return(sts);
      if ((result&CAM_VALID)==0) return(ESWITCH_NOTFOUND);
      HalDev->VlanidValid[i]=FALSE;
      return(ESWITCH_NOERROR);
      }
    }
  return(ESWITCH_INVALIDVLANID);
  }

#ifdef _CPSWHAL_DOXYGEN
/**
 * @ingroup ESWITCH_Functions
 * This function finds an Address
 *
 * @param   HalDev  CPSWHAL module instance. (set by cpswHalEswitchInitModule())
 * @param   addr    6 byte Mac Address to be searched from the VLAN
 * @param   VlanId  VLAN ID
 * @param   member  Not used
 *
 * @return  0 - OK                 - Address was found in the CAM<BR>        
 *          ESWITCH_INVALIDVLANID  - No such VlanId found<BR>
 *          ESWITCH_NOTFOUND       - CAM failed to find the requested VlanId<BR>
 *          ESWITCH_HWTIMEOUT      - The PDSP timeout error has occurred 
 */
#endif
int cpswHalEswitchCamFindAddress(HAL_DEVICE *HalDev,char *addr,bit32u VlanId,bit32u *member)
  {
  int i;
  bit32u dhi,dlo,sts,result;

  for(i=0;i<16;i++)
    {
    if ((HalDev->VlanidValid[i])&&(HalDev->VlanidValue[i]==VlanId))
      {
      dlo=(addr[3]<<24)|(addr[2]<<16)|(addr[1]<<8)|(addr[0]<<0);
      dhi=(i<<27)|(addr[5]<<8)|(addr[4]<<0);
      sts=cpswHalEswitchPdspCmdCamOp(HalDev,0,dlo,dhi,&result);
      if (sts) return(sts);
      if ((result&CAM_VALID)==0) return(ESWITCH_NOTFOUND);
      return(ESWITCH_NOERROR);
      }
    }
  return(ESWITCH_INVALIDVLANID);
  }

#ifdef _CPSWHAL_DOXYGEN
/**
 * @ingroup ESWITCH_Functions
 * Finds the specified VLAN entry in the lookup engine.  The information associated with the entry is returned.
 *
 * @param   HalDev            -   CPSWHAL module instance. (set by cpswHalEswitchInitModule())
 * @param   VlanId            -   VLAN ID to find.
 * @param   MemberSet         -   Returns a bitmap of ports belonging to this VLAN ID
 * @param   FwdAllMulticast   -   Returns a bitmap of ports allowed to forward all multicast addresses.  Each bit represents a one port.  Any value representing a port greater than 2 will be truncated.
 * @param   FwdUnregMulticast -   Returns a bitmap of ports allowed to forward unregistered multicast addresses.  Each bit represents a one port.  Any value representing a port greater than 2 will be truncated.
 * @param   FwdUntagged       -   Returns a bitmap of ports that forward packets untagged.  Each bit represents a one port.  Any value representing a port greater than 2 will be truncated.
 *
 * @return  0 - OK<BR>
 *          ESWITCH_INVALIDVLANID - No such VlanId found
 */
#endif
int cpswHalEswitchCamFindVlan(HAL_DEVICE *HalDev,bit32u VlanId,bit32u *MemberSet,bit32u *FwdAllMulticast,bit32u *FwdUnregMulticast,bit32u *FwdUntagged)
  {
  int i;

  for(i=0;i<16;i++)
    {
    if ((HalDev->VlanidValid[i])&&(HalDev->VlanidValue[i]==VlanId))
      {
      *MemberSet=0x7&(HalDev->VlanidInfo[i]>>24);
      *FwdUnregMulticast=0x7&(HalDev->VlanidInfo[i]>>16);
      *FwdAllMulticast=0x7&(HalDev->VlanidInfo[i]>>8);
      *FwdUntagged=0x7&(HalDev->VlanidInfo[i]>>0);
      return(ESWITCH_NOERROR);
      }
    }
  return(ESWITCH_INVALIDVLANID);
  }

int cpswHalEswitchPhyCheck(HAL_DEVICE *HalDev,int inst);

int  cpswHalEswitchTick(HAL_DEVICE *HalDev)
  {
  bit32u sts=FALSE;

  if (HalDev->State != enOpened)
     return(EC_HDMA|EC_FUNC_DEVICE_INT|EC_VAL_INVALID_STATE);
  sts|=cpswHalEswitchPhyCheck(HalDev,0);
  sts|=cpswHalEswitchPhyCheck(HalDev,1);
  if (sts)
    HalDev->OsFunc->Control(HalDev->OsDev, hcStateChange, hcSet, HalDev->PhyStatus); 
  return(0);
  }

int cpswHalEswitchPhyCheck(HAL_DEVICE *HalDev,int inst)
  {
  bit32u CurrentLinkState,Speed,Duplex,PhyNum,StsChange=FALSE,PhyBitIndex;

  if (cpswHalUtlMiiMdioTic(HalDev->PhyState[inst]))
    {
    PhyBitIndex=inst<<3;
    PhyNum=cpswHalUtlMiiMdioGetPhyNum(HalDev->PhyState[inst]);
    if (PhyNum!=HalDev->PhyNum[inst])
      {
      StsChange=TRUE;
      HalDev->PhyNum[inst]=PhyNum;
      HalDev->PhyStatus&=~(0x1f<<PhyBitIndex);
      HalDev->PhyStatus|=PhyNum<<PhyBitIndex;
      }
    CurrentLinkState=cpswHalUtlMiiMdioGetLinked(HalDev->PhyState[inst]);
    if (CurrentLinkState!=HalDev->PhyLinked[inst])
      {
      HalDev->PhyStatus&=~(1<<(PhyBitIndex+7));  /* Clear Link bit */
      StsChange=TRUE;
      HalDev->PhyLinked[inst]=CurrentLinkState;
      if (CurrentLinkState)
        { /*New Link                                                         */
        HalDev->PhyStatus&=~(3<<(PhyBitIndex+5)); /* Clear Speed/Duplex bits */

        Duplex=cpswHalUtlMiiMdioGetDuplex(HalDev->PhyState[inst]);
        if (Duplex) Duplex=1;    /* Must be in bit 0 for PDSP call */
        cpswHalEswitchPdspCmdSetDuplex(HalDev,inst,Duplex);
        if (HalDev->EnabledPorts&(1<<inst))
          cpswHalEswitchPdspCmdEnablePort(HalDev,inst);

        Speed=cpswHalUtlMiiMdioGetSpeed(HalDev->PhyState[inst]);
        if (Speed) Speed=2;

        CurrentLinkState=4;
        HalDev->PhyStatus|=(CurrentLinkState|Speed|Duplex)<<(PhyBitIndex+5);
        if (HalDev->debug&(1<<26))
          {
          HalDev->OsFunc->Printf("\nEmac %d: Ethernet Phy= %d, Speed=%sM bps, Duplex=%s\n",
                                 inst,PhyNum,(Speed)?"100":"10",
                                 (Duplex)?"Full":"Half");
          }
        }
       else
        {
        cpswHalEswitchPdspCmdDisablePort(HalDev,inst);
        }
      }
    }
  return(StsChange);
  }

int  cpswHalEswitchInit(HAL_DEVICE *HalDev)
  {
  cpswHalEswitchGetConfig(HalDev);
  return(0);
  }

int  cpswHalEswitchPacketProcessEnd(HAL_DEVICE *HalDev)
  {
  HDMA_EOI(HalDev->HdmaBase)=0;
  return(0);
  }

int  cpswHalEswitchProbe(HAL_DEVICE *HalDev)
  {
  return(0);
  }

int  cpswHalEswitchShutdown(HAL_DEVICE *HalDev)
  {

   if (HalDev->State == enOpened)
     cpswHalEswitchClose(HalDev,2);

  HalDev->OsFunc->Free(HalDev->HalFunc->HalFuncExt);
  HalDev->OsFunc->Free(HalDev->HalFunc);
  HalDev->OsFunc->Free(HalDev->PhyState[0]);
  HalDev->OsFunc->Free(HalDev->PhyState[1]);
  HalDev->OsFunc->FreeDev(HalDev);
  return(0);
  }

#ifdef _CPSWHAL_DOXYGEN
/**
 * @ingroup ESWITCH_Functions
 * This function stes the Tx Port priorty mapping. If a packet is transmitted by a 
 * port the priority can be remapped to a different VLAN priority and that packet 
 * can be transmitted on a particular channel. 
 *
 * @param   HalDev          -   CPSWHAL module instance. (set by cpswHalEswitchInitModule())
 * @param   Port            -   Switch Tx Port to modify.  Valid values are 0, 1, or 2.
 * @param   InputPriority   -   Input VLAN/Port Priority (0-7, where 7 is the highest priority)
 * @param   OutputPriority  -   Output VLAN Priority (0-7, where 7 is the highest priority)
 * @param   OutputChannel   -   Output Channel Number (0-7, where 7 is the highest priority channel) This value is ignored for port 2 (host port)
 *
 * @return  0 - OK<BR>
 *          ESWITCH_INVALIDPORT  -   Invalid Port specified<BR> 
 *          ESWITCH_HWTIMEOUT    -   The PDSP timeout error has occurred 
 */
#endif
int cpswHalEswitchSetPriorityMapping(HAL_DEVICE *HalDev, bit32u Port, bit32u InputPriority, bit32u OutputPriority, bit32u OutputChannel)
  {
  bit32u OutPriorityAndChannel;

  if (Port>=3) return(ESWITCH_INVALIDPORT);
  Port&=3;
  InputPriority&=7;
  OutputPriority&=7;
  OutputChannel&=7;

  if (Port>1)
    OutPriorityAndChannel = (OutputPriority<<8) + PDSP_IP_TXCHAN;
   else
    OutPriorityAndChannel = (OutputPriority<<8) + OutputChannel;

  return(cpswHalEswitchPdspCmdSetPriorityMapping(HalDev,Port, InputPriority, OutPriorityAndChannel));
  }

#ifdef _CPSWHAL_DOXYGEN
/**
 * @ingroup ESWITCH_Functions
 * This function Defines the default VLAN ID of a port 
 *
 * @param   HalDev  CPSWHAL module instance. (set by cpswHalEswitchInitModule())
 * @param   Port    -   Port to set VLAN ID.  Valid values are 0, 1, or 2.
 * @param   VlanId  -   VLAN ID to associate with port (0 - 0xFFF)
 *
 * @return  0 - OK<BR>
 *          ESWITCH_INVALIDPORT  -   Invalid Port specified<BR> 
 *          ESWITCH_HWTIMEOUT    -   The PDSP timeout error has occurred 
 */
#endif
int cpswHalEswitchSetPortVlanId(HAL_DEVICE *HalDev, bit32u Port, bit32u VlanId)
  {
  if (Port>=3) return(ESWITCH_INVALIDPORT);
  return(cpswHalEswitchPdspCmdSetPortVlanId(HalDev,Port,VlanId));
  }

#ifdef _CPSWHAL_DOXYGEN
/**
 * @ingroup ESWITCH_Functions
 * This function finds an VLAN Id
 *
 * @param   HalDev  CPSWHAL module instance. (set by cpswHalEswitchInitModule())
 * @param   Port    -   Port to change state.  Valid values are 0, 1, or 2.
 * @param   State   -   Enumerated value.  Valid values are BLOCK, LEARN, LEARN_FWD.
 *
 * @return  0 - OK<BR>
 *          ESWITCH_INVALIDPORT  -   Invalid Port specified<BR> 
 *          ESWITCH_HWTIMEOUT    -   The PDSP timeout error has occurred 
*/
#endif
int cpswHalEswitchStpPortState(HAL_DEVICE *HalDev, bit32u Port, STP_STATE State)
  {

  if (Port>=3) return(ESWITCH_INVALIDPORT);

  if (State!=BLOCK) /* Deal with learn bit */
    HalDev->CurrentPortState|=(1<<(Port+8));
   else
    HalDev->CurrentPortState&=~(1<<(Port+8));
  if (State==LEARN_FWD) /* Deal with forward bit */
    HalDev->CurrentPortState|=(1<<Port);
   else
    HalDev->CurrentPortState&=~(1<<Port);
  return(cpswHalEswitchPdspCmdStpPortMask(HalDev,HalDev->CurrentPortState)); /* set ports forward and learn*/
  }
#ifdef _CPSWHAL_DOXYGEN
/**
 * @ingroup ESWITCH_Functions
 * This function finds an VLAN Id
 *
 * @param   HalDev  CPSWHAL module instance. (set by cpswHalEswitchInitModule())
 * @param   count   -   aging time in seconds(1-0xffffffff).   Valid values are 1 second to 4971 days.
 *
 * @return  0 - OK<BR>
 *          ESWITCH_INVALIDPARM  -   Invalid count specified<BR> 
 *          ESWITCH_HWTIMEOUT    -   The PDSP timeout error has occurred 
 */
#endif
int cpswHalEswitchSetAgeInterval(HAL_DEVICE *HalDev,bit32u count)
  {

  if (count)
    return(cpswHalEswitchPdspCmdSetAgeInterval(HalDev,count*ESWITCH_TICS_PER_SEC));    /*Age Time in Tics*/
  return(ESWITCH_INVALIDPARM);
  }

