#include "cpswhaleswitchprivate.h"
typedef void PHY_DEVICE;
#include "cpswhalutlmiimdio.h"

#ifndef offsetof
/* offset of field m in a struct s */

#define offsetof(s,m)   (size_t)( (char *)&(((s *)0)->m) - (char *)0 )
#endif

#include "cpswhalutlparams.h"

#define REQUIRED REQ
#define WRITABLE_ANYTIME 0
#define WRITABLE_BEFORE_OPEN 0

static const DEVICE_PARAMS EswitchParams[] = 
  { 
    /* primary parameters (stat group 5)                                     */
    {hcid,                    GROUP5|DO_INIT|HAL_DEV|ISSTRING,offsetof(HAL_DEVICE, id),(bit32u)"eswitch", 0, 9},
    {hcoffset,                GROUP5|DO_INIT|HAL_DEV|WIDTH(32),offsetof(HAL_DEVICE, offset),0,0,-1},
    {hcInst,                  GROUP5|DO_INIT|HAL_DEV|WIDTH(32),offsetof(HAL_DEVICE, Inst),0,0,16},

    {hcCpuFrequency,          GROUP5|DO_INIT|HAL_DEV|REQUIRED|WIDTH(32),offsetof(HAL_DEVICE, CpuFrequency),0,1,300000000},
    {hcEswitchFrequency,         GROUP5|DO_INIT|HAL_DEV|REQUIRED|WIDTH(32),offsetof(HAL_DEVICE, EswitchFrequency),0,1,300000000},
    {hcResetBase,             GROUP5|DO_INIT|HAL_DEV|REQUIRED|WIDTH(32),offsetof(HAL_DEVICE, ResetBase),-1,0,0xffffff00},
    {hcEswitchResetBit,       GROUP5|DO_INIT|HAL_DEV|REQUIRED|WIDTH(32),offsetof(HAL_DEVICE, EswitchResetBit),32,0,31},
    {hcCpmac0ResetBit,        GROUP5|DO_INIT|HAL_DEV|REQUIRED|WIDTH(32),offsetof(HAL_DEVICE, Cpmac0ResetBit),32,0,31},
    {hcCpmac1ResetBit,        GROUP5|DO_INIT|HAL_DEV|REQUIRED|WIDTH(32),offsetof(HAL_DEVICE, Cpmac1ResetBit),32,0,31},
    {hcMdioResetBit,          GROUP5|DO_INIT|HAL_DEV|REQUIRED|WIDTH(32),offsetof(HAL_DEVICE, MdioResetBit),32,0,31},
    {hcPhy0ResetBit,          GROUP5|DO_INIT|HAL_DEV|REQUIRED|WIDTH(32),offsetof(HAL_DEVICE, Phy0ResetBit),32,0,31},
    {hcPhy1ResetBit,          GROUP5|DO_INIT|HAL_DEV|REQUIRED|WIDTH(32),offsetof(HAL_DEVICE, Phy1ResetBit),32,0,31},
    {hcHdmaResetBit,          GROUP5|DO_INIT|HAL_DEV|REQUIRED|WIDTH(32),offsetof(HAL_DEVICE, HdmaResetBit),32,0,31},

    {hcCpmac0IntBit,          GROUP5|DO_INIT|HAL_DEV|REQUIRED|WIDTH(32),offsetof(HAL_DEVICE, Cpmac0IntBit),40,0,39},
    {hcCpmac1IntBit,          GROUP5|DO_INIT|HAL_DEV|REQUIRED|WIDTH(32),offsetof(HAL_DEVICE, Cpmac1IntBit),40,0,39},
    {hcEswitchIntBit,         GROUP5|DO_INIT|HAL_DEV|REQUIRED|WIDTH(32),offsetof(HAL_DEVICE, EswitchIntBit),40,0,39},

    {hcEswitchBase,           GROUP5|DO_INIT|HAL_DEV|REQUIRED|WIDTH(32),offsetof(HAL_DEVICE, EswitchBase),-1,0,0xffffff00},

    {hcEswitchBufferSize,     GROUP5|DO_INIT|HAL_DEV|WIDTH(32),offsetof(HAL_DEVICE, EswitchBufferSize),64,64,256},
    {hcEswitchHostBufCount,   GROUP5|DO_INIT|HAL_DEV|WIDTH(32),offsetof(HAL_DEVICE, EswitchHostBufCount),0,0,1024},
    {hcEswitchDefaultCamSize, GROUP5|DO_INIT|HAL_DEV|WIDTH(32),offsetof(HAL_DEVICE, EswitchDefaultCamSize),4,4,64},
    {hcEswitchOverFlowCount,  GROUP5|DO_INIT|HAL_DEV|WIDTH(32),offsetof(HAL_DEVICE, EswitchOverFlowCount),200,0,1024},
    {hcEswitchOverFlowSize,   GROUP5|DO_INIT|HAL_DEV|WIDTH(32),offsetof(HAL_DEVICE, EswitchOverFlowSize),256,64,1536},
    {hcMaxChannel,            GROUP5|DO_INIT|HAL_DEV|WIDTH(32),offsetof(HAL_DEVICE, MaxChannel), 8, 1, 8},

    {hcdebug,                 GROUP5|DO_INIT|HAL_DEV|WRITABLE|WIDTH(32),offsetof(HAL_DEVICE, debug),0,0,-1},

    {hcTxMaxPending,          GROUP5|DO_INIT|HAL_DEV|WIDTH(32),offsetof(HAL_DEVICE, TxMaxPending),128,1,512},
    {hcTxServiceMax,          GROUP5|DO_INIT|HAL_DEV|WIDTH(32),offsetof(HAL_DEVICE, TxServiceMax),32,1,512},
    {hcRxSSBufSize,           GROUP5|DO_INIT|HAL_DEV|WIDTH(32),offsetof(HAL_DEVICE, RxSSBufSize),1518,64,1536},
    {hcRxIPBufSize,           GROUP5|DO_INIT|HAL_DEV|WIDTH(32),offsetof(HAL_DEVICE, RxIPBufSize),1518,64,1536},
    {hcRxServiceMax,          GROUP5|DO_INIT|HAL_DEV|WIDTH(32),offsetof(HAL_DEVICE, RxServiceMax),31,1,512},
    {hcRxSSNumBuffers,        GROUP5|DO_INIT|HAL_DEV|WIDTH(32),offsetof(HAL_DEVICE, RxSSNumBuffers),32,1,1024},
    {hcRxIPNumBuffers,        GROUP5|DO_INIT|HAL_DEV|WIDTH(32),offsetof(HAL_DEVICE, RxIPNumBuffers),128,1,1024},

    {hcMdioPhyMask0,          GROUP5|DO_INIT|HAL_DEV|WIDTH(32),offsetof(HAL_DEVICE, MdioPhyMask0),0x55555555,0,-1},
    {hcMdioPhyMask1,          GROUP5|DO_INIT|HAL_DEV|WIDTH(32),offsetof(HAL_DEVICE, MdioPhyMask1),0xaaaaaaaa,0,-1},
    {hcMdioPhyMode0,          GROUP5|DO_INIT|HAL_DEV|WIDTH(32),offsetof(HAL_DEVICE, MdioPhyMode0),SNWAY_AUTO|SNWAY_FD100|SNWAY_HD100|SNWAY_FD10|SNWAY_HD10,0,-1},
    {hcMdioPhyMode1,          GROUP5|DO_INIT|HAL_DEV|WIDTH(32),offsetof(HAL_DEVICE, MdioPhyMode1),SNWAY_AUTO|SNWAY_FD100|SNWAY_HD100|SNWAY_FD10|SNWAY_HD10,0,-1},

    {hcdev_base,              GROUP5|HAL_DEV|WIDTH(32),offsetof(HAL_DEVICE, dev_base),0,0,-1},
    {hcPhyStatus,             GROUP5|HAL_DEV|WIDTH(32),offsetof(HAL_DEVICE, PhyStatus),0,0,-1},

    /* end of array                                                          */
    {0,0,0,0,0,0}
  };

static const CONFIG_PARAMS IParm[] =
  {
    {"reset",0,20},
    {"base",hcResetBase,21},
    {"eswitch",0,10},
    {"base",hcEswitchBase,11},
    {"int",hcEswitchIntBit,12},
    {"reset_bit",hcEswitchResetBit,13},
    {"reset_base",hcResetBase,21},
    {"rx_buf_count.3.2","RxBufferCount",14},
    {0,0,0}
  };

int  cpswHalEswitchGetConfig(HAL_DEVICE *HalDev);

int  cpswHalEswitchControl(HAL_DEVICE *HalDev, const char *Key, int Action, void *Value)
  {
  int sts;

  if ((cpswHalUtlstrcmp("Action",Key)==0)&&(Action==SETSTR)&&(cpswHalUtlstrcmp("halGetConfig",(char*)Value)==0)) 
    return(cpswHalEswitchGetConfig(HalDev));
  if ((cpswHalUtlstrcmp(hcMacAddress,Key)==0)&&(Action==SETBIN))
    {
    bit8u *ucp;
    int i;

    ucp=Value;
    for(i=0;i<6;i++) HalDev->MacAdr[i]=ucp[i];
    return(0);
    }
  sts=cpswHalUtlParamsControl(HalDev,HalDev->OsFunc,EswitchParams,HalDev->SubStructs,Key,Action,Value);

  /* Temporary After Open Changes, This will be delt with in the Params instead */

  if (HalDev->State == enOpened)
    {
    if (HalDev->CurrentMdioPhyMode0!=HalDev->MdioPhyMode0)
      {
      cpswHalUtlMiiMdioSetPhyMode(HalDev->PhyState[0],HalDev->MdioPhyMode0);
      HalDev->CurrentMdioPhyMode0=HalDev->MdioPhyMode0;
      }
    if (HalDev->CurrentMdioPhyMode1!=HalDev->MdioPhyMode1)
      {
      cpswHalUtlMiiMdioSetPhyMode(HalDev->PhyState[1],HalDev->MdioPhyMode1);
      HalDev->CurrentMdioPhyMode1=HalDev->MdioPhyMode1;
      }
    }
  return(sts);
  }

int  cpswHalEswitchGetConfig(HAL_DEVICE *HalDev)
  {
  return(cpswHalUtlParamsGetConfig(HalDev,HalDev->OsFunc,IParm,EswitchParams,HalDev->SubStructs));
  }

int cpswHalEswitchInitParams(HAL_DEVICE *HalDev)
  {

  /*Setup the substruct                                                      */

  return(cpswHalUtlParamsInit(HalDev,HalDev->OsFunc,EswitchParams));
  }

int cpswHalEswitchCheckAllParams(HAL_DEVICE *HalDev)
  {
  return(cpswHalUtlParamsCheckAllParams(HalDev,HalDev->OsFunc,EswitchParams));
  }

int cpswHalEswitchMdioMode0SetAtction(HAL_DEVICE *HalDev)
  {
/*  cpswHalUtlMiiMdioSetPhyMode(HalDev->PhyState[0],HalDev->MdioPhyMode0);   */
  return(0);
  }

const char hcMacAddress[]="MacAddress";
const char hcEswitchResetBit[]="EswitchResetBit";
const char hcCpmac0ResetBit[]="Cpmac0ResetBit";
const char hcCpmac1ResetBit[]="Cpmac1ResetBit";
const char hcMdioResetBit[]="MdioResetBit";
const char hcPhy0ResetBit[]="Phy0ResetBit";
const char hcPhy1ResetBit[]="Phy1ResetBit";
const char hcHdmaResetBit[]="HdmaResetBit";
const char hcCpmac0IntBit[]="Cpmac0IntBit";
const char hcCpmac1IntBit[]="Cpmac1IntBit";
const char hcEswitchIntBit[]="EswitchIntBit";
const char hcEswitchBase[]="EswitchBase";
const char hcEswitchBufferSize[]="EswitchBufferSize";
const char hcEswitchHostBufCount[]="EswitchHostBufCount";
const char hcEswitchDefaultCamSize[]="EswitchDefaultCamSize";
const char hcEswitchOverFlowCount[]="EswitchOverFlowCount";
const char hcEswitchOverFlowSize[]="EswitchOverFlowSize";
const char hcTxMaxPending[]="TxMaxPending";
const char hcRxSSBufSize[]="RxSSBufSize";
const char hcRxIPBufSize[]="RxIPBufSize";
const char hcRxSSNumBuffers[]="RxSSNumBuffers";
const char hcRxIPNumBuffers[]="RxIPNumBuffers";
const char hcMdioPhyMask0[]="MdioPhyMask0";
const char hcMdioPhyMask1[]="MdioPhyMask1";
const char hcMdioPhyMode0[]="MdioPhyMode0";
const char hcMdioPhyMode1[]="MdioPhyMode1";
const char hcEswitchFrequency[]="EswitchFrequency";
const char hcPhyStatus[]="PhyStatus";
