#include "cpswhaleswitchprivate.h"

int cpswHalEswitchPdspCmdWait(HAL_DEVICE *HalDev)
  {
  bit32u di;

  di=0;
  while(PDSP_COMMAND)
    {
    di+=6;
    if (di>HalDev->OneSec)
      {
      if (HalDev->debug)
        HalDev->OsFunc->Printf("ERROR: PDSP Command Timeout!");
      return(ESWITCH_HWTIMEOUT);
      }
    }
  return(ESWITCH_NOERROR);
  }

int cpswHalEswitchOverFlowQueueInit(HAL_DEVICE *HalDev,int count, int size,int threshold,int topoffto)
  {
  char *buf_ptr;
  dcb_s *dcb_next,*dcb_head,*dcb_tail=0;
  int tmpcount=count;
  bit32u sts=0,MallocSize;

/*temp                                                                       */
size+=63;
size&=~63;

  MallocSize=((size+ESWITCH_DESCRIPTOR_SIZE)*count)+(ESWITCH_DESCRIPTOR_SIZE-1)+64;
  HalDev->EswitchOverFlowArea=HalDev->OsFunc->MallocDmaXfer(MallocSize,0,0xffffffff);
  if (!HalDev->EswitchOverFlowArea) return(cpswHalEswitchError(HalDev,"Cannot Malloc HalDev->EswitchOverFlowArea"));

  cpswHalUtlmemset(HalDev->EswitchOverFlowArea,0,MallocSize);
  HalDev->OsFunc->DataCacheHitWriteback(HalDev->EswitchOverFlowArea, MallocSize);
  HalDev->OsFunc->DataCacheHitInvalidate(HalDev->EswitchOverFlowArea, MallocSize);


  buf_ptr=(char*)((((bit32u)VirtToVirtNoCache(HalDev->EswitchOverFlowArea))+0x1f)&~0x1f);
  dcb_next=(dcb_s*)buf_ptr;
  buf_ptr+=ESWITCH_DESCRIPTOR_SIZE*count;
  buf_ptr+=63;
  buf_ptr=(char *)(((bit32u)buf_ptr)&~63);
  dcb_head=dcb_next;

  while(tmpcount--)
    {
    dcb_tail=dcb_next;
    dcb_tail->HNext=VirtToPhys(++dcb_next);
    dcb_tail->HBuf=VirtToPhys(buf_ptr);
    dcb_tail->OffLen=size&0x0ffff;
    dcb_tail->Status=OWNERSHIP;
    buf_ptr+=size;
    }
  if (dcb_tail)
    {
    dcb_tail->HNext=0;
    sts=cpswHalEswitchPdspCmdSetOverflowQueue(HalDev,(bit32u)VirtToPhys(dcb_head),(bit32u)VirtToPhys(dcb_tail),count,(topoffto<<16)|(threshold));
    sts|=cpswHalEswitchPdspCmdSetOverflowBufLen(HalDev,size);
    }
  return(sts);
  }

int cpswHalEswitchQueueInit(HAL_DEVICE *HalDev,int Port,int Channel,bit32u HeadPtr,bit32u BufPtr,int Count,int Size)
  {
  bit32u dcb_next,dcb_tail=0;
  int tmpcount=Count,sts=0;

  if (!Count) return(cpswHalEswitchError(HalDev,"cpswHalEswitchQueueInit cannot have Count of Zero!"));

  dcb_next=HeadPtr;
  while(tmpcount--)
    {
    dcb_tail=dcb_next;
    dcb_next+=ESWITCH_DESCRIPTOR_SIZE;
    *((bit32u*)(HalDev->EswitchBase+dcb_tail+0x0))=dcb_next;
    *((bit32u*)(HalDev->EswitchBase+dcb_tail+0x4))=BufPtr;
    *((bit32u*)(HalDev->EswitchBase+dcb_tail+0x8))=Size&0x0ffff;
    *((bit32u*)(HalDev->EswitchBase+dcb_tail+0xc))=OWNERSHIP;
    BufPtr+=Size;
    }

  *((bit32u*)(HalDev->EswitchBase+dcb_tail+0x0))=0;

  sts=cpswHalEswitchPdspCmdSetRxQueue(HalDev,(Channel<<8)|Port,HeadPtr,dcb_tail,Count);
  return(sts);
  }
#ifdef _CPSWHAL_DOXYGEN
/**
 * @ingroup ESWITCH_Functions
 * This function enables a Switch port
 *
 * @param   HalDev  CPHAL module instance. (set by cpswHalEswitchInitModule())
 * @param   port    Switch Port to enable<br>
 *                  Valid values are 0, 1 or 2
 *
 * @return  N/A
 */
#endif
int cpswHalEswitchEnablePort(HAL_DEVICE *HalDev,bit32u port)
  {
  if (port>=3) return(ESWITCH_INVALIDPORT);
  HalDev->EnabledPorts|=(1<<port);
  if (port==2) 
    return(cpswHalEswitchPdspCmdEnablePort(HalDev,port));
  if (HalDev->PhyStatus&(1<<((port<<3)+7)))
    return(cpswHalEswitchPdspCmdEnablePort(HalDev,port));
  return(0);
  }
#ifdef _CPSWHAL_DOXYGEN
/**
 * @ingroup ESWITCH_Functions
 * This function disables a Switch port
 *
 * @param   HalDev  CPHAL module instance. (set by cpswHalEswitchInitModule())
 * @param   port    Switch Port to disable<br>
 *                  Valid values are 0, 1 or 2
 *
 * @return  N/A
 */
#endif
int cpswHalEswitchDisablePort(HAL_DEVICE *HalDev,bit32u port)
  {
  if (port>=3) return(ESWITCH_INVALIDPORT);
  HalDev->EnabledPorts&=~(1<<port);
  return(cpswHalEswitchPdspCmdDisablePort(HalDev,port));
  }

int cpswHalEswitchInitCposPool(HAL_DEVICE *HalDev,int count)
  {
  char *pChar;
  dcb_s *dcb_next,*dcb_head,*dcb_tail=0;
  int tmpcount=count;
  bit32u sts=0,MallocSize;

  MallocSize=(ESWITCH_DESCRIPTOR_SIZE*count)+(ESWITCH_DESCRIPTOR_SIZE-1);
  HalDev->EswitchCposArea=HalDev->OsFunc->MallocDmaXfer(MallocSize,0,0xffffffff);
  if (!HalDev->EswitchCposArea) return(cpswHalEswitchError(HalDev,"Cannot Malloc HalDev->EswitchCposArea"));

  cpswHalUtlmemset(HalDev->EswitchCposArea,0,MallocSize);
  HalDev->OsFunc->DataCacheHitWriteback(HalDev->EswitchCposArea, MallocSize);
  HalDev->OsFunc->DataCacheHitInvalidate(HalDev->EswitchCposArea, MallocSize);

  pChar=(char*)((((bit32u)VirtToVirtNoCache(HalDev->EswitchCposArea))+0x1f)&~0x1f);
  dcb_next=(dcb_s*)pChar;
  dcb_head=dcb_next;

  while(tmpcount--)
    {
    dcb_tail=dcb_next;
    dcb_tail->HNext  =VirtToPhys(++dcb_next);
    dcb_tail->HBuf   =0xdeadbeef;
    dcb_tail->OffLen =0xdeadbeef;
    dcb_tail->Status =OWNERSHIP;
    dcb_tail->HNext2 =VirtToPhys(dcb_tail);
    dcb_tail->HBuf2  =VirtToPhys(0x0001fff8);
    dcb_tail->OffLen2=4;
    dcb_tail->Status2=OWNERSHIP;
    }
  if (dcb_tail)
    {
    dcb_tail->HNext=0;
    cpswHalEswitchPDSPCommand(HalDev, MIPS_SET_CPOS_DESCPOOL, (bit32u)VirtToPhys(dcb_head), (bit32u)VirtToPhys(dcb_tail), 0, 0);
    }
  return(sts);
  }

