/*************************************************************************
 *    TNETDxxxx Software Support
 *  Copyright (c) 2003 Texas Instruments Incorporated. All Rights Reserved.
 * 
 *  FILE:
 *
 *  DESCRIPTION:
 *   This file contains all the global constructs for the module : cpswhalcommon
 *   This file is automatically created from cpswhalcommon_global.c using parse_global version 1.00
 *
 *  HISTORY:
 *  Maintained in cpswhalcommon_global.c 
 ***************************************************************************/
#ifndef _cpswhalcommon_GLOBAL_H
#define _cpswhalcommon_GLOBAL_H


extern HAL_CONTROL_KEY hkVersion_COMMON[];

extern HAL_CONTROL_KEY hkVersionCommon[];
extern HAL_CONTROL_KEY hkVersionCppi[];

/* Keys which do not need to appear in the document (unused by OS code) */
extern HAL_CONTROL_KEY hkReset[];
extern HAL_CONTROL_KEY hkLocator[];
extern HAL_CONTROL_KEY hkVlynq[];
extern HAL_CONTROL_KEY hkInst[];
extern HAL_CONTROL_KEY hkStats[];
extern HAL_CONTROL_KEY hkNULL[];

/**
@defgroup Common_Config_Params Common Configuration Parameters

This section documents the configuration parameters that are valid across
all CPHAL devices.
@{
*/

/********************************************************************
**
**  Environment Parameters - Environment is case-sensitive,
**                           parameters do not need to match but it is desirable
**
********************************************************************/

/** @hideinitializer CPU Frequency. */
extern HAL_CONTROL_KEY hkCpuFrequency[];

/********************************************************************
**
**  OPTIONS.CONF Parameters - Must match value in options.conf
**
********************************************************************/

/** @hideinitializer Id string used in options.conf */
extern HAL_CONTROL_KEY hkId[];
/** @hideinitializer Base address for the module. */
extern HAL_CONTROL_KEY hkBase[];
/** @hideinitializer Size of the register map for the module */
extern HAL_CONTROL_KEY hkSize[];
/** @hideinitializer Reset bit for the module. */
extern HAL_CONTROL_KEY hkResetBit[];
/** @hideinitializer Interrupt line for the module. */
extern HAL_CONTROL_KEY hkIntLine[];
/** @hideinitializer VLYNQ offset for the module.  Disregard if not using VLYNQ. */
extern HAL_CONTROL_KEY hkOffset[];
/** @hideinitializer Reset base address for the module. */
extern HAL_CONTROL_KEY hkResetBase[];
/** @hideinitializer This is the debug level.  The field is bit defined, such that the user
should set to 1 all the bits corresponding to desired debug outputs.  The following
are the meanings for each debug bit:
- bit0 (LSB): CPHAL Function Trace
- b1 : OS Function call trace
- b2 : Critical section entry/exit
- b3 : Memory allocation/destruction
- b4 : Detailed information in Rx path
- b5 : Detailed information in Tx path
- b6 : Extended error information
- b7 : General info
*/
extern HAL_CONTROL_KEY hkDebug[];
/** @hideinitializer The OS may "Get" this parameter, which is a pointer
    to a character string that indicates the version of CPHAL. */
extern HAL_CONTROL_KEY hkVersion[];
/** @hideinitializer The maximum number of Rx channels supported by the module. */
extern HAL_CONTROL_KEY hkRxMaxChan[];
/** @hideinitializer The maximum number of Tx channels supported by the module. */
extern HAL_CONTROL_KEY hkTxMaxChan[];
/** @hideinitializer Used to determine the status of the module.  Currently used by the
    CPMAC module only. */
extern HAL_CONTROL_KEY hkStatus[];
/** @hideinitializer Used by the HAL to signal the OS that the system is ready to be taken
    out of reset.  This gives the OS a window to perform any desired configuration changes
    prior to the module being started.  Currently used by the CPMAC module only. */
extern HAL_CONTROL_KEY hkReady2Go[];
/** @hideinitializer Key to control Phy connect ability */
extern HAL_CONTROL_KEY hkMdioPhyMode[];
/** @hideinitializer Controls setting of MDI/MDIX */
extern HAL_CONTROL_KEY hkMdioMdixSwitch[];
/** @hideinitializer Specifies which PHYs support AutoMidx */
extern HAL_CONTROL_KEY hkMdixMask[];
/** @hideinitializer Controls PHY groupings */
extern HAL_CONTROL_KEY hkPhyMask[];
/** @hideinitializer Specifies LINK usage per PHY */
extern HAL_CONTROL_KEY hkMLink[];

/* @} */

/**
@defgroup Common_Control_Params Common Keys for [os]Control()

This section documents the keys used with the OS @c Control() interface that
are required by CPHAL devices.

@{
*/

/** @hideinitializer The HAL will call the OS during initialization to configure the tick
    timer using this key.  The associated value should be used as an interval to time OS
    calls to the HAL Tick() function. Currently used by the CPMAC module only. */
extern HAL_CONTROL_KEY hkTick[];
/** @hideinitializer Used to wait for an integer number of cpu cycles.  No actions are defined. */
extern HAL_CONTROL_KEY hkSleep[];
/** @hideinitializer Used by the HAL to signal a change in state to the OS.  The OS should
    then check the module status by performing an haGETbyREF action on the key hkStatus. */
extern HAL_CONTROL_KEY hkStateChange[];
/** @hideinitializer Requests the OS to flush it's I/O buffers.  No actions are defined. */
extern HAL_CONTROL_KEY hkSioFlush[];
/* @} */


/**
@defgroup Common_Control_Params_MAC_HDMA Other Configuration Keys

This section documents keys which are currently used by the CPMAC
and HDMA modules only.  This includes keys for accessing GPIO and
CPMDIO fields, among others.

@{
*/

/* Mick, please document these keys. */
extern HAL_CONTROL_KEY hkGpio[];
extern HAL_CONTROL_KEY hkGpioMux[];
extern HAL_CONTROL_KEY hkGpioBase[];
extern HAL_CONTROL_KEY hkGpioResetBit[];

extern HAL_CONTROL_KEY hkCpmdio[];
extern HAL_CONTROL_KEY hkCpmdioBase[];
extern HAL_CONTROL_KEY hkCpmdioIntLine[];
extern HAL_CONTROL_KEY hkCpmdioResetBit[];
extern HAL_CONTROL_KEY hkMdioBusFrequency[];
extern HAL_CONTROL_KEY hkMdioClockFrequency[];
/** @hideinitializer Valid after Link seen. Returns the current PHY number. */
extern HAL_CONTROL_KEY hkPhyNum[];
extern HAL_CONTROL_KEY hkPhyAccess[];
extern HAL_CONTROL_KEY hkMacAddr[];

/** @} */

/* Below are all the CPPI specific keys */
/* Configurable items */

/**
@defgroup Common_CPPI_Params Configuration Parameters for CPPI Modules

This section documents the configuration parameters that are valid across
all CPPI-based CPHAL modules.
@{
*/
/** @hideinitializer Rx buffer size. */
extern HAL_CONTROL_KEY hkRxBufSize[];
/** @hideinitializer Rx buffer offset. */
extern HAL_CONTROL_KEY hkRxBufferOffset[];
/** @hideinitializer Number of Rx buffers to use. */
extern HAL_CONTROL_KEY hkRxNumBuffers[];
/** @hideinitializer Maximum number of Rx packets to service at once during an interrupt. */
extern HAL_CONTROL_KEY hkRxServiceMax[];
/** @hideinitializer Number of Tx buffer descriptors to use. */
extern HAL_CONTROL_KEY hkTxNumBuffers[];
/** @hideinitializer Maximum number of Tx packets to service at once during an interrupt. */
extern HAL_CONTROL_KEY hkTxServiceMax[];
/** @hideinitializer This is the maximum number of Rx fragments to support in the system.  The
    default value is documented on a module-by-module basis.  */
extern HAL_CONTROL_KEY hkRxMaxFrags[];

/** @} */
/* Internal information, for debug only (do not generate documentation for these) */

extern HAL_CONTROL_KEY hkTxActQueueCount[];
extern HAL_CONTROL_KEY hkRxActQueueCount[];
extern HAL_CONTROL_KEY hkTxActQueueHead[];
extern HAL_CONTROL_KEY hkRxActQueueHead[];
extern HAL_CONTROL_KEY hkTxActQueueTail[];
extern HAL_CONTROL_KEY hkRxActQueueTail[];
extern HAL_CONTROL_KEY hkTxActive[];
extern HAL_CONTROL_KEY hkRxActive[];
extern HAL_CONTROL_KEY hkTcbPool[];
extern HAL_CONTROL_KEY hkRcbPool[];
extern HAL_CONTROL_KEY hkTcbStart[];
extern HAL_CONTROL_KEY hkRcbStart[];
extern HAL_CONTROL_KEY hkDumpParams[];

/* Useful stats */
/** @hideinitializer Number of RX CRC errors reported by the SAR and USB modules. This error is only
    reported for AAL5,OAM and USB channels. */
extern HAL_CONTROL_KEY hkRxCrcErrors[];
/** @hideinitializer Incremented if the sum of the fragment lengths differs from the packet length. */
extern HAL_CONTROL_KEY hkDmaLenErrors[];
/** @hideinitializer Number of times the Tx controller needed to requeue a send request (not an error). */
extern HAL_CONTROL_KEY hkTxMisQCnt[];
/** @hideinitializer Number of times the Rx controller needed to requeue a send request (not an error). */
extern HAL_CONTROL_KEY hkRxMisQCnt[];
/** @hideinitializer Number of times the module reached the end of the Tx queue. */
extern HAL_CONTROL_KEY hkTxEOQCnt[];
/** @hideinitializer Number of times the module reached the end of the Rx queue. */
extern HAL_CONTROL_KEY hkRxEOQCnt[];
/** @hideinitializer Number of Tx packets serviced for a given channel and queue. */
extern HAL_CONTROL_KEY hkTxPacketsServiced[];
/** @hideinitializer Number of Rx packets serviced for a given channel. */
extern HAL_CONTROL_KEY hkRxPacketsServiced[];
/** @hideinitializer The maximum number of Tx packets ever serviced on a given channel and queue. */
extern HAL_CONTROL_KEY hkTxMaxServiced[];
/** @hideinitializer The maximum number of Rx packets ever serviced on a given channel. */
extern HAL_CONTROL_KEY hkRxMaxServiced[];
/** @hideinitializer The total number of Tx packets serviced across all channels and queues. */
extern HAL_CONTROL_KEY hkTxTotal[];
/** @hideinitializer The total number of Rx packets serviced across all channels. */
extern HAL_CONTROL_KEY hkRxTotal[];
extern HAL_CONTROL_KEY hkLastTcbProcessed[];

/* Internal */
extern HAL_CONTROL_KEY hkDio[];
extern HAL_CONTROL_KEY hkPhyReset[];
#endif /*ends #ifndef _cpswhalcommon_GLOBAL_H*/





