/*-----------------------------------------------------------------------------------------------*\
 *
 *  $id$
 *
 *  $Log: tffs_main.c $
 *  Revision 1.14  2004/02/10 12:00:04Z  mpommerenke
 *  Revision 1.13  2003/12/22 08:57:59Z  mpommerenke
 *  Revision 1.12  2003/12/11 11:21:58Z  mpommerenke
 *  Revision 1.11  2003/11/26 12:01:35Z  mpommerenke
 *  Revision 1.10  2003/11/25 12:05:14Z  mpommerenke
 *  Revision 1.9  2003/11/25 11:05:03Z  mpommerenke
 *  Revision 1.8  2003/11/24 14:48:14Z  mpommerenke
 *  Revision 1.7  2003/11/24 14:15:52Z  mpommerenke
 *  Revision 1.6  2003/11/19 17:30:16Z  mpommerenke
 *
\*-----------------------------------------------------------------------------------------------*/
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/proc_fs.h>
#include <linux/sched.h>
#include <asm/uaccess.h>
#include <linux/config.h>
#include <linux/slab.h>
#include <asm/fcntl.h>
#include <asm/ioctl.h>
#include <linux/devfs_fs_kernel.h>
#include <asm/semaphore.h>
#include <asm/errno.h>
#include <linux/wait.h>
#include <asm/smplock.h>
#include <linux/tffs.h>
#if defined(CONFIG_TFFS_MINOR_COMPRESS)
#include <linux/zlib.h>
#endif /*--- #if defined(CONFIG_TFFS_MINOR_COMPRESS) ---*/
#if defined(CONFIG_PROC_FS)
#include <linux/proc_fs.h>
#endif /*--- #if defined(CONFIG_PROC_FS) ---*/

/*-----------------------------------------------------------------------------------------------*\
\*-----------------------------------------------------------------------------------------------*/
/*--- #define TFFS_DEBUG ---*/

#if defined(TFFS_DEBUG)
#define DBG(a)  printk a
#else /*--- #if defined(TFFS_DEBUG) ---*/
#define DBG(a)  
#endif /*--- #else ---*/ /*--- #if defined(TFFS_DEBUG) ---*/

/*-----------------------------------------------------------------------------------------------*\
\*-----------------------------------------------------------------------------------------------*/
#define MODULE_NAME     "tffs"
#if defined(CONFIG_TFFS_DOUBLE_BUFFER)
static int mtd[2] = { CONFIG_TFFS_MTD_DEVICE_0 - 1, CONFIG_TFFS_MTD_DEVICE_1 - 1 };
static int anzahl = 0;  /*--- default alle ---*/
MODULE_PARM_DESC (tffs_mtd, "MTD device used for tiny flash file system with double buffering");
/*--- MODULE_AUTHOR("Martin Pommerenke (AVM Berlin)  m.pommerenke@avm.de"); ---*/
MODULE_DESCRIPTION("TFFS 'tiny flash file system with double buffering' driver version 2.2");
MODULE_SUPPORTED_DEVICE("MTD[0..n]");
#define MODULE_VERSION "1.2"


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
#else /*--- #if defined(CONFIG_TFFS_DOUBLE_BUFFER) ---*/
static int mtd[1] = { CONFIG_TFFS_MTD_DEVICE - 1 };
static int anzahl = 0;  /*--- default alle ---*/
MODULE_PARM( anzahl, "i" );
#if CONFIG_MTD_AVALANCHE_PARTITIONS == 6
MODULE_PARM_DESC (tffs_mtd, "MTD device used for tiny flash file system (default mtd5)");
#endif
#if CONFIG_MTD_AVALANCHE_PARTITIONS == 4
MODULE_PARM_DESC (tffs_mtd, "MTD device used for tiny flash file system (default mtd3)");
#endif
/*--- MODULE_AUTHOR("Martin Pommerenke (AVM Berlin)  m.pommerenke@avm.de"); ---*/
MODULE_DESCRIPTION("TFFS 'tiny flash file system' driver version 1.2");
MODULE_SUPPORTED_DEVICE("MTD[0..n]");
#define MODULE_VERSION "1.0"
#endif /*--- #else ---*/ /*--- #if defined(CONFIG_TFFS_DOUBLE_BUFFER) ---*/


#ifdef CONFIG_PROC_FS
static struct proc_dir_entry *tffs_proc;
#endif /*--- #ifdef CONFIG_PROC_FS ---*/

/*-----------------------------------------------------------------------------------------------*\
\*-----------------------------------------------------------------------------------------------*/
#if defined(CONFIG_TFFS_MINOR_COMPRESS)
struct _tffs_z {
    z_stream deflate_strm;
    z_stream inflate_strm;
};
struct _tffs_z tffs_z;
static unsigned char tffs_z_buffer[MAX_MINOR_100_BLOCK_SIZE];
#endif /*--- #if defined(CONFIG_TFFS_MINOR_COMPRESS) ---*/
#if defined(CONFIG_TFFS_EXCLUSIVE)
static unsigned char tffs_buffer[MAX_MINOR_100_BLOCK_SIZE];
static unsigned int tffs_buffer_length;
static struct semaphore tffs_buffer_sema;
#endif /*--- #if defined(CONFIG_TFFS_EXCLUSIVE) ---*/

/*-----------------------------------------------------------------------------------------------*\
\*-----------------------------------------------------------------------------------------------*/
static int tffs_open(struct inode *, struct file *);
static int tffs_release(struct inode *, struct file *);
static ssize_t tffs_read(struct file *, char *, size_t, loff_t *);
static ssize_t tffs_write(struct file *, const char *, size_t, loff_t *);
static int tffs_ioctl(struct inode *, struct file *, unsigned int, unsigned long);
static void tffs_cleanup(void);

struct file_operations tffs_fops = {
    owner:   THIS_MODULE,
    open:    tffs_open,
    release: tffs_release,
    read:    tffs_read,
    write:   tffs_write,
    ioctl:   tffs_ioctl,
};


struct _tffs tffs;
static struct semaphore tffs_sema;
static struct semaphore tffs_event_sema;
static struct semaphore tffs_event_lock_sema;
static int tffs_thread_pid;
static int tffs_thread_event;
static DECLARE_COMPLETION(tffs_evt_dead);

enum _tffs_thread_state {
    tffs_thread_state_off,
    tffs_thread_state_init,
    tffs_thread_state_idle,
    tffs_thread_state_process,
    tffs_thread_state_down
};
static enum _tffs_thread_state tffs_thread_state;
unsigned int tffs_request_count = 0;

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
#ifdef CONFIG_PROC_FS
static int tffs_read_proc ( char *page, char **start, off_t off,int count, int *eof, void *data_unused);
static int tffs_write_proc(struct file *file, const char *buffer, unsigned long count, void *data);
#endif /*--- #ifdef CONFIG_PROC_FS ---*/

/*-----------------------------------------------------------------------------------------------*\ 
\*-----------------------------------------------------------------------------------------------*/
int tffs_lock(void) {
    if(down_interruptible(&tffs_sema)) {
        return -1;
    }
    return 0;
}

/*-----------------------------------------------------------------------------------------------*\ 
\*-----------------------------------------------------------------------------------------------*/
void tffs_unlock(void) {
    up(&tffs_sema);
}

/*-----------------------------------------------------------------------------------------------*\ 
\*-----------------------------------------------------------------------------------------------*/
static int tffs_thread(void *reply_data) {
    unsigned int event;
    void *handle;
    tffs_thread_state = tffs_thread_state_init;
	lock_kernel();
	daemonize();
	unlock_kernel();
	strcpy(current->comm, "tffsd_mtd_");
    current->comm[9] = '0' + mtd[0];

    handle = TFFS_Open(0, tffs.anzahl, 0);
    tffs_thread_state = tffs_thread_state_idle;
    for( ;; ) {
        DBG((KERN_INFO "tffsd: wait for events\n"));
        if(down_interruptible(&tffs_event_sema)) {
            break;
        }
        tffs_thread_state = tffs_thread_state_process;

        /*--- lesen des Events ---*/
        down(&tffs_event_lock_sema);
        event = tffs_thread_event;
        tffs_thread_event = 0;
        up(&tffs_event_lock_sema);

        tffs_request_count++;
        /*--- auswerten des Events ---*/
        DBG((KERN_INFO "tffsd: process event 0x%x\n", event));

        if(event & TFFS_EVENT_CLEANUP) {
            DBG((KERN_DEBUG "%s: tffsd: cleanup\n", MODULE_NAME));
            if(down_interruptible(&tffs_sema)) {
                break;
            }
            TFFS_Cleanup(handle);
            up(&tffs_sema);
        }
    }

    TFFS_Close(handle);
    DBG((KERN_INFO "tffsd: event thread dead\n"));
    tffs_thread_state = tffs_thread_state_down;
    complete_and_exit(&tffs_evt_dead, 0);
    return 0;
}

/*-----------------------------------------------------------------------------------------------*\
\*-----------------------------------------------------------------------------------------------*/
void tffs_send_event(unsigned int event) {
    DBG((KERN_INFO "tffs_send_event(%x)\n", event));
    down(&tffs_event_lock_sema);
    tffs_thread_event |= event;
    up(&tffs_event_lock_sema);
    up(&tffs_event_sema);
}

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
#if defined(CONFIG_TFFS_MINOR_COMPRESS)
void *tffs_alloc(voidpf opaque, uInt items, uInt size) {
    void *ptr;
    /*--- printk(KERN_ERR "tffs_alloc(0x%x, 0x%x, 0x%x):", opaque, items, size); ---*/
    ptr = kmalloc(items * size, GFP_KERNEL);
    if(ptr) {
        memset(ptr, 0, items * size);
        /*--- printk(KERN_ERR " ptr=0x%x\n", ptr); ---*/
    } else {
        printk(KERN_ERR " failed\n");
    }
    return ptr;
}

void tffs_free(voidpf opaque, voidpf address) {
    /*--- printk(KERN_ERR "tffs_free(0x%x, 0x%x)\n", opaque, address); ---*/
    kfree(address);
}
#endif /*--- #if defined(CONFIG_TFFS_MINOR_COMPRESS) ---*/

/*-----------------------------------------------------------------------------------------------*\
\*-----------------------------------------------------------------------------------------------*/
int __init tffs_init(void) {
    static char tffs_mtd[6] = "mtd_";


    DBG((KERN_INFO "%s: register_chrdev(%s) param=mtd%u\n", MODULE_NAME, MODULE_NAME, mtd[0]));
    tffs.major = register_chrdev(0 /*--- dynamic major ---*/, MODULE_NAME, &tffs_fops);
    if(tffs.major < 1) {
        DBG((KERN_ERR "%s: register_chrdrv failed: reason %d\n", MODULE_NAME, tffs.major));
        return -ERESTARTSYS;
    }

	tffs.devfs_dir_handle = devfs_mk_dir(NULL, "tffs", NULL);
    tffs_mtd[3]  = mtd[0] + '0';
    tffs.minor   = 0;
    tffs.anzahl  = anzahl;
	tffs.devfs_handle = devfs_register(tffs.devfs_dir_handle, tffs_mtd, DEVFS_FL_DEFAULT, tffs.major, tffs.minor,
                                	   S_IFCHR | S_IRUGO | S_IWUSR, &tffs_fops, NULL);

#if defined(CONFIG_TFFS_DOUBLE_BUFFER)
    if(TFFS_Init(mtd[0], mtd[1])) {
        tffs_cleanup();
        return -ERESTARTSYS;
    }
#else /*--- #if defined(CONFIG_TFFS_DOUBLE_BUFFER) ---*/
    if(TFFS_Init(mtd[0])) {
        tffs_cleanup();
        return -ERESTARTSYS;
    }
#endif /*--- #else ---*/ /*--- #if defined(CONFIG_TFFS_DOUBLE_BUFFER) ---*/

#if defined(CONFIG_TFFS_EXCLUSIVE)
    sema_init(&tffs_buffer_sema, 1);
#endif /*--- #if defined(CONFIG_TFFS_EXCLUSIVE) ---*/

    sema_init(&tffs_sema, 1);
    sema_init(&tffs_event_sema, 0);
    sema_init(&tffs_event_lock_sema, 1);

	tffs.devfs_ticfg_handle = devfs_register(NULL, "ticfg", DEVFS_FL_DEFAULT, tffs.major, FLASH_FS_ID_TICFG,
                                	   S_IFCHR | S_IRUGO | S_IWUSR, &tffs_fops, NULL);

	/*--- tffs.devfs_shutdown_handle = devfs_register(NULL, "shutdown_cmd", DEVFS_FL_DEFAULT, tffs.major, FLASH_FS_ID_SHUTDOWN, ---*/
                                	   /*--- S_IFCHR | S_IRUGO | S_IWUSR, &tffs_fops, NULL); ---*/

    DBG((KERN_INFO "%s: major %d (success)\n", MODULE_NAME, tffs.major));

#ifdef CONFIG_PROC_FS
	if ((tffs_proc = create_proc_entry( "tffs", 0, 0 )))
	  tffs_proc->read_proc = tffs_read_proc;
	  tffs_proc->write_proc = tffs_write_proc;
#endif

    tffs_thread_state = tffs_thread_state_off;
	tffs_thread_pid = kernel_thread(tffs_thread, &tffs_thread_event, CLONE_SIGHAND);

#if defined(CONFIG_TFFS_MINOR_COMPRESS)
    memset(&tffs_z.inflate_strm, 0, sizeof(tffs_z.inflate_strm));
    tffs_z.inflate_strm.data_type = Z_ASCII;
    tffs_z.inflate_strm.zalloc    = tffs_alloc;
    tffs_z.inflate_strm.opaque    = Z_NULL;
    tffs_z.inflate_strm.zfree     = tffs_free;

    memset(&tffs_z.deflate_strm, 0, sizeof(tffs_z.deflate_strm));
    tffs_z.deflate_strm.data_type = Z_ASCII;
    tffs_z.deflate_strm.zalloc    = tffs_alloc;
    tffs_z.deflate_strm.opaque    = Z_NULL;
    tffs_z.deflate_strm.zfree     = tffs_free;
#endif /*--- #if defined(CONFIG_TFFS_MINOR_COMPRESS) ---*/
    printk("TFFS: tiny flash file system driver. (c) AVM Berlin, M.Pommerenke@AVM.de\n");
#if defined(CONFIG_TFFS_DOUBLE_BUFFER) 
    printk("      mount on mtd%u and mtd%u (double buffering)\n", mtd[0], mtd[1]);
#else /*--- #if defined(CONFIG_TFFS_DOUBLE_BUFFER) ---*/ 
    printk("      mount on mtd%u (single buffering)\n", mtd[0]);
#endif /*--- #else ---*/ /*--- #if defined(CONFIG_TFFS_DOUBLE_BUFFER) ---*/ 

#ifdef CONFIG_SYSCTL
    adam2_env_init();
#endif /* CONFIG_SYSCTL */
    return 0;
}


module_init(tffs_init);

/*-----------------------------------------------------------------------------------------------*\
\*-----------------------------------------------------------------------------------------------*/
static void tffs_cleanup(void) {
#if defined(MODLE)
    DBG((KERN_INFO "%s: unregister_chrdev(%u)\n", MODULE_NAME, tffs.major));
    if(down_interruptible(&tffs_sema)) {
        DBG((KERN_ERR "%s: tffs_cleanup: down_interruptible() failed\n", MODULE_NAME));
        return;
    }
#ifdef CONFIG_SYSCTL
    adam2_env_exit();
#endif /* CONFIG_SYSCTL */

    if(tffs_thread_pid) {
        int stat;
        DBG((KERN_INFO "%s: Terminating thread ", MODULE_NAME));
		stat = kill_proc(tffs_thread_pid, SIGTERM, 1);
		if(!stat) {
			DBG(("waiting..."));
			wait_for_completion(&tffs_evt_dead);
		}
		DBG(("done.\n"));
    }

    TFFS_Deinit();
    if(tffs_proc)
         remove_proc_entry( "tffs", 0);
    if(tffs.devfs_handle)
	    devfs_unregister(tffs.devfs_handle);
    if(tffs.devfs_ticfg_handle)
	    devfs_unregister(tffs.devfs_ticfg_handle);
    /*--- if(tffs.devfs_shutdown_handle) ---*/
	    /*--- devfs_unregister(tffs.devfs_shutdown_handle); ---*/
	if(tffs.devfs_dir_handle)
        unregister_chrdev(tffs.major, MODULE_NAME);
    up(&tffs_sema);
#endif /*--- #if defined(MODLE) ---*/
    return;
}

module_exit(tffs_cleanup);

/*-----------------------------------------------------------------------------------------------*\
\*-----------------------------------------------------------------------------------------------*/
static int tffs_open(struct inode *inode, struct file *filp) {
    int minor = inode->i_rdev & 0xFF;

    DBG((KERN_INFO "%s: tffs_open: MAJOR %u MINOR %u\n", MODULE_NAME, (inode->i_rdev >> 8), minor));

    if(down_interruptible(&tffs_sema)) {
        DBG((KERN_ERR "%s tffs_open: down_interruptible() failed\n", MODULE_NAME));
        return -ERESTARTSYS;
    }
    if(!filp->private_data) {
        filp->private_data = TFFS_Open(0, tffs.anzahl, minor);
    }
    if(!filp->private_data) {
        DBG((KERN_ERR "%s: tffs_open: TFFS_Open failed\n", MODULE_NAME));
        up(&tffs_sema);
        return -EFAULT;
    }
    if(filp->f_flags & O_APPEND) {
        DBG((KERN_ERR "%s: tffs_open: TFFS_Open O_APPEND not supported\n", MODULE_NAME));
        up(&tffs_sema);
        return -EFAULT;
    }
#if defined(CONFIG_TFFS_EXCLUSIVE)
    if((filp->f_flags & O_RDONLY) && (filp->f_flags & O_WRONLY)) {
        DBG(("%s: tffs_open: open O_RDONLY and O_WRONLY simultary not supported\n", MODULE_NAME));
        up(&tffs_sema);
        return -EFAULT;
    }
#endif /*--- #if defined(CONFIG_TFFS_EXCLUSIVE) ---*/

    MOD_INC_USE_COUNT;

    DBG((KERN_INFO "%s: tffs_open: TFFS_Open success flags=0x%x\n", MODULE_NAME, filp->f_flags));

    up(&tffs_sema);

#if defined(CONFIG_TFFS_EXCLUSIVE)
    if(minor) {
        if(down_interruptible(&tffs_buffer_sema)) {
            DBG((KERN_ERR "%s tffs_open: down_interruptible(&tffs_buffer_sema) failed\n", MODULE_NAME));
            return -ERESTARTSYS;
        }
        DBG((KERN_WARNING "%s tffs_open: got 'tffs_buffer_sema' for %s\n", MODULE_NAME, (filp->f_flags & O_WRONLY) ? "write" : "read"));
        tffs_buffer_length = 0;
    }
#endif /*--- #if defined(CONFIG_TFFS_EXCLUSIVE) ---*/
    return 0;
}

/*-----------------------------------------------------------------------------------------------*\
\*-----------------------------------------------------------------------------------------------*/
static int tffs_release(struct inode *inode, struct file *filp) {
    unsigned int minor = TFFS_Minor(filp->private_data);
    if(down_interruptible(&tffs_sema)) {
#if defined(CONFIG_TFFS_EXCLUSIVE)
        if(minor) {
            up(&tffs_buffer_sema);
            DBG((KERN_WARNING "%s tffs_release(error): 'tffs_buffer_sema' released\n", MODULE_NAME));
        }
#endif /*--- #if defined(CONFIG_TFFS_EXCLUSIVE) ---*/
        printk(KERN_ERR "%s tffs_release: down_interruptible() failed\n", MODULE_NAME);
        return -ERESTARTSYS;
    }
    if(minor && (filp->f_flags & O_WRONLY)) {
        unsigned int *write_buffer_length = NULL;
#if defined(CONFIG_TFFS_EXCLUSIVE)
        char *write_buffer;
        if(tffs_buffer_length) {
            write_buffer  =  tffs_buffer;
            write_buffer_length = &tffs_buffer_length;
        } else {
            write_buffer  =  NULL;
        }
#else /*--- #if defined(CONFIG_TFFS_EXCLUSIVE) ---*/
        char *write_buffer = TFFS_Write_Buffer(filp->private_data, &write_buffer_length);
#endif /*--- #else ---*/ /*--- #if defined(CONFIG_TFFS_EXCLUSIVE) ---*/
        if(write_buffer) {
            DBG((KERN_INFO "%s write on close: %u bytes\n", MODULE_NAME, *write_buffer_length));
#if defined(CONFIG_TFFS_MINOR_COMPRESS)
            {

                /*--------------------------------------------------------------------------*\
                   Z_FINISH can be used immediately after deflateInit if all the compression
                  is to be done in a single step. In this case, avail_out must be at least
                  0.1% larger than avail_in plus 12 bytes.  If deflate does not return
                  Z_STREAM_END, then it must be called again as described above.
                \*--------------------------------------------------------------------------*/
                int status;

                /*--------------------------------------------------------------------------*\
                \*--------------------------------------------------------------------------*/
                status = deflateInit2 (&tffs_z.deflate_strm,
                                       Z_DEFAULT_COMPRESSION,
                                       Z_DEFLATED,
                                       9, /*--- (fast) smallest buffers ---*/
                                       2, /*--- use little memory ---*/
                                       Z_DEFAULT_STRATEGY 
                                      );
                /*--- status = deflateInit(&tffs_z.deflate_strm, Z_DEFAULT_COMPRESSION); ---*/
                if(status == Z_OK) {
                    DBG((KERN_INFO "deflateInit: Z_OK: ...\n"));
                    tffs_z.deflate_strm.avail_in  = *write_buffer_length;
                    tffs_z.deflate_strm.total_in  = 0;
                    tffs_z.deflate_strm.avail_out = sizeof(tffs_z_buffer);
                    tffs_z.deflate_strm.total_out = 0;
                    tffs_z.deflate_strm.next_in   = write_buffer;
                    tffs_z.deflate_strm.next_out  = tffs_z_buffer;

                    DBG((KERN_INFO "deflate(start): avail_in=%u total_in=%u avail_out=%u total_out=%u\n",
                            tffs_z.deflate_strm.avail_in,
                            tffs_z.deflate_strm.total_in,
                            tffs_z.deflate_strm.avail_out,
                            tffs_z.deflate_strm.total_out
                    ));

                    /*-----------------------------------------------------------------------*\
                      If the parameter flush is set to Z_FINISH, pending input is processed,
                      pending output is flushed and deflate returns with Z_STREAM_END if there
                      was enough output space; if deflate returns with Z_OK, this function must be
                      called again with Z_FINISH and more output space (updated avail_out) but no
                      more input data, until it returns with Z_STREAM_END or an error. After
                      deflate has returned Z_STREAM_END, the only possible operations on the
                      stream are deflateReset or deflateEnd.
                    \*-----------------------------------------------------------------------*/
                    status = deflate(&tffs_z.deflate_strm, Z_FINISH);

                    DBG((KERN_INFO "deflate(done): avail_in=%u total_in=%u avail_out=%u total_out=%u\n",
                            tffs_z.deflate_strm.avail_in,
                            tffs_z.deflate_strm.total_in,
                            tffs_z.deflate_strm.avail_out,
                            tffs_z.deflate_strm.total_out
                    ));

                    DBG((KERN_INFO "deflate: status=0x%x\n", status));
                    if(status == Z_OK) {
                        printk(KERN_ERR "deflate(1): status=Z_OK (%u) (Z_STREAM_END expected) internel Error\n", status);
                        status = Z_MEM_ERROR;
                    }
                    deflateEnd(&tffs_z.deflate_strm);
                }
                DBG((KERN_INFO "%s: deflate(status=%u): in %u bytes --> out %u bytes\n", MODULE_NAME, -status, *write_buffer_length, tffs_z.deflate_strm.total_out));

                /*--------------------------------------------------------------------------*\
                \*--------------------------------------------------------------------------*/
                if(status >= 0) {
                    TFFS_Write(filp->private_data, minor, tffs_z_buffer, tffs_z.deflate_strm.total_out, 0);
                }
                *write_buffer = 0;
            }
#else /*--- #if defined(CONFIG_TFFS_MINOR_COMPRESS) ---*/
            /*--- ignore return value ---*/
            TFFS_Write(filp->private_data, minor, write_buffer, *write_buffer_length, 0);
            *write_buffer = 0;
#endif /*--- #else ---*/ /*--- #if defined(CONFIG_TFFS_MINOR_COMPRESS) ---*/
        }
    }

    TFFS_Close(filp->private_data);
    up(&tffs_sema);
#if defined(CONFIG_TFFS_EXCLUSIVE)
    if(minor) {
        up(&tffs_buffer_sema);
        DBG((KERN_WARNING "%s tffs_release: 'tffs_buffer_sema' released\n", MODULE_NAME));
    }
#endif /*--- #if defined(CONFIG_TFFS_EXCLUSIVE) ---*/
    MOD_DEC_USE_COUNT;
    return 0;
}

/*-----------------------------------------------------------------------------------------------*\
\*-----------------------------------------------------------------------------------------------*/
static ssize_t tffs_read(struct file *filp, char *read_buffer, size_t max_read_length, loff_t *offp) {
    unsigned int minor = TFFS_Minor(filp->private_data);
    unsigned int status;
    unsigned int *raw_read_buffer_length;
    unsigned char *raw_read_buffer;
    unsigned int copy_length = 0;

    if(minor == 0) {
        DBG((KERN_ERR "tffs_read: no read supported on minor 0\n"));
        return -EOPNOTSUPP;
    }

    /*-------------------------------------------------------------------------------------------*\
     *  aktuellen read_buffer holen 
    \*-------------------------------------------------------------------------------------------*/
#if defined(CONFIG_TFFS_EXCLUSIVE)
    if(tffs_buffer_length) {
        raw_read_buffer_length = &tffs_buffer_length;
        raw_read_buffer        =  tffs_buffer;
    } else {
        raw_read_buffer        = NULL; /*--- lesen aus TFFS wird getriggert ---*/
        raw_read_buffer_length = NULL;
    }
#else /*--- #if defined(CONFIG_TFFS_EXCLUSIVE) ---*/
    raw_read_buffer_length = NULL;
    raw_read_buffer = TFFS_Read_Buffer(filp->private_data, &raw_read_buffer_length);
#endif /*--- #else ---*/ /*--- #if defined(CONFIG_TFFS_EXCLUSIVE) ---*/

    /*-------------------------------------------------------------------------------------------*\
     *  es gab keinen aktuellen read_buffer 
    \*-------------------------------------------------------------------------------------------*/
    if(raw_read_buffer == NULL) { /*--- noch keiner vorhanden ---*/
        DBG((KERN_INFO "tffs_read: new buffer current offset pos = %u\n", *offp));
#if defined(CONFIG_TFFS_EXCLUSIVE)
        raw_read_buffer_length = &tffs_buffer_length;
        raw_read_buffer        =  tffs_buffer;
#else /*--- #if defined(CONFIG_TFFS_EXCLUSIVE) ---*/
        raw_read_buffer_length = (unsigned int *)MAX_MINOR_100_BLOCK_SIZE;

        /*---------------------------------------------------------------------------------------*\
        *  read_buffer holen mit maximal lnge
        \*---------------------------------------------------------------------------------------*/
        raw_read_buffer = TFFS_Read_Buffer(filp->private_data, &raw_read_buffer_length);

        DBG((KERN_INFO "%s: tffs_read: alloc(%u) new raw_read_buffer\n", MODULE_NAME, MAX_MINOR_100_BLOCK_SIZE));

        if(raw_read_buffer == NULL) { /*--- keiner vorhanden system fehler ---*/
            printk(KERN_ERR "%s: tffs_read: alloc(%u) failed\n", MODULE_NAME, MAX_MINOR_100_BLOCK_SIZE);
            return -EFAULT;
        }
#endif /*--- #else ---*/ /*--- #if defined(CONFIG_TFFS_EXCLUSIVE) ---*/
        *offp = 0; /*--- am begin des Files ---*/

        /*---------------------------------------------------------------------------------------*\
        *  'id' in read_buffer lesen (komplett)
        \*---------------------------------------------------------------------------------------*/
        DBG((KERN_DEBUG "%s: tffs_read(max %u)\n", MODULE_NAME, MAX_MINOR_100_BLOCK_SIZE));

        if(down_interruptible(&tffs_sema)) {
            printk(KERN_ERR "%s: tffs_read: down_interruptible() failed\n", MODULE_NAME);
            return -ERESTARTSYS;
        }
#if defined(CONFIG_TFFS_MINOR_COMPRESS)
        {
            unsigned int z_length = sizeof(tffs_z_buffer); /*--- maximal groesse setzen ---*/

            status = TFFS_Read(filp->private_data, minor, tffs_z_buffer, &z_length);
            if(status == 0) {
                status = inflateInit2(&tffs_z.inflate_strm, 9);
                if(status == Z_OK) {
                    tffs_z.inflate_strm.avail_in  = z_length;
                    tffs_z.inflate_strm.total_in  = 0;
                    tffs_z.inflate_strm.avail_out = MAX_MINOR_100_BLOCK_SIZE;
                    tffs_z.inflate_strm.total_out = 0;
                    tffs_z.inflate_strm.next_in   = tffs_z_buffer;
                    tffs_z.inflate_strm.next_out  = raw_read_buffer;

                    DBG((KERN_INFO "inflate(start): avail_in=%u total_in=%u avail_out=%u total_out=%u\n",
                            tffs_z.inflate_strm.avail_in,
                            tffs_z.inflate_strm.total_in,
                            tffs_z.inflate_strm.avail_out,
                            tffs_z.inflate_strm.total_out
                    ));

                    /*-------------------------------------------------------------------------*\
                      inflate() should normally be called until it returns Z_STREAM_END or an
                      error. However if all decompression is to be performed in a single step
                      (a single call of inflate), the parameter flush should be set to
                      Z_FINISH. In this case all pending input is processed and all pending
                      output is flushed; avail_out must be large enough to hold all the
                      uncompressed data. (The size of the uncompressed data may have been saved
                      by the compressor for this purpose.) The next operation on this stream must
                      be inflateEnd to deallocate the decompression state. The use of Z_FINISH
                      is never required, but can be used to inform inflate that a faster routine
                      may be used for the single inflate() call.
                    \*-------------------------------------------------------------------------*/
                    status = inflate(&tffs_z.inflate_strm, Z_FINISH);

                    DBG((KERN_INFO "inflate(done): avail_in=%u total_in=%u avail_out=%u total_out=%u\n",
                            tffs_z.inflate_strm.avail_in,
                            tffs_z.inflate_strm.total_in,
                            tffs_z.inflate_strm.avail_out,
                            tffs_z.inflate_strm.total_out
                    ));

                    inflateEnd(&tffs_z.inflate_strm);
                }
                if(status == Z_STREAM_END) {
                    *raw_read_buffer_length = tffs_z.inflate_strm.total_out; /*--- maximal groesse setzen ---*/
                    status = 0;
                } else {
                    *raw_read_buffer_length = 0;
                    status = (unsigned int)-1;
                }
                DBG((KERN_INFO "%s: inflate(status=%u): in %u bytes --> out %u bytes\n", MODULE_NAME, status, z_length, tffs_z.inflate_strm.total_out));
            }
        }
#else /*--- #if defined(CONFIG_TFFS_MINOR_COMPRESS) ---*/
        *raw_read_buffer_length = MAX_MINOR_100_BLOCK_SIZE; /*--- maximal groesse setzen ---*/
        status = TFFS_Read(filp->private_data, minor, raw_read_buffer, raw_read_buffer_length);
#endif /*--- #else ---*/ /*--- #if defined(CONFIG_TFFS_MINOR_COMPRESS) ---*/
        up(&tffs_sema);
        tffs_request_count++;

    } else {
        status = 0;
    }

    if(status == 0) {
        copy_length = min(max_read_length, *raw_read_buffer_length - (unsigned int)*offp);
        DBG((KERN_INFO "%s: tffs_read: copy_length %u (max_read_length %u, *offp %u, MAX %u)\n", 
                MODULE_NAME, copy_length, max_read_length, *offp, MAX_MINOR_100_BLOCK_SIZE));

        if(!copy_to_user(read_buffer, raw_read_buffer + *offp, copy_length)) {
            *offp += copy_length;
            DBG((KERN_INFO "%s: tffs_read: %u bytes status %d\n", MODULE_NAME, copy_length, status));
            return (int)copy_length;
        }
        printk(KERN_ERR "%s: tffs_read: copy_to_user failed\n", MODULE_NAME);
    } else {
        DBG((KERN_INFO "%s: tffs_read: %u bytes status %d\n", MODULE_NAME, copy_length, status));
    }
    return (unsigned int)-EFAULT;
}

/*-----------------------------------------------------------------------------------------------*\
\*-----------------------------------------------------------------------------------------------*/
static ssize_t tffs_write(struct file *filp, const char *write_buffer, size_t write_length, loff_t *offp) {
    unsigned int minor = TFFS_Minor(filp->private_data);
    unsigned char *raw_write_buffer;
    unsigned int *raw_write_buffer_length;
    unsigned int copy_length;

    DBG((KERN_DEBUG "%s: tffs_write(%u bytes)\n", MODULE_NAME, write_length));

    if(minor == 0) {
        DBG((KERN_ERR "tffs_write: no read supported on minor 0\n"));
        return -EOPNOTSUPP;
    }
    if(down_interruptible(&tffs_sema)) {
        printk(KERN_ERR "%s: tffs_write: down_interruptible() failed\n", MODULE_NAME);
        return -ERESTARTSYS;
    }

    /*-------------------------------------------------------------------------------------------*\
     *  aktuellen write_buffer holen 
    \*-------------------------------------------------------------------------------------------*/
#if defined(CONFIG_TFFS_EXCLUSIVE)
    raw_write_buffer_length = &tffs_buffer_length;
    raw_write_buffer        =  tffs_buffer;
#else /*--- #if defined(CONFIG_TFFS_EXCLUSIVE) ---*/
    raw_write_buffer_length = NULL;
    raw_write_buffer = TFFS_Write_Buffer(filp->private_data, &raw_write_buffer_length);

    /*-------------------------------------------------------------------------------------------*\
     *  es gab keinen aktuellen write_buffer 
    \*-------------------------------------------------------------------------------------------*/
    if(raw_write_buffer == NULL) { /*--- noch keiner vorhanden ---*/
        raw_write_buffer_length = (unsigned int *)MAX_MINOR_100_BLOCK_SIZE;
        *offp = 0; /*--- am begin des Files ---*/
        raw_write_buffer = TFFS_Write_Buffer(filp->private_data, &raw_write_buffer_length);

        DBG((KERN_INFO "%s: tffs_write: alloc(%u) new raw_write_buffer\n", MODULE_NAME, MAX_MINOR_100_BLOCK_SIZE));

        if(raw_write_buffer == NULL) { /*--- immernoch keiner vorhanden ---*/
            printk(KERN_ERR "%s: tffs_write: alloc(%u) failed\n", MODULE_NAME, write_length);
            up(&tffs_sema);
            return -EFAULT;
        }
    }
#endif /*--- #else ---*/ /*--- #if defined(CONFIG_TFFS_EXCLUSIVE) ---*/

    copy_length = min(write_length, MAX_MINOR_100_BLOCK_SIZE - (unsigned int)*offp);
    DBG((KERN_INFO "%s: tffs_write: copy_length %u (write_length %u, *offp %u, MAX %u)\n", 
            MODULE_NAME, copy_length, write_length, *offp, MAX_MINOR_100_BLOCK_SIZE));

    if(copy_from_user(raw_write_buffer + *offp, write_buffer, copy_length)) {
        printk(KERN_ERR "%s: tffs_write: copy_from_user failed (copy_length=%u)\n", MODULE_NAME, copy_length);
        up(&tffs_sema);
        return -EFAULT;
    }
    *offp                    += copy_length;
    *raw_write_buffer_length += copy_length;

    DBG((KERN_INFO "%s: tffs_write: %u bytes\n", MODULE_NAME, write_length));

    up(&tffs_sema);
    return (ssize_t)copy_length;
}

/*-----------------------------------------------------------------------------------------------*\
\*-----------------------------------------------------------------------------------------------*/
static int tffs_ioctl(struct inode *inode, struct file *filp, unsigned int ioctl_param, unsigned long argv) {
    struct _tffs_cmd tffs_cmd;
    unsigned char *buffer;
    unsigned int result_size = sizeof(tffs_cmd);
    unsigned int number = _IOC_NR(ioctl_param);
    unsigned int minor  = TFFS_Minor(filp->private_data);

    DBG((KERN_INFO "%s: tffs_ioctl: type %u number %u size %u\n", MODULE_NAME, _IOC_TYPE(ioctl_param), number, _IOC_SIZE(ioctl_param)));

    if(minor != 0) {
        /*--- printk(KERN_WARNING "%s: tffs_ioctl: ioctl not supported by minor != 0\n", MODULE_NAME); ---*/
        return -EOPNOTSUPP;
    }

    if(argv == 0) {
        printk(KERN_ERR "%s: tffs_ioctl: no data pointer for cmd number %u\n", MODULE_NAME, number);
        return -EFAULT;
    }
    if(copy_from_user(&tffs_cmd, (const void *)argv, sizeof(tffs_cmd))) {
        printk(KERN_ERR "%s: tffs_ioctl: copy_from_user failed\n", MODULE_NAME);
        return -EFAULT;
    }

    switch(number) {
        case _TFFS_READ_ID:
            if((filp->f_flags & O_ACCMODE) == O_WRONLY) {
                printk(KERN_ERR "%s: tffs_ioctl: read failed: flags=O_WRONLY\n", MODULE_NAME);
                return -EFAULT;
            }
            buffer = kmalloc(tffs_cmd.size, GFP_KERNEL);
            if(!buffer) {
                printk(KERN_ERR "%s: tffs_ioctl: alloc(%u) failed\n", MODULE_NAME, tffs_cmd.size);
                return -EFAULT;
            }
            DBG((KERN_DEBUG "%s: tffs_ioctl: read \n", MODULE_NAME));

            if(down_interruptible(&tffs_sema)) {
                kfree(buffer);
                printk(KERN_ERR "%s: tffs_ioctl(READ): down_interruptible() failed\n", MODULE_NAME);
                return -ERESTARTSYS;
            }
            tffs_cmd.status = TFFS_Read(filp->private_data, tffs_cmd.id, buffer, &tffs_cmd.size);
            up(&tffs_sema);
            tffs_request_count++;

            if(tffs_cmd.status == 0) {
                if(copy_to_user((void *)tffs_cmd.buffer, buffer, (unsigned long)tffs_cmd.size)) {
                    kfree(buffer);
                    printk(KERN_ERR "%s: tffs_ioctl: copy_to_user failed\n", MODULE_NAME);
                    return -EFAULT;
                }
                result_size += tffs_cmd.size;
            }
            kfree(buffer);
            break;

        case _TFFS_WRITE_ID:
            if((filp->f_flags & O_ACCMODE) == O_RDONLY) {
                printk(KERN_ERR "%s: tffs_ioctl: write failed: flags=O_RDONLY\n", MODULE_NAME);
                return -EFAULT;
            }
            buffer = kmalloc(tffs_cmd.size, GFP_KERNEL);
            if(!buffer) {
                printk(KERN_ERR "%s: tffs_ioctl: alloc(%u) failed\n", MODULE_NAME, tffs_cmd.size);
                return -EFAULT;
            }
            if(copy_from_user(buffer, (void *)tffs_cmd.buffer, (unsigned long)tffs_cmd.size)) {
                kfree(buffer);
                printk(KERN_ERR "%s: tffs_ioctl: copy_from_user failed\n", MODULE_NAME);
                return -EFAULT;
            }
            DBG((KERN_DEBUG "%s: tffs_ioctl: write\n", MODULE_NAME));

            if(down_interruptible(&tffs_sema)) {
                kfree(buffer);
                printk(KERN_ERR "%s: tffs_ioctl(WRITE): down_interruptible() failed\n", MODULE_NAME);
                return -ERESTARTSYS;
            }
            tffs_cmd.status = TFFS_Write(filp->private_data, tffs_cmd.id, buffer, tffs_cmd.size, 0);
            up(&tffs_sema);
            tffs_request_count++;

            kfree(buffer);
            break;

        case _TFFS_FORMAT:
            DBG((KERN_DEBUG "%s: tffs_ioctl: format\n", MODULE_NAME));
            if(down_interruptible(&tffs_sema)) {
                printk(KERN_ERR "%s: tffs_ioctl(FORMAT): down_interruptible() failed\n", MODULE_NAME);
                return -ERESTARTSYS;
            }
            tffs_cmd.status = TFFS_Format(filp->private_data);
            up(&tffs_sema);
            tffs_request_count++;
            break;

        case _TFFS_WERKSEINSTELLUNG:
            DBG((KERN_DEBUG "%s: tffs_ioctl: format\n", MODULE_NAME));
            if(down_interruptible(&tffs_sema)) {
                printk(KERN_ERR "%s: tffs_ioctl(WERKSEINSTELLUNG): down_interruptible() failed\n", MODULE_NAME);
                return -ERESTARTSYS;
            }
            tffs_cmd.status = TFFS_Werkseinstellungen(filp->private_data);
            up(&tffs_sema);
            tffs_request_count++;
            break;

        case _TFFS_CLEAR_ID:
            DBG((KERN_DEBUG "%s: tffs_ioctl: clear id\n", MODULE_NAME));
            if(down_interruptible(&tffs_sema)) {
                printk(KERN_ERR "%s: tffs_ioctl(CLEAR_ID): down_interruptible() failed\n", MODULE_NAME);
                return -ERESTARTSYS;
            }
            tffs_cmd.status = TFFS_Clear(filp->private_data, tffs_cmd.id);
            up(&tffs_sema);
            tffs_request_count++;
            break;

        case _TFFS_CLEANUP:
            DBG((KERN_DEBUG "%s: tffs_ioctl: cleanup\n", MODULE_NAME));
            if(down_interruptible(&tffs_sema)) {
                printk(KERN_ERR "%s: tffs_ioctl(CLEANUP): down_interruptible() failed\n", MODULE_NAME);
                return -ERESTARTSYS;
            }
            tffs_cmd.status = TFFS_Cleanup(filp->private_data);
            up(&tffs_sema);
            tffs_request_count++;
            break;


        case _TFFS_REINDEX:
            DBG((KERN_DEBUG "%s: tffs_ioctl: reindex\n", MODULE_NAME));
            if(down_interruptible(&tffs_sema)) {
                printk(KERN_ERR "%s: tffs_ioctl(REINDEX): down_interruptible() failed\n", MODULE_NAME);
                return -ERESTARTSYS;
            }
            tffs_cmd.status = TFFS_Create_Index(filp->private_data);
            up(&tffs_sema);
            tffs_request_count++;
            break;

        case _TFFS_INFO:
            DBG((KERN_DEBUG "%s: tffs_ioctl: info\n", MODULE_NAME));
            if(down_interruptible(&tffs_sema)) {
                printk(KERN_ERR "%s: tffs_ioctl(INFO): down_interruptible() failed\n", MODULE_NAME);
                return -ERESTARTSYS;
            }
            tffs_cmd.status = TFFS_Info(filp->private_data, &tffs_cmd.id);
            up(&tffs_sema);
            tffs_request_count++;
            break;

        default:
            printk(KERN_ERR "%s: tffs_ioctl: unknwon\n", MODULE_NAME);
	        return -EINVAL;
    }

    if(copy_to_user((void *)argv, &tffs_cmd, sizeof(tffs_cmd))) {
        printk(KERN_ERR "%s: tffs_ioctl: copy_to_user failed\n", MODULE_NAME);
        return -EFAULT;
    }
    return 0;
}

/*-----------------------------------------------------------------------------------------------*\
\*-----------------------------------------------------------------------------------------------*/
#ifdef CONFIG_PROC_FS
unsigned int tffs_info_value;
static int tffs_read_proc ( char *page, char **start, off_t off,int count, int *eof, void *data_unused) {
	int     len   = 0;
    off_t   begin = 0;

    if(tffs_lock()) {
        printk("tffs_read_proc: lock() failed\n");
        return -EINVAL;
    }

	len = sprintf(page + len, "TFFS \n");
    if(len + begin > off + count)
        goto tffs_read_proc_done;
    if(len + begin < off) {
        begin += len;
        len = 0;
    }

	len += sprintf(page + len, "mount=mtd%u\n", mtd[0]);
    if(len + begin > off + count)
        goto tffs_read_proc_done;
    if(len + begin < off) {
        begin += len;
        len = 0;
    }

#if defined(CONFIG_TFFS_DOUBLE_BUFFER)
	len += sprintf(page + len, "mount=mtd%u\n", mtd[1]);
    if(len + begin > off + count)
        goto tffs_read_proc_done;
    if(len + begin < off) {
        begin += len;
        len = 0;
    }
#endif /*--- #if defined(CONFIG_TFFS_DOUBLE_BUFFER) ---*/

	len += sprintf(page + len, "request=%u\n", tffs_request_count);
    if(len + begin > off + count)
        goto tffs_read_proc_done;
    if(len + begin < off) {
        begin += len;
        len = 0;
    }

    if(tffs_info_value) {
	    len += sprintf(page + len, "fill=%u\n", tffs_info_value);
        if(len + begin > off + count)
            goto tffs_read_proc_done;
        if(len + begin < off) {
            begin += len;
            len = 0;
        }
    }

    if(tffs_thread_event) {
	    len += sprintf(page + len, "event panding\n");
        if(len + begin > off + count)
            goto tffs_read_proc_done;
        if(len + begin < off) {
            begin += len;
            len = 0;
        }
    }

#if defined(CONFIG_TFFS_EXCLUSIVE)
    len += sprintf(page + len, "mode: read/write: exclusive\n");
#else /*--- #if defined(CONFIG_TFFS_EXCLUSIVE) ---*/
    len += sprintf(page + len, "mode: read/write: sheared\n");
#endif /*--- #else ---*/ /*--- #if defined(CONFIG_TFFS_EXCLUSIVE) ---*/

    if(len + begin > off + count)
        goto tffs_read_proc_done;
    if(len + begin < off) {
        begin += len;
        len = 0;
    }

    switch(tffs_thread_state) {
        case tffs_thread_state_off:
	        len += sprintf(page + len, "thread state=off\n");
            break;
        case tffs_thread_state_init:
	        len += sprintf(page + len, "thread state=init\n");
            break;
        case tffs_thread_state_idle:
	        len += sprintf(page + len, "thread state=idle\n");
            break;
        case tffs_thread_state_process:
	        len += sprintf(page + len, "thread state=process\n");
            break;
        case tffs_thread_state_down:
	        len += sprintf(page + len, "thread state=down\n");
            break;
    }
    if(len + begin > off + count)
        goto tffs_read_proc_done;
    if(len + begin < off) {
        begin += len;
        len = 0;
    }
    *eof = 1;

tffs_read_proc_done:
    tffs_unlock();

    if (off >= len + begin)
            return 0;
    *start = page + (begin - off);
    return ((count < begin + len - off) ? count : begin + len - off);
}

/*-----------------------------------------------------------------------------------------------*\
\*-----------------------------------------------------------------------------------------------*/
static int tffs_atoi(char *p) {
    int value = 0;
    while(*p && *p >= '0' && *p <= '9') {
        value *= 10;
        value += *p - '0';
        p++;
    }
    return value;
}

/*-----------------------------------------------------------------------------------------------*\
\*-----------------------------------------------------------------------------------------------*/
static int tffs_write_proc(struct file *file, const char *buffer, unsigned long count, void *data) {
    char page[80];
    char *p = page;
    int type = -1;
    void *Handle;
    int status = 0;


	if( copy_from_user(page, buffer, count) ) {
		return -EFAULT;
	}
	if( count )
		page[count-1]='\0';
	else
		page[count]='\0';

    if(!strncmp(p, "info", sizeof("info") - 1)) {
        type = _TFFS_INFO;
    } else if(!strncmp(p, "index", sizeof("index") - 1)) {
        type = _TFFS_REINDEX;
    } else if(!strncmp(p, "cleanup", sizeof("cleanup") - 1)) {
        type = _TFFS_CLEANUP;
    } else if(!strncmp(p, "clear_id", sizeof("clear_id") - 1)) {
        type = _TFFS_CLEAR_ID;
    } else if(!strncmp(p, "werkseinstellung", sizeof("werkseinstellung") - 1)) {
        type = _TFFS_WERKSEINSTELLUNG;
    }
    if(type != -1) {
        if(tffs_lock()) {
            printk("tffs_write_proc: lock() failed\n");
            return -EINVAL;
        }
        Handle = TFFS_Open(0, tffs.anzahl, 0);
        switch(type) {
            case _TFFS_INFO:
                status = TFFS_Info(Handle, &tffs_info_value);
                printk("/proc/tffs: info request: %s\n", status ? "failed" : "success");
                break;
            case _TFFS_REINDEX:
                status = TFFS_Create_Index(Handle);
                printk("/proc/tffs: index request: %s\n", status ? "failed" : "success");
                break;
            case _TFFS_CLEANUP:
                status = TFFS_Cleanup(Handle);
                printk("/proc/tffs: cleanup request: %s\n", status ? "failed" : "success");
                break;
            case _TFFS_WERKSEINSTELLUNG:
                status = TFFS_Werkseinstellungen(Handle);
                printk("/proc/tffs: werkseinstellungen request: %s\n", status ? "failed" : "success");
                break;
            case _TFFS_CLEAR_ID:
                while(*p && *p != ' ' && *p != '\t') p++;
                while(*p && ( *p == ' ' || *p == '\t')) p++;
                type = tffs_atoi(p);
                if(p) {
                    status = TFFS_Clear(Handle, type);
                    printk("/proc/tffs: clear id 0x%x request: %s\n", type, status ? "failed" : "success");
                } else {
                    printk("/proc/tffs: clear id request: invallid id 0x%x\n", type);
                }
                break;
        }
        TFFS_Close(Handle);
        tffs_unlock();
    }
    return status == 0 ? count : -EFAULT;
}
#endif /*--- #ifdef CONFIG_PROC_FS ---*/



EXPORT_NO_SYMBOLS;

/*-----------------------------------------------------------------------------------------------*\
\*-----------------------------------------------------------------------------------------------*/
