/*-----------------------------------------------------------------------------------------------*\
\*-----------------------------------------------------------------------------------------------*/
#include <linux/config.h>
#ifdef CONFIG_SMP
#define __SMP__
#endif /*--- #ifdef CONFIG_SMP ---*/

#include <linux/module.h>
#include <linux/slab.h>
#include <asm/fcntl.h>
#include <linux/devfs_fs_kernel.h>
#include <linux/mtd/mtd.h>
#include <linux/tffs.h>
#include <linux/errno.h>
#include <linux/vmalloc.h>

/*-----------------------------------------------------------------------------------------------*\
\*-----------------------------------------------------------------------------------------------*/
/*--- #define TFFS_DEBUG ---*/

/*--- static unsigned int test_count; ---*/

/*-----------------------------------------------------------------------------------------------*\
\*-----------------------------------------------------------------------------------------------*/
struct _TFFS {
    unsigned int mtd_size;
    unsigned int mtd_start;
    unsigned int minor;
    struct _TFFS_Entry **Index;
#if !defined(CONFIG_TFFS_EXCLUSIVE)
    unsigned char *read_buffer;
    unsigned int read_buffer_length;
    unsigned char *write_buffer;
    unsigned int write_buffer_length;
#endif /*--- #if !defined(CONFIG_TFFS_EXCLUSIVE) ---*/
};
static unsigned int TFFS_mtd_number = (unsigned int)-1;
static unsigned int TFFS_mtd_number2 = (unsigned int)-1;

#define MAX_PAR 8

static struct _TFFS_Entry **TFFS_Global_Index;
static unsigned int TFFS_Global_Link;
struct mtd_info *TFFS_mtd;
unsigned int current_mtd = 0;
unsigned int avail_mtd[2];
unsigned char *TFFS_Cleanup_Buffer = NULL;

/*-----------------------------------------------------------------------------------------------*\
\*-----------------------------------------------------------------------------------------------*/
#if defined(CONFIG_TFFS_DOUBLE_BUFFER)
extern unsigned int TFFS_Init(unsigned int mtd_number, unsigned int mtd_number2)
#else /*--- #if defined(CONFIG_TFFS_DOUBLE_BUFFER) ---*/
extern unsigned int TFFS_Init(unsigned int mtd_number)
#endif /*--- #else ---*/ /*--- #if defined(CONFIG_TFFS_DOUBLE_BUFFER) ---*/
    {
#if defined(TFFS_DEBUG)
    printk(KERN_INFO "TFFS_Init(mtd%u)\n", mtd_number);
#endif /*--- #if defined(TFFS_DEBUG) ---*/
    TFFS_mtd_number = mtd_number;

	TFFS_mtd = get_mtd_device(NULL, TFFS_mtd_number);
    if(TFFS_mtd == 0) {
        printk(KERN_ERR "TFFS_Init: can't get mtd%u\n", TFFS_mtd_number);
        return (unsigned int)-ENXIO;
    }
    /*------------------------------------------------------------------------------------------*\
     * prfen welcher buffer der "richtige" ist
    \*------------------------------------------------------------------------------------------*/
#if defined(CONFIG_TFFS_DOUBLE_BUFFER)
    TFFS_mtd_number2 = mtd_number2;
    if(TFFS_mtd) {
        int ret, retlen;
        unsigned int segment[2] = { 0, 0 };
        union _tffs_segment_entry u;
        ret = MTD_READ(TFFS_mtd, 0, sizeof(union _tffs_segment_entry), &retlen, u.Buffer);
        if(u.Entry.ID == FLASH_FS_ID_SEGMENT) {
            segment[0] = TFFS_GET_SEGMENT_VALUE(&u);
#if defined(TFFS_DEBUG)
            printk(KERN_INFO "double_buffer(0): segment value %u\n", segment[0]);
        } else {
            printk(KERN_INFO "double_buffer(0): no SEGMENT VALUE\n");
#endif /*--- #if defined(TFFS_DEBUG) ---*/
        }

        put_mtd_device(TFFS_mtd);
        TFFS_mtd = get_mtd_device(NULL, TFFS_mtd_number2);
        if(TFFS_mtd == 0) {
            printk(KERN_ERR "TFFS_Init: can't get mtd%u\n", TFFS_mtd_number2);
            return (unsigned int)-ENXIO;
        }
        current_mtd = TFFS_mtd_number2;

        ret = MTD_READ(TFFS_mtd, 0, sizeof(union _tffs_segment_entry), &retlen, u.Buffer);
        if(u.Entry.ID == FLASH_FS_ID_SEGMENT) {
            segment[1] = TFFS_GET_SEGMENT_VALUE(&u);
#if defined(TFFS_DEBUG)
            printk(KERN_INFO "double_buffer(1): segment value %u\n", segment[1]);
        } else {
            printk(KERN_INFO "double_buffer(1): no SEGMENT VALUE\n");
#endif /*--- #if defined(TFFS_DEBUG) ---*/
        }

        if(segment[0] > segment[1]) {
            put_mtd_device(TFFS_mtd);
            TFFS_mtd = get_mtd_device(NULL, TFFS_mtd_number);
            current_mtd = TFFS_mtd_number;
        }
        if(segment[0] == 0 && segment[1] == 0) {
            panic("TFFS: no vallid filesystem");
        }
        avail_mtd[0] = TFFS_mtd_number;
        avail_mtd[1] = TFFS_mtd_number2;
#if defined(TFFS_DEBUG)
        printk(KERN_INFO "double_buffer: use segment %u (avail: %u + %u)\n", current_mtd, avail_mtd[0], avail_mtd[1]);
#endif /*--- #if defined(TFFS_DEBUG) ---*/
    }
#endif /*--- #if defined(CONFIG_TFFS_DOUBLE_BUFFER) ---*/

#if defined(TFFS_DEBUG)
    printk(KERN_INFO "mtd%u size=0x%x erasesize=0x%x\n", TFFS_mtd_number, TFFS_mtd->size, TFFS_mtd->erasesize);
#endif /*--- #if defined(TFFS_DEBUG) ---*/
    return 0;
}

/*-----------------------------------------------------------------------------------------------*\
\*-----------------------------------------------------------------------------------------------*/
void TFFS_Deinit(void) {
#if defined(TFFS_DEBUG)
    printk(KERN_INFO "TFFS_Deinit()\n");
#endif /*--- #if defined(TFFS_DEBUG) ---*/
	if(TFFS_mtd)
	    put_mtd_device(TFFS_mtd);
    TFFS_mtd_number = (unsigned int)-1;
	TFFS_mtd        = NULL;
}

/*-----------------------------------------------------------------------------------------------*\
\*-----------------------------------------------------------------------------------------------*/
void *TFFS_Open(unsigned int par_nr, unsigned int anzahl, unsigned int minor) {
    struct _TFFS *TFFS;
#if defined(TFFS_DEBUG)
    printk(KERN_INFO "TFFS_Open(%u)\n", par_nr);
#endif /*--- #if defined(TFFS_DEBUG) ---*/
    if(TFFS_mtd_number < 1 || TFFS_mtd_number > 6) {
        printk(KERN_ERR "TFFS_Open(%u) failed mtd_name number not correkt (is: mtd%u)\n", par_nr, TFFS_mtd_number);
        return 0;
    }

    TFFS = kmalloc(sizeof(struct _TFFS), GFP_KERNEL);
    if(TFFS == NULL) {
        printk(KERN_ERR "TFFS_Open: malloc(%u) failed\n", sizeof(struct _TFFS));
        return 0;
    }
    memset(TFFS, 0, sizeof(struct _TFFS));

    TFFS->mtd_size        = TFFS_mtd->size;
    TFFS->mtd_start       = 0;
    TFFS->minor           = minor;

#if defined(TFFS_DEBUG)
    printk(KERN_INFO "TFFS_Open: mtd open success (erasesize 0x%x size 0x%x flags 0x%x)\n",
        TFFS_mtd->erasesize, TFFS_mtd->size, TFFS_mtd->flags);
#endif /*--- #if defined(TFFS_DEBUG) ---*/

    if(TFFS_Global_Index == NULL) {
        TFFS_Global_Index = kmalloc(sizeof(struct _TFFS_Entry *) * FLASH_FS_ID_LAST, GFP_KERNEL);
        if(TFFS_Global_Index == NULL) {
            kfree(TFFS);
            printk(KERN_ERR "TFFS_Open: malloc(%u) failed\n", sizeof(struct _TFFS_Entry *) * FLASH_FS_ID_LAST);
            return 0;
        }
        TFFS->Index = TFFS_Global_Index;
        TFFS_Create_Index((void *)TFFS);
    } else {
        TFFS->Index = TFFS_Global_Index;
    }
    TFFS_Global_Link++;

#if defined(TFFS_DEBUG)
    printk(KERN_INFO "TFFS_Open: success (erasesize 0x%x start 0x%x blocksize 0x%x, anzahl %u)\n",
        TFFS_mtd->erasesize, TFFS->mtd_start, TFFS->mtd_size, anzahl);
    printk(KERN_INFO "TFFS_Open: handle: 0x%x\n", (int)TFFS);
#endif /*--- #if defined(TFFS_DEBUG) ---*/

    return TFFS;
}

/*-----------------------------------------------------------------------------------------------*\
\*-----------------------------------------------------------------------------------------------*/
unsigned int TFFS_Minor(void *handle) {
    struct _TFFS *TFFS = (struct _TFFS *)handle;
    return TFFS->minor;
}

/*-----------------------------------------------------------------------------------------------*\
 * wenn *read_buffer_length nicht NULL wird *read_buffer_length als 'int' value fuer malloc
 * verwendet. Ist *read_buffer_length == NULL wird der read buffer nur abgefragt.
\*-----------------------------------------------------------------------------------------------*/
#if !defined(CONFIG_TFFS_EXCLUSIVE)
unsigned char *TFFS_Read_Buffer(void *handle, unsigned int **read_buffer_length) {
    struct _TFFS *TFFS = (struct _TFFS *)handle;
    if(*read_buffer_length) {
        TFFS->read_buffer = kmalloc((unsigned int)*read_buffer_length, GFP_KERNEL);
#if defined(TFFS_DEBUG)
        printk(KERN_INFO "TFFS_Read_Buffer: malloc(%u) %s\n", (unsigned int)*read_buffer_length, TFFS->read_buffer ? "success" : "failed");
#endif /*--- #if defined(TFFS_DEBUG) ---*/
        TFFS->read_buffer_length = 0;
    }
    *read_buffer_length = &(TFFS->read_buffer_length);
    return TFFS->read_buffer;
}
#endif /*--- #if !defined(CONFIG_TFFS_EXCLUSIVE) ---*/

/*-----------------------------------------------------------------------------------------------*\
 * wenn *write_buffer_length nicht NULL wird *write_buffer_length als 'int' value fuer malloc
 * verwendet. Ist *write_buffer_length == NULL wird der write buffer nur abgefragt.
\*-----------------------------------------------------------------------------------------------*/
#if !defined(CONFIG_TFFS_EXCLUSIVE)
unsigned char *TFFS_Write_Buffer(void *handle, unsigned int **write_buffer_length) {
    struct _TFFS *TFFS = (struct _TFFS *)handle;
    if(*write_buffer_length) {
        TFFS->write_buffer = kmalloc((unsigned int)*write_buffer_length, GFP_KERNEL);
#if defined(TFFS_DEBUG)
        printk(KERN_INFO "TFFS_Write_Buffer: malloc(%u) %s\n", (unsigned int)*write_buffer_length, TFFS->write_buffer ? "success" : "failed");
#endif /*--- #if defined(TFFS_DEBUG) ---*/
        TFFS->write_buffer_length = 0;
    }
    *write_buffer_length = &(TFFS->write_buffer_length);
    return TFFS->write_buffer;
}
#endif /*--- #if !defined(CONFIG_TFFS_EXCLUSIVE) ---*/

/*-----------------------------------------------------------------------------------------------*\
\*-----------------------------------------------------------------------------------------------*/
void TFFS_Close(void *handle) {
    struct _TFFS *TFFS = (struct _TFFS *)handle;

#if defined(TFFS_DEBUG)
    printk(KERN_INFO "TFFS_Close(0x%x):", (int)handle);
#endif /*--- #if defined(TFFS_DEBUG) ---*/

	if( TFFS_mtd->sync ) 
		TFFS_mtd->sync(TFFS_mtd);

    TFFS_Global_Link--;
    if(TFFS_Global_Link == 0) {
        kfree(TFFS_Global_Index);
        TFFS_Global_Index = NULL;
    }
#if !defined(CONFIG_TFFS_EXCLUSIVE)
    if(TFFS->write_buffer)
        kfree(TFFS->write_buffer);
    if(TFFS->read_buffer)
        kfree(TFFS->read_buffer);
#endif /*--- #if !defined(CONFIG_TFFS_EXCLUSIVE) ---*/

    kfree(TFFS);
#if defined(TFFS_DEBUG)
    printk(KERN_INFO " success\n");
#endif /*--- #if defined(TFFS_DEBUG) ---*/
}

/*-----------------------------------------------------------------------------------------------*\
\*-----------------------------------------------------------------------------------------------*/
static void TFFS_Format_Callback(struct erase_info *instr) {
#if defined(TFFS_DEBUG)
	printk("TFFS: mtd_erase_callback\n");
#endif /*--- #if defined(TFFS_DEBUG) ---*/
	wake_up((wait_queue_head_t *)instr->priv);
    return;
}

/*-----------------------------------------------------------------------------------------------*\
\*-----------------------------------------------------------------------------------------------*/
unsigned int TFFS_Format(void *handle) {
    struct _TFFS *TFFS = (struct _TFFS *)handle;
    struct erase_info *erase;
	int ret=0;
	DECLARE_WAITQUEUE(wait,current);
	wait_queue_head_t wait_q;

	init_waitqueue_head(&wait_q);

    erase = (struct erase_info *)kmalloc(sizeof(struct erase_info), GFP_KERNEL);
    if(erase == NULL) {
        printk(KERN_ERR "TFFS_Format: malloc(%u) failed\n", sizeof(struct erase_info));
        return (unsigned int)-ENOMEM;
    }

#if defined(TFFS_DEBUG)
    printk(KERN_INFO "TFFS_Format: malloc(%u) success\n", sizeof(struct erase_info));
#endif /*--- #if defined(TFFS_DEBUG) ---*/
	memset(erase, 0, sizeof(struct erase_info));

    erase->mtd      = TFFS_mtd;
    erase->addr     = TFFS->mtd_start;
    erase->len      = TFFS->mtd_size;
    erase->callback = TFFS_Format_Callback;
    erase->priv     = (u_long)&wait_q;
    erase->next     = 0;

#if defined(TFFS_DEBUG)
    printk(KERN_INFO "TFFS_Format(handle=0x%x): erase: addr %x len %x\n",
            (unsigned int)handle, erase->addr, erase->len);
#endif /*--- #if defined(TFFS_DEBUG) ---*/

    ret = TFFS_mtd->erase(TFFS_mtd, erase);

	if( !ret ) {
		set_current_state(TASK_UNINTERRUPTIBLE);
		add_wait_queue( &wait_q, &wait);
		if( erase->state != MTD_ERASE_DONE && erase->state != MTD_ERASE_FAILED) {
			schedule();
		}
		
		remove_wait_queue(&wait_q, &wait);
		set_current_state(TASK_RUNNING);
		
		ret = (erase->state == MTD_ERASE_FAILED) ? -EIO : 0;
        if(ret)
		    printk(KERN_ERR "Failed (callback) to erase mtd, region [0x%x, 0x%x]\n", erase->addr,erase->len);
	} else {
		printk(KERN_ERR "Failed to erase mtd, region [0x%x, 0x%x]\n", erase->addr,erase->len);
        ret = -EIO;
	}
	kfree(erase);
    return (unsigned int)ret;
}

/*-----------------------------------------------------------------------------------------------*\
\*-----------------------------------------------------------------------------------------------*/
unsigned int TFFS_Clear(void *handle, enum _tffs_id id) {
    return TFFS_Write(handle, id, NULL, 0, 0);
}

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
unsigned int TFFS_Werkseinstellungen(void *handle) {
    enum _tffs_id id;
    struct _TFFS *TFFS = (struct _TFFS *)handle;
    unsigned int count = 0;
#if defined(TFFS_DEBUG)
    printk(KERN_INFO "TFFS_Werkseinstellungen(0x%x)\n", (int)handle);
#endif /*--- #if defined(TFFS_DEBUG) ---*/
    for(id = FLASH_FS_ID_TICFG ; id <= FLASH_FS_ID_FIRMWARE_CONFIG_LAST ; id++) {
        if(TFFS->Index[id] != (struct _TFFS_Entry *)FLASH_FS_ID_FREE) {
            unsigned int error = TFFS_Write(handle, id, NULL, 0, 0);
            if(error) {
#if defined(TFFS_DEBUG)
                printk(KERN_INFO "TFFS_Werkseinstellungen(0x%x): clear id 0x%x failed (%u cleared)\n", (int)handle, (int)id, count);
#endif /*--- #if defined(TFFS_DEBUG) ---*/
                return error;
            }
            count++;
        }
    }
#if defined(TFFS_DEBUG)
    printk(KERN_INFO "TFFS_Werkseinstellungen(0x%x): success (%u cleared)\n", (int)handle, count);
#endif /*--- #if defined(TFFS_DEBUG) ---*/
    return 0;
}

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
enum _tffs_memcmp {
    tffs_memcmp_equal,
    tffs_memcmp_writeable,
    tffs_memcmp_clear_required
};

/*-----------------------------------------------------------------------------------------------*\
\*-----------------------------------------------------------------------------------------------*/
static enum _tffs_memcmp TFFS_Memcmp(unsigned char *FlashMemory, unsigned char *NewMemory, unsigned int Length) {
    unsigned int ret;
    unsigned short *_Flash     = (unsigned short *)FlashMemory;
    unsigned short *_neuerWert = (unsigned short *)NewMemory;
    
    ret = memcmp(FlashMemory, NewMemory, Length);
    if(ret == 0)
        return tffs_memcmp_equal;

#if 1
    Length = (Length + 1) >> 1;
    while(Length--) {
        if((((*_Flash++) | ((~(*_neuerWert++)) & 0xFFFF)) != 0xFFFF)) {
            return tffs_memcmp_clear_required;
        }
    }
    return tffs_memcmp_writeable;
#else
    return tffs_memcmp_clear_required;
#endif
}

/*-----------------------------------------------------------------------------------------------*\
\*-----------------------------------------------------------------------------------------------*/
unsigned int TFFS_Write(void *handle, enum _tffs_id Id, unsigned char *write_buffer, unsigned int write_length, unsigned int level) {
    int ret, retlen;
    struct _TFFS_Entry *E, Entry;
    struct _TFFS_Entry *E_ToClear = NULL, Entry_ToClear;
    struct _TFFS *TFFS = (struct _TFFS *)handle;
    unsigned int Len = 0;

    E = (struct _TFFS_Entry *)TFFS->mtd_start;
    while((unsigned int)E + sizeof(struct _TFFS_Entry) + write_length < TFFS->mtd_start + TFFS->mtd_size) {
        ret = MTD_READ(TFFS_mtd, (unsigned long)E, sizeof(struct _TFFS_Entry), &retlen, (unsigned char *)&Entry);
        if(ret) return ret;

        /*---------------------------------------------------------------------------------------*\
         * ID gefunden
        \*---------------------------------------------------------------------------------------*/
        if(Entry.ID == (unsigned short)Id && Id < FLASH_FS_DROPABLE_DATA) { /*---  Alter Eintrag gefunden ----*/
#if defined(TFFS_DEBUG)
            printk(KERN_INFO "alter Eintrag mit ID=%x gefunden ", Id);
#endif /*--- #if defined(TFFS_DEBUG) ---*/

            /*-----------------------------------------------------------------------------------*\
             * prfen ob eintrag identisch oder berschreibbar
            \*-----------------------------------------------------------------------------------*/
            if(Entry.Length == write_length) {
                unsigned char *buff = kmalloc(Entry.Length, GFP_KERNEL);
                if(!buff) {
                    printk(KERN_ERR "TFFS_Write: malloc(%u) failed\n", Entry.Length);
                    return (unsigned int)-ENOMEM;
                }
#if defined(TFFS_DEBUG)
                printk(KERN_INFO "TFFS_Write: malloc(%u) success\n", Entry.Length);
#endif /*--- #if defined(TFFS_DEBUG) ---*/
                ret = MTD_READ(TFFS_mtd, (unsigned long)E + sizeof(struct _TFFS_Entry), Entry.Length, &retlen, buff);
                if(ret) return ret;

                switch(TFFS_Memcmp(buff, write_buffer, Entry.Length)) {
                    case tffs_memcmp_equal:
#if defined(TFFS_DEBUG)
                        printk(KERN_INFO "alter Eintrag mit neuem identisch\n");
#endif /*--- #if defined(TFFS_DEBUG) ---*/
                        kfree(buff);
                        return 0;

#if 1
                    case tffs_memcmp_writeable:
#if defined(TFFS_DEBUG)
                        printk(KERN_INFO "alter Eintrag durch neuen berschreibbar\n");
#endif /*--- #if defined(TFFS_DEBUG) ---*/
                        ret = MTD_WRITE(TFFS_mtd, (unsigned long)E + sizeof(struct _TFFS_Entry), Entry.Length, &retlen, write_buffer);
                        kfree(buff);
                        return ret;
#endif

                    default:
                    case tffs_memcmp_clear_required:
                        break;
                }
                kfree(buff);
            }

            /*-----------------------------------------------------------------------------------*\
             * gefundenen Eintrag auf FLASH_FS_ID_SKIP setzen
            \*-----------------------------------------------------------------------------------*/
            Entry.ID     = FLASH_FS_ID_SKIP;
            if(write_buffer == NULL) {
                ret = MTD_WRITE(TFFS_mtd, (unsigned long)E, sizeof(struct _TFFS_Entry), &retlen, (unsigned char *)&Entry);
                if(ret || (retlen != sizeof(struct _TFFS_Entry))) {
                    printk(KERN_ERR "TFFS: write TFFS_Entry (id=SKIP) failed\n");
                    return ret || (unsigned int)-EIO;
                }

#if defined(TFFS_DEBUG)
                printk(KERN_INFO "geloescht\n");
#endif /*--- #if defined(TFFS_DEBUG) ---*/

                TFFS->Index[Id] = (struct _TFFS_Entry *)FLASH_FS_ID_FREE;
            } else {
                E_ToClear = E;
                Entry_ToClear = Entry;
            }
            
            /*-----------------------------------------------------------------------------------*\
             * sollte es keine Daten fr neuen satz geben dann sind wir fertig
            \*-----------------------------------------------------------------------------------*/
            if(write_buffer == NULL) {
#if defined(TFFS_DEBUG)
                printk(KERN_INFO "(1) nur loeschen kein neuer Eintrag\n");
#endif /*--- #if defined(TFFS_DEBUG) ---*/
                return 0;
            }
        }
        /*---------------------------------------------------------------------------------------*\
         * freie ID gefunden
        \*---------------------------------------------------------------------------------------*/
        if(Entry.ID == FLASH_FS_ID_FREE) { /*---  Freier Eintrag gefunden ----*/
#if defined(TFFS_DEBUG)
            printk(KERN_INFO "freier Eintrag gefunden\n");
#endif /*--- #if defined(TFFS_DEBUG) ---*/
            if(write_buffer == NULL) {
#if defined(TFFS_DEBUG)
                printk(KERN_INFO "(2) nur loeschen kein neuer Eintrag\n");
#endif /*--- #if defined(TFFS_DEBUG) ---*/
                return 0;
            }
            Entry.ID     = (unsigned short)Id;
            Entry.Length = (unsigned short)write_length;
            TFFS->Index[Entry.ID] = E;

/*--- if(test_count++ & 0x20) { ---*/
    /*--- printk("simmulated write error (before header): \n"); ---*/
    /*--- return (unsigned int)-EIO; ---*/
/*--- } ---*/
            ret = MTD_WRITE(TFFS_mtd, (unsigned long)E, sizeof(struct _TFFS_Entry), &retlen, (unsigned char *)&Entry);
            if(ret || (retlen != sizeof(struct _TFFS_Entry))) {
                printk(KERN_ERR "TFFS: write TFFS_Entry (id=0x%x) failed\n", Id);
                return ret ? ret : (unsigned int)-EIO;
            }
#if defined(TFFS_DEBUG)
            printk(KERN_INFO "header mit id %x geschrieben\n", Id);
#endif /*--- #if defined(TFFS_DEBUG) ---*/
/*--- if(test_count++ & 0x20) { ---*/
    /*--- printk("simmulated write error (before data): \n"); ---*/
    /*--- return (unsigned int)-EIO; ---*/
/*--- } ---*/

            ret = MTD_WRITE(TFFS_mtd, (unsigned long)E + sizeof(struct _TFFS_Entry), Entry.Length, &retlen, write_buffer);
            if(ret || (retlen != Entry.Length)) {
                printk(KERN_ERR "TFFS: write data (id=0x%x) failed\n", Id);
                return ret ? ret : (unsigned int)-EIO;
            }

/*--- if(test_count++ & 0x20) { ---*/
    /*--- printk("simmulated write error (after data): \n"); ---*/
    /*--- test_count = 0; ---*/
    /*--- return (unsigned int)-EIO; ---*/
/*--- } ---*/
#if defined(TFFS_DEBUG)
            printk(KERN_INFO "%d bytes daten geschrieben\n", write_length);
#endif /*--- #if defined(TFFS_DEBUG) ---*/

            if(E_ToClear != NULL) {
                ret = MTD_WRITE(TFFS_mtd, (unsigned long)E_ToClear, sizeof(struct _TFFS_Entry), &retlen, (unsigned char *)&Entry_ToClear);
                if(ret || (retlen != sizeof(struct _TFFS_Entry))) {
                    printk(KERN_ERR "TFFS: write TFFS_Entry (id=SKIP) failed\n");
                    return ret ? ret : (unsigned int)-EIO;
                }

#if defined(TFFS_DEBUG)
                printk(KERN_INFO "geloescht (postum) ID=0x%x Len=%u\n", Entry_ToClear.ID, Entry_ToClear.Length);
#endif /*--- #if defined(TFFS_DEBUG) ---*/
            }

            /*-------------------------------------------------------------------------*\
                Prfen ob Filesystem mehr als 75% Voll ist 
            \*-------------------------------------------------------------------------*/
            {
                unsigned int Fuell = (unsigned int)E + Entry.Length; /*--- aktueller Fuellstand ---*/
                Fuell -= TFFS->mtd_start;
                Fuell *= 100;
                Fuell /= TFFS->mtd_size;

#if defined(TFFS_DEBUG)
                printk(KERN_INFO "fuellstand %u%%\n", Fuell);
#endif /*--- #if defined(TFFS_DEBUG) ---*/
                if(Fuell > 75) {
                    tffs_send_event(TFFS_EVENT_CLEANUP);
                }
            }
            return 0;
        }
        
        Len = (Entry.Length + 3) & ~0x03;
        E = (struct _TFFS_Entry *)((unsigned int)E + sizeof(struct _TFFS_Entry) + Len);
    }
    if(level == 0) {
        if(TFFS_Cleanup(handle) == 0) 
            return TFFS_Write(handle, Id, write_buffer, write_length, 1);
    }
    return (unsigned int)-EIO;
}

/*-----------------------------------------------------------------------------------------------*\
\*-----------------------------------------------------------------------------------------------*/
unsigned int TFFS_Read(void *handle, enum _tffs_id Id, unsigned char *read_buffer, unsigned int *read_length) {
    unsigned int retlen, ret;
    struct _TFFS_Entry *E, Entry;
    struct _TFFS *TFFS = (struct _TFFS *)handle;
    unsigned int Len;

#if defined(TFFS_DEBUG)
    printk(KERN_INFO "TFFS_Read(handle=%x): id = 0x%x, max_length = %u\n", (unsigned int)handle, (unsigned int)Id, *read_length);
#endif /*--- #if defined(TFFS_DEBUG) ---*/

    E = TFFS->Index[Id];
    if(E != (struct _TFFS_Entry *)FLASH_FS_ID_FREE) {
#if defined(TFFS_DEBUG)
        printk(KERN_INFO "Eintrag gefunden\n");
#endif /*--- #if defined(TFFS_DEBUG) ---*/
		ret = MTD_READ(TFFS_mtd, (unsigned long)E, sizeof(struct _TFFS_Entry), &retlen, (unsigned char *)&Entry);
        if(ret)
            return ret;
        Len = min((unsigned int)Entry.Length, *read_length);
		ret = MTD_READ(TFFS_mtd, (unsigned long)E + sizeof(struct _TFFS_Entry), Len, &retlen, read_buffer);
        if(ret)
            return ret;
        *read_length = retlen;
#if defined(TFFS_DEBUG)
        printk(KERN_INFO "daten kopiert\n");
#endif /*--- #if defined(TFFS_DEBUG) ---*/
        return 0;
    }
#if defined(TFFS_DEBUG)
    printk(KERN_INFO "Eintrag %i nicht gefunden\n", Id);
#endif /*--- #if defined(TFFS_DEBUG) ---*/
    return (unsigned int)-ENOENT;
}

/*-----------------------------------------------------------------------------------------------*\
\*-----------------------------------------------------------------------------------------------*/
unsigned int TFFS_Create_Index(void *handle) {
    unsigned int retlen, ret;
    struct _TFFS_Entry *E, Entry;
    unsigned int Len;
    unsigned int i;
    struct _TFFS *TFFS = (struct _TFFS *)handle;

    for(i = 0 ; i < FLASH_FS_ID_LAST ; i++) {
        TFFS->Index[i] = (struct _TFFS_Entry *)FLASH_FS_ID_FREE;
    }
    E = (struct _TFFS_Entry *)TFFS->mtd_start;
#if defined(TFFS_DEBUG)
    printk(KERN_INFO "TFFS_Create_Index(handle=%x): ", (unsigned int)handle);
#endif /*--- #if defined(TFFS_DEBUG) ---*/

    while((unsigned int)E + sizeof(struct _TFFS_Entry) < TFFS->mtd_start + TFFS->mtd_size) {
        ret = MTD_READ(TFFS_mtd, (unsigned long)E, sizeof(struct _TFFS_Entry), &retlen, (unsigned char *)&Entry);
        if(ret) return ret;

        if(Entry.ID >= (unsigned short)FLASH_FS_ID_LAST) { /*---  Eintrag gefunden ----*/
#if defined(TFFS_DEBUG)
            printk(KERN_INFO " [end found]\n");
#endif /*--- #if defined(TFFS_DEBUG) ---*/
            return 0;
        }
#if defined(TFFS_DEBUG)
        printk(KERN_INFO " [<0x%x> %u bytes]\n", Entry.ID, Entry.Length);
#endif /*--- #if defined(TFFS_DEBUG) ---*/

        /*--- doppelter Eintrag gefunden, diesen Eintrag lschen ---*/
        if(TFFS->Index[Entry.ID] != (struct _TFFS_Entry *)FLASH_FS_ID_FREE) { 
            if(Entry.ID != FLASH_FS_ID_SKIP) {
                printk(KERN_ERR " [<0x%x> %u bytes, cleared]\n", Entry.ID, Entry.Length);
                Entry.ID = FLASH_FS_ID_SKIP;
                ret = MTD_WRITE(TFFS_mtd, (unsigned long)E, sizeof(struct _TFFS_Entry), &retlen, (unsigned char *)&Entry);
                if(ret) return ret;
            }
        } else {
            TFFS->Index[Entry.ID] = E;
        }
        Len = (Entry.Length + 3) & ~0x03;
        E = (struct _TFFS_Entry *)((unsigned int)E + sizeof(struct _TFFS_Entry) + Len);
    }
#if defined(TFFS_DEBUG)
    printk(KERN_INFO " [filesystem full]\n");
#endif /*--- #if defined(TFFS_DEBUG) ---*/
    return 0;
}

/*-----------------------------------------------------------------------------------------------*\
\*-----------------------------------------------------------------------------------------------*/
unsigned int TFFS_Cleanup(void *handle) {
    unsigned int retlen, ret;
    struct _TFFS_Entry *E, *pEntry;
    unsigned int Len, SkipCount = 0;
    struct _TFFS *TFFS = (struct _TFFS *)handle;
    unsigned char *P;
    unsigned int current_number = 0;

#if defined(TFFS_DEBUG)
    printk(KERN_INFO "TFFS_Cleanup(handle=%x): ", (unsigned int)handle);
#endif /*--- #if defined(TFFS_DEBUG) ---*/

    if(TFFS_Cleanup_Buffer == NULL) {
        /*--- TFFS_Cleanup_Buffer = TFFS_MALLOC(TFFS->mtd_size, GFP_KERNEL); ---*/
        TFFS_Cleanup_Buffer = vmalloc(TFFS->mtd_size);
        if(TFFS_Cleanup_Buffer == NULL) {
            printk(KERN_ERR "TFFS_Cleanup: malloc(%u) failed\n", TFFS->mtd_size);
            return (unsigned int)-ENOMEM;
        }
#if defined(TFFS_DEBUG)
        printk(KERN_INFO "TFFS_Cleanup: malloc(%u) success\n", TFFS->mtd_size);
#endif /*--- #if defined(TFFS_DEBUG) ---*/
    }

    E = (struct _TFFS_Entry *)TFFS->mtd_start;
    P = TFFS_Cleanup_Buffer;
    while((unsigned int)E + sizeof(struct _TFFS_Entry) < TFFS->mtd_start + TFFS->mtd_size) {

        pEntry = (struct _TFFS_Entry *)P;
        ret = MTD_READ(TFFS_mtd, (unsigned long)E, sizeof(struct _TFFS_Entry), &retlen, (unsigned char *)pEntry);
        if(ret) {
            return ret;
        }

        if(pEntry->ID >= (unsigned short)FLASH_FS_ID_LAST) { /*---  Eintrag gefunden ----*/
#if defined(TFFS_DEBUG)
            printk(KERN_INFO " [end found]\n");
#endif /*--- #if defined(TFFS_DEBUG) ---*/
            break;
        }

        Len = (pEntry->Length + 3) & ~0x03;

        if(pEntry->ID != FLASH_FS_ID_SKIP) {

            P += sizeof(struct _TFFS_Entry *);
            ret = MTD_READ(TFFS_mtd, (unsigned long)E + sizeof(struct _TFFS_Entry), pEntry->Length, &retlen, (unsigned char *)P);
            if(ret) {
                return ret;
            }
#if defined(TFFS_DEBUG)
            printk(KERN_INFO " [<0x%x> %u bytes]\n", pEntry->ID, pEntry->Length);
#endif /*--- #if defined(TFFS_DEBUG) ---*/
            P += Len;
        } else {
            SkipCount++;
#if defined(TFFS_DEBUG)
            printk(KERN_INFO " [SKIP %u bytes]\n", pEntry->Length);
#endif /*--- #if defined(TFFS_DEBUG) ---*/
        }

        E = (struct _TFFS_Entry *)((unsigned int)E + sizeof(struct _TFFS_Entry) + Len);
    }
#if defined(TFFS_DEBUG)
    printk(KERN_INFO " [%u data records skiped]\n", SkipCount);
#endif /*--- #if defined(TFFS_DEBUG) ---*/
    if(SkipCount == 0) {
#if defined(TFFS_DEBUG)
        printk(KERN_INFO "TFFS_Cleanup: no IDs skiped, leave it as it is\n");
#endif /*--- #if defined(TFFS_DEBUG) ---*/
        return 0;
    }
    /*------------------------------------------------------------------------------------------*\
    \*------------------------------------------------------------------------------------------*/
#if defined(CONFIG_TFFS_DOUBLE_BUFFER)
    {
        union _tffs_segment_entry *pu;
        unsigned int other = current_mtd == avail_mtd[0] ? avail_mtd[1] : avail_mtd[0];
#if defined(TFFS_DEBUG)
        printk(KERN_INFO " [double buffer: close old mtd, open mtd%u]\n", other);
#endif /*--- #if defined(TFFS_DEBUG) ---*/
        put_mtd_device(TFFS_mtd);
        TFFS_mtd = get_mtd_device(NULL, other);
        if(TFFS_mtd == 0) {
            panic("TFFS_Cleanup: can't get mtd%u\n", other);
            return (unsigned int)-ENXIO;
        }

        pu = (union _tffs_segment_entry *)TFFS_Cleanup_Buffer;
        if(pu->Entry.ID != FLASH_FS_ID_SEGMENT) {
            panic("TFFS_Cleanup: flash segment %u file invallid\n", current_mtd);
        }
        current_mtd = other;
        current_number = TFFS_GET_SEGMENT_VALUE(pu);
#if defined(TFFS_DEBUG)
        printk(KERN_INFO " [double buffer: read current number %u]\n", current_number);
#endif /*--- #if defined(TFFS_DEBUG) ---*/
        TFFS_SET_SEGMENT_VALUE(pu, 0);
    }
#endif /*--- #if defined(CONFIG_TFFS_DOUBLE_BUFFER) ---*/

    /*------------------------------------------------------------------------------------------*\
    \*------------------------------------------------------------------------------------------*/
#if defined(TFFS_DEBUG)
    printk(KERN_INFO " [erase filesystem]\n");
#endif /*--- #if defined(TFFS_DEBUG) ---*/
    ret = TFFS_Format(handle);
    if(ret) {
#if defined(TFFS_DEBUG)
        printk(KERN_ERR "TFFS_Cleanup: format failed\n");
#endif /*--- #if defined(TFFS_DEBUG) ---*/
        return ret;
    }

#if defined(TFFS_DEBUG)
    printk(KERN_INFO " [write filesystem]\n");
#endif /*--- #if defined(TFFS_DEBUG) ---*/
    ret = MTD_WRITE(TFFS_mtd, TFFS->mtd_start, (unsigned int)P - (unsigned int)TFFS_Cleanup_Buffer, &retlen, (unsigned char *)TFFS_Cleanup_Buffer);
    if(ret) return ret;

    /*------------------------------------------------------------------------------------------*\
    \*------------------------------------------------------------------------------------------*/
#if defined(CONFIG_TFFS_DOUBLE_BUFFER)
    {
        union _tffs_segment_entry u;
        u.Entry.ID = FLASH_FS_ID_SEGMENT;
        u.Entry.Length = sizeof(unsigned int);
        current_number++;
#if defined(TFFS_DEBUG)
        printk(KERN_INFO " [double buffer: write current number %u]\n", current_number);
#endif /*--- #if defined(TFFS_DEBUG) ---*/
        TFFS_SET_SEGMENT_VALUE(&u, current_number);
        ret = MTD_WRITE(TFFS_mtd, TFFS->mtd_start, sizeof(union _tffs_segment_entry), &retlen, (unsigned char *)&u);
        if(ret) {
            return ret;
        }
    }
#endif /*--- #if defined(CONFIG_TFFS_DOUBLE_BUFFER) ---*/
    /*------------------------------------------------------------------------------------------*\
    \*------------------------------------------------------------------------------------------*/

#if defined(TFFS_DEBUG)
    printk(KERN_INFO " [recreate index]\n");
#endif /*--- #if defined(TFFS_DEBUG) ---*/
    TFFS_Create_Index(handle);

#if defined(TFFS_DEBUG)
    printk(KERN_INFO "TFFS_Cleanup: success\n");
#endif /*--- #if defined(TFFS_DEBUG) ---*/
    return 0;
}


/*-----------------------------------------------------------------------------------------------*\
\*-----------------------------------------------------------------------------------------------*/
unsigned int TFFS_Info(void *handle, unsigned int *Fill) {
    unsigned int retlen, ret;
    struct _TFFS_Entry *E, Entry;
    unsigned int Count, Max = 0;
    struct _TFFS *TFFS = (struct _TFFS *)handle;

#if defined(TFFS_DEBUG)
    printk(KERN_INFO "TFFS_Info(handle=%x): ", (unsigned int)handle);
#endif /*--- #if defined(TFFS_DEBUG) ---*/

    for(Count = 0, E = 0 ; Count < FLASH_FS_ID_LAST ; Count++) {
        E = TFFS->Index[Count];
        if(E != (struct _TFFS_Entry *)FLASH_FS_ID_FREE) {
#if defined(TFFS_DEBUG)
            printk(KERN_INFO "E = 0x%x", (unsigned int)E);
#endif /*--- #if defined(TFFS_DEBUG) ---*/
            if(Max < (unsigned int)E) {
                Max = (unsigned int)E;
            }
        }
    }
    if(!Max) {
        *Fill = 0;
        return 0;
    }

#if defined(TFFS_DEBUG)
    printk(KERN_INFO "Header des hoechsten Eintrags lesen (0x%x)\n", Max);
#endif /*--- #if defined(TFFS_DEBUG) ---*/
    ret = MTD_READ(TFFS_mtd, (unsigned long)Max, sizeof(struct _TFFS_Entry), &retlen, (unsigned char *)&Entry);
    if(ret) return ret;

#if defined(TFFS_DEBUG)
    printk(KERN_INFO " [<0x%x> %u bytes]\n", Entry.ID, Entry.Length);
#endif /*--- #if defined(TFFS_DEBUG) ---*/
    Max += Entry.Length;
    Max -= TFFS->mtd_start;

#if defined(TFFS_DEBUG)
    printk(KERN_INFO "mtd_offset %x mtd_size %x Max %x\n", 
        TFFS->mtd_start, TFFS->mtd_size, Max);
#endif /*--- #if defined(TFFS_DEBUG) ---*/

    *Fill = (Max * 100) / TFFS->mtd_size;
#if defined(TFFS_DEBUG)
    printk(KERN_INFO "TFFS_Info: Fill=%u%% success\n", *Fill);
#endif /*--- #if defined(TFFS_DEBUG) ---*/
    return 0;
}


EXPORT_SYMBOL(TFFS_Open);
EXPORT_SYMBOL(TFFS_Close);
EXPORT_SYMBOL(TFFS_Read);
EXPORT_SYMBOL(TFFS_Write);
