
#define WDT_OK                   0
#define WDT_REQUEST              1
#define WDT_ALREADY_REGISTERED  -1
#define WDT_UNKNOWN_HANDLE      -2
#define WDT_INVALID_TIMEOUT     -3
#define WDT_RESOURCE_ERROR      -4

/*-------------------------------------------------------------------------------------*\
 * Kndigt das Starten einer Applikation an.
 *
 * name: null-terminierter Name des Prozesses (mu eindeutig sein)
 * timeout: Intervall in Sekunden, innerhalb dessen sich die Applikation mit demselben Namen
 *          registrieren mu.
 *
 * returns: WDT_OK, wenn erfolgreich
\*-------------------------------------------------------------------------------------*/
int watchdog_announce(char* name, int timeout);

/*-------------------------------------------------------------------------------------*\
 * Registriert einen Proze beim Watchdog
 * Der Default Timeout betrgt 20s, nach 10s wird die Applikation benachrichtigt (optional)
 *
 * name: null-terminierter Name des Prozesses (mu eindeutig sein)
 *
 * returns:    > 0: handle fr registrierten Proze, sonst Fehler
\*-------------------------------------------------------------------------------------*/
int watchdog_register(char* name);

/*-------------------------------------------------------------------------------------*\
 * Liefert den Filehandle zum Watchdog-Handle zurck
 *
 * handle: Beim Register erhaltener Handle
 * returns: file handle
\*-------------------------------------------------------------------------------------*/
int watchdog_get_fd(int handle);

/*-------------------------------------------------------------------------------------*\
 * Deregistriert einen Proze beim Watchdog
 *
 * handle: Beim Register erhaltener Handle
 *
 * returns: WDT_OK, wenn erfolgreich, sonst Fehler
 *
 * NOTE: Ein Proze mu sich per watchdog_release() abmelden, bevor er sich beendet,
 *       anderenfalls wird ein Reboot ausgelst! Dies gilt auch fr den Fall, da der
 *       Proze von einem anderen Proze (gewollt) beendet wird. Hierfr ist ein
 *       watchdog_release() im Signal-Handler vorzusehen.
\*-------------------------------------------------------------------------------------*/
int watchdog_release(int handle);

/*-------------------------------------------------------------------------------------*\
 * ndert den default Timeout fr den registrierten Proze
 *
 * handle:  Beim Register erhaltener Handle
 * timeout: neuer Timeout in Sekunden ab sofort (Maximum 40s)
 *
 * returns: WDT_OK, sonst Fehler
 *
 * NOTE: watchdog_settimeout() dient gleichzeitig als Trigger fr den Watchdog
\*-------------------------------------------------------------------------------------*/
int watchdog_settimeout(int handle, int timeout);

/*-------------------------------------------------------------------------------------*\
 * Triggert den Watchdog fr den angegebenen Proze
 *
 * handle:  Beim Register erhaltener Handle
 *
 * returns: WDT_OK, sonst Fehler
\*-------------------------------------------------------------------------------------*/
int watchdog_trigger(int handle);

/*-------------------------------------------------------------------------------------*\
 * Blockiert, bis Watchdog neu getriggert werden mu
 * Nach Rckkehr aus der Funktion hat die Applikation noch die Hfte des gesetzten Timeouts
 * Zeit zum Zurcksetzen des Watchdogs.
 *
 * handle:  Beim Register erhaltener Handle
 *
 * returns: WDT_OK, sonst Fehler
\*-------------------------------------------------------------------------------------*/
int watchdog_wait(int handle);

/*-------------------------------------------------------------------------------------*\
 * Teilt mit, ob der Watchdog neu getriggert werden mu
 * Die Funktion blockiert nicht.
 * 
 * handle:  Beim Register erhaltener Handle
 *
 * returns: WDT_OK vor Ablauf der Hlfte des gesetzten Timeouts, ungleich WDT_OK danach
\*-------------------------------------------------------------------------------------*/
int watchdog_poll(int handle);

